@@ -1,25 +1,13 @@
-//ff-c++-LIBRARY-dep: cxx11 hpddm [umfpack|mumps parmetis ptscotch scotch scalapack] [mkl|blas] mpi pthread mpifc fc
+//ff-c++-LIBRARY-dep: cxx11 hpddm [mumps parmetis ptscotch scotch scalapack|umfpack] [mkl|blas] mpi pthread mpifc fc
 //ff-c++-cpp-dep:
 
 #define HPDDM_SCHWARZ 1
 #define HPDDM_FETI    0
 #define HPDDM_BDD     0
+
 #include "common.hpp"
 
 namespace Schwarz {
-double getOpt(string* const& ss) {
-    return HPDDM::Option::get()->val(*ss);
-}
-bool isSetOpt(string* const& ss) {
-    return HPDDM::Option::get()->set(*ss);
-}
-template<class Type, class K>
-bool clearRecycling(Type* const& Op, const long& mu) {
-    HPDDM::Recycling<K>::get(mu)->~Recycling();
-    return false;
-}
-
-
 template<class Type, class K>
 class initDDM_Op : public E_F0mps {
     public:
@@ -27,7 +15,7 @@ class initDDM_Op : public E_F0mps {
         Expression Mat;
         Expression o;
         Expression R;
-        static const int n_name_param = 3;
+        static const int n_name_param = 4;
         static basicAC_F0::name_and_type name_param[];
         Expression nargs[n_name_param];
         initDDM_Op(const basicAC_F0& args, Expression param1, Expression param2, Expression param3, Expression param4) : A(param1), Mat(param2), o(param3), R(param4) {
@@ -40,7 +28,8 @@ template<class Type, class K>
 basicAC_F0::name_and_type initDDM_Op<Type, K>::name_param[] = {
     {"communicator", &typeid(pcommworld)},
     {"scaling", &typeid(KN<HPDDM::underlying_type<K>>*)},
-    {"deflation", &typeid(FEbaseArrayKn<K>*)}
+    {"deflation", &typeid(FEbaseArrayKn<K>*)},
+    {"prefix", &typeid(string*)}
 };
 template<class Type, class K>
 class initDDM : public OneOperator {
@@ -53,23 +42,15 @@ class initDDM : public OneOperator {
 };
 template<class Type, class K>
 AnyType initDDM_Op<Type, K>::operator()(Stack stack) const {
-    const char** argv = new const char*[pkarg->n];
-    for(int i = 0; i < pkarg->n; ++i)
-        argv[i] = (*((*pkarg)[i].getap()))->data();
-    HPDDM::Option::get()->parse(pkarg->n, argv, mpirank == 0);
-    delete [] argv;
     Type* ptA = GetAny<Type*>((*A)(stack));
-    MatriceMorse<K>* mA = static_cast<MatriceMorse<K>*>(&(*GetAny<Matrice_Creuse<K>*>((*Mat)(stack))->A));
+    Matrice_Creuse<K>* pA = GetAny<Matrice_Creuse<K>*>((*Mat)(stack));
+    MatriceMorse<K>* mA = pA->A ? static_cast<MatriceMorse<K>*>(&(*pA->A)) : nullptr;
     KN<long>* ptO = GetAny<KN<long>*>((*o)(stack));
     KN<KN<long>>* ptR = GetAny<KN<KN<long>>*>((*R)(stack));
-    MPI_Comm* comm = nargs[0] ? (MPI_Comm*)GetAny<pcommworld>((*nargs[0])(stack)) : 0;
-    if(ptO && mA) {
-        HPDDM::MatrixCSR<K>* dA = new HPDDM::MatrixCSR<K>(mA->n, mA->m, mA->nbcoef, mA->a, mA->lg, mA->cl, mA->symetrique);
-        ptA->HPDDM::template Subdomain<K>::initialize(dA, STL<long>(*ptO), *ptR, comm);
-    }
+    if(ptO)
+        ptA->HPDDM::template Subdomain<K>::initialize(mA ? new HPDDM::MatrixCSR<K>(mA->n, mA->m, mA->nbcoef, mA->a, mA->lg, mA->cl, mA->symetrique) : 0, STL<long>(*ptO), *ptR, nargs[0] ? (MPI_Comm*)GetAny<pcommworld>((*nargs[0])(stack)) : 0);
     FEbaseArrayKn<K>* deflation = nargs[2] ? GetAny<FEbaseArrayKn<K>*>((*nargs[2])(stack)) : 0;
-    K** const& v = ptA->getVectors();
-    if(deflation && deflation->N > 0 && !v) {
+    if(deflation && deflation->N > 0 && !ptA->getVectors()) {
         K** ev = new K*[deflation->N];
         *ev = new K[deflation->N * deflation->get(0)->n];
         for(int i = 0; i < deflation->N; ++i) {
@@ -79,11 +60,12 @@ AnyType initDDM_Op<Type, K>::operator()(Stack stack) const {
         ptA->setVectors(ev);
         ptA->Type::super::initialize(deflation->N);
     }
-    KN<HPDDM::underlying_type<K>>* ptD = nargs[1] ? GetAny<KN<HPDDM::underlying_type<K>>*>((*nargs[1])(stack)) : 0;
-    if(ptD)
-        ptA->initialize(*ptD);
+    if(nargs[1])
+        ptA->initialize(*GetAny<KN<HPDDM::underlying_type<K>>*>((*nargs[1])(stack)));
     else
         std::cerr << "Something is really wrong here !" << std::endl;
+    if(nargs[3])
+        ptA->setPrefix(*(GetAny<string*>((*nargs[3])(stack))));
     return ptA;
 }
 
@@ -127,33 +109,35 @@ AnyType attachCoarseOperator_Op<Type, K>::operator()(Stack stack) const {
     MatriceMorse<K>* mA = nargs[0] ? static_cast<MatriceMorse<K>*>(&(*GetAny<Matrice_Creuse<K>*>((*nargs[0])(stack))->A)) : 0;
     Pair<K>* pair = nargs[5] ? GetAny<Pair<K>*>((*nargs[5])(stack)) : 0;
     HPDDM::Option& opt = *HPDDM::Option::get();
-    unsigned short nu = opt.val<unsigned short>("geneo_nu", 20);
-    HPDDM::underlying_type<K> threshold = opt.val("geneo_threshold", 0.0);
     KN<double>* timing = nargs[4] ? GetAny<KN<double>*>((*nargs[4])(stack)) : 0;
     std::pair<MPI_Request, const K*>* ret = nullptr;
-    double t;
     if(mA) {
-        nu = std::max(nu, static_cast<unsigned short>(1));
         long nbSolver = 0;
         std::vector<const HPDDM::MatrixCSR<K>*> vecAIJ;
         if(mA) {
             HPDDM::MatrixCSR<K> dA(mA->n, mA->m, mA->nbcoef, mA->a, mA->lg, mA->cl, mA->symetrique);
             MatriceMorse<K>* mB = nargs[1] ? static_cast<MatriceMorse<K>*>(&(*GetAny<Matrice_Creuse<K>*>((*nargs[1])(stack))->A)) : nullptr;
             MatriceMorse<K>* mP = nargs[2] && opt.any_of("schwarz_method", { 1, 2, 4 }) ? static_cast<MatriceMorse<K>*>(&(*GetAny<Matrice_Creuse<K>*>((*nargs[2])(stack))->A)) : nullptr;
-            t = MPI_Wtime();
             if(dA._n == dA._m) {
+                if(timing) { // tic
+                    timing->resize(timing->n + 1);
+                    (*timing)[timing->n - 1] = MPI_Wtime();
+                }
                 const HPDDM::MatrixCSR<K>* const dP = mP ? new HPDDM::MatrixCSR<K>(mP->n, mP->m, mP->nbcoef, mP->a, mP->lg, mP->cl, mP->symetrique) : nullptr;
                 if(mB) {
                     HPDDM::MatrixCSR<K> dB(mB->n, mB->m, mB->nbcoef, mB->a, mB->lg, mB->cl, mB->symetrique);
-                    ptA->template solveGEVP<EIGENSOLVER>(&dA, nu, threshold, &dB, dP);
+                    ptA->template solveGEVP<EIGENSOLVER>(&dA, &dB, dP);
                 }
                 else
-                    ptA->template solveGEVP<EIGENSOLVER>(&dA, nu, threshold, nullptr, dP);
+                    ptA->template solveGEVP<EIGENSOLVER>(&dA, nullptr, dP);
                 mA->nbcoef = dA._nnz;
                 mA->a = dA._a;
                 mA->lg = dA._ia;
                 mA->cl = dA._ja;
                 delete dP;
+                if(timing) { // toc
+                    (*timing)[timing->n - 1] = MPI_Wtime() - (*timing)[timing->n - 1];
+                }
             }
             else {
                 vecAIJ.emplace_back(&dA);
@@ -171,7 +155,7 @@ AnyType attachCoarseOperator_Op<Type, K>::operator()(Stack stack) const {
                 int dof = ptA->getDof();
                 HPDDM::Eigensolver<K> solver(dof);
                 const HPDDM::MatrixCSR<K>& first = *vecAIJ.front();
-                nu = std::min(nu, static_cast<unsigned short>(first._m));
+                unsigned short nu = std::min(opt.template val<unsigned short>("geneo_nu", 20), static_cast<unsigned short>(first._m));
                 K** ev = new K*[nu];
                 *ev = new K[nu * dof];
                 for(int i = 0; i < nu; ++i)
@@ -247,16 +231,16 @@ AnyType attachCoarseOperator_Op<Type, K>::operator()(Stack stack) const {
                 ptA->Type::super::initialize(nu);
                 if(nbSolver == 100)
                     std::for_each(vecAIJ.begin(), vecAIJ.end(), std::default_delete<const HPDDM::MatrixCSR<K>>());
+                opt["geneo_nu"] = nu;
             }
             else
                 ptA->Type::super::initialize(0);
         }
-        if(timing)
-            (*timing)[3] = MPI_Wtime() - t;
-        opt["geneo_nu"] = nu;
-        MPI_Barrier(MPI_COMM_WORLD);
-        if(timing)
-            t = MPI_Wtime();
+        MPI_Barrier(comm);
+        if(timing) { // tic
+            timing->resize(timing->n + 1);
+            (*timing)[timing->n - 1] = MPI_Wtime();
+        }
         if(ptA->exclusion(comm)) {
             if(pair)
                 pair->p = ptA->template buildTwo<1>(comm);
@@ -269,13 +253,18 @@ AnyType attachCoarseOperator_Op<Type, K>::operator()(Stack stack) const {
             else
                 ret = ptA->template buildTwo<0>(comm);
         }
-        if(timing)
-            (*timing)[4] = MPI_Wtime() - t;
+        if(timing) { // toc
+            (*timing)[timing->n - 1] = MPI_Wtime() - (*timing)[timing->n - 1];
+        }
     }
     else {
-        MPI_Barrier(MPI_COMM_WORLD);
-        if(!threshold)
+        MPI_Barrier(comm);
+        if(!ptA->getVectors())
             ret = ptA->template buildTwo<2>(comm);
+        else if(ptA->exclusion(comm))
+            ret = ptA->template buildTwo<1>(comm);
+        else
+            ret = ptA->template buildTwo<0>(comm);
     }
     if(ret)
         delete ret;
@@ -288,7 +277,7 @@ class solveDDM_Op : public E_F0mps {
         Expression A;
         Expression x;
         Expression rhs;
-        static const int n_name_param = 8;
+        static const int n_name_param = 9;
         static basicAC_F0::name_and_type name_param[];
         Expression nargs[n_name_param];
         solveDDM_Op(const basicAC_F0& args, Expression param1, Expression param2, Expression param3) : A(param1), x(param2), rhs(param3) {
@@ -306,7 +295,8 @@ basicAC_F0::name_and_type solveDDM_Op<Type, K>::name_param[] = {
     {"excluded", &typeid(bool)},
     {"ret", &typeid(Pair<K>*)},
     {"O", &typeid(Matrice_Creuse<K>*)},
-    {"solver", &typeid(long)}
+    {"solver", &typeid(long)},
+    {"communicator", &typeid(pcommworld)}
 };
 template<class Type, class K>
 class solveDDM : public OneOperator {
@@ -325,26 +315,27 @@ AnyType solveDDM_Op<Type, K>::operator()(Stack stack) const {
     if(ptX->n != ptRHS->n || ptRHS->n < ptA->getDof())
         return 0L;
     HPDDM::Option& opt = *HPDDM::Option::get();
+    const std::string& prefix = ptA->prefix();
     HPDDM::underlying_type<K> eps = nargs[0] ? GetAny<HPDDM::underlying_type<K>>((*nargs[0])(stack)) : -1.0;
     if(nargs[0])
         std::cerr << "Please do not use the legacy option \"-eps\", set instead \"-hpddm_tol\", cf. \"-hpddm_help\"" << std::endl;
     if(std::abs(eps + 1.0) > 1.0e-6)
-        opt["tol"] = eps;
+        opt[prefix + "tol"] = eps;
     int dim = nargs[1] ? GetAny<long>((*nargs[1])(stack)) : -1;
     if(nargs[1])
         std::cerr << "Please do not use the legacy option \"-dim\", set instead \"-hpddm_gmres_restart\", cf. \"-hpddm_help\"" << std::endl;
     if(dim != -1)
-        opt["gmres_restart"] = dim;
+        opt[prefix + "gmres_restart"] = dim;
     int iter = nargs[2] ? GetAny<long>((*nargs[2])(stack)) : -1;
     if(nargs[2])
         std::cerr << "Please do not use the legacy option \"-iter\", set instead \"-hpddm_max_it\", cf. \"-hpddm_help\"" << std::endl;
     if(iter != -1)
-        opt["max_it"] = iter;
+        opt[prefix + "max_it"] = iter;
     if(nargs[7])
         std::cerr << "Please do not use the legacy option \"-solver\", set instead \"-hpddm_schwarz_method\" and \"-hpddm_schwarz_coarse_correction\", cf. \"-hpddm_help\"" << std::endl;
     KN<double>* timing = nargs[3] ? GetAny<KN<double>*>((*nargs[3])(stack)) : 0;
     Pair<K>* pair = nargs[5] ? GetAny<Pair<K>*>((*nargs[5])(stack)) : 0;
-    if(opt.set("schwarz_coarse_correction") && pair)
+    if(opt.set(prefix + "schwarz_coarse_correction") && pair)
         if(pair->p) {
             int flag;
             MPI_Test(&(pair->p->first), &flag, MPI_STATUS_IGNORE);
@@ -359,8 +350,11 @@ AnyType solveDDM_Op<Type, K>::operator()(Stack stack) const {
         std::for_each(A->_ia, A->_ia + A->_n + 1, [](int& i) { ++i; });
         std::for_each(A->_ja, A->_ja + A->_nnz, [](int& i) { ++i; });
     }
-    double timer = MPI_Wtime();
-    if(mpisize > 1 && (mA && opt.any_of("schwarz_method", { 1, 2, 4 }))) {
+    if(timing) { // tic
+        timing->resize(timing->n + 1);
+        (*timing)[timing->n - 1] = MPI_Wtime();
+    }
+    if(mpisize > 1 && (mA && opt.any_of(prefix + "schwarz_method", { 1, 2, 4 }))) {
         HPDDM::MatrixCSR<K> dA(mA->n, mA->m, mA->nbcoef, mA->a, mA->lg, mA->cl, mA->symetrique);
         ptA->callNumfact(&dA);
     }
@@ -372,54 +366,47 @@ AnyType solveDDM_Op<Type, K>::operator()(Stack stack) const {
             ptA->template callNumfact<'F'>();
 #endif
     }
-    if(timing) (*timing)[1] = MPI_Wtime() - timer;
+    if(timing) { // toc
+        (*timing)[timing->n - 1] = MPI_Wtime() - (*timing)[timing->n - 1];
+    }
     if(HPDDM::Wrapper<K>::I == 'F' && SUBDOMAIN<K>::_numbering == 'C' && mu > 1) {
         std::for_each(A->_ia, A->_ia + A->_n + 1, [](int& i) { ++i; });
         std::for_each(A->_ja, A->_ja + A->_nnz, [](int& i) { ++i; });
     }
-    bool excluded = nargs[4] ? GetAny<bool>((*nargs[4])(stack)) : false;
+    bool excluded = nargs[4] && GetAny<bool>((*nargs[4])(stack));
     if(excluded)
-        opt["master_exclude"];
+        opt[prefix + "master_exclude"];
     if(pair)
         if(pair->p) {
-            if(timing)
-                timer = MPI_Wtime();
             MPI_Wait(&(pair->p->first), MPI_STATUS_IGNORE);
-            if(timing)
-                (*timing)[timing->n - 1] = MPI_Wtime() - timer;
             delete [] pair->p->second;
             pair->destroy();
             pair = nullptr;
-            timer = MPI_Wtime();
         }
-    MPI_Barrier(MPI_COMM_WORLD);
-    if(opt.val<unsigned short>("reuse_preconditioner") <= 1 && !excluded && pair && pair->p && timing && mpisize > 1)
-        (*timing)[timing->n - 1] += MPI_Wtime() - timer;
+    MPI_Comm comm = nargs[8] ? *(MPI_Comm*)GetAny<pcommworld>((*nargs[8])(stack)) : MPI_COMM_WORLD;
     int rank;
     MPI_Comm_rank(ptA->getCommunicator(), &rank);
-    if(rank != 0 || excluded)
+    if(rank != mpirank || rank != 0) {
         opt.remove("verbosity");
-    timer = MPI_Wtime();
-    if(!excluded)
-        HPDDM::IterativeMethod::solve(*ptA, (K*)*ptRHS, (K*)*ptX, mu, MPI_COMM_WORLD);
-    else
-        HPDDM::IterativeMethod::solve<true>(*ptA, (K*)nullptr, (K*)nullptr, mu, MPI_COMM_WORLD);
-    timer = MPI_Wtime() - timer;
-    if(!excluded) {
-        if(rank == 0)
-            std::cout << scientific << " --- system solved (in " << timer << ")" << std::endl;
-        HPDDM::underlying_type<K>* storage = new HPDDM::underlying_type<K>[2 * mu];
-        ptA->computeError(*ptX, *ptRHS, storage, mu);
-        if(rank == 0) {
-            std::cout << scientific << " --- error = " << storage[1] << " / " << storage[0];
-            for(unsigned short nu = 1; nu < mu; ++nu)
-                std::cout << " (rhs #" << nu << ")" << std::endl << scientific << "             " << storage[2 * nu + 1] << " / " << storage[2 * nu];
-            if(mu > 1)
-                std::cout << " (rhs #" << mu << ")";
-            std::cout << std::endl;
-        }
-        delete [] storage;
+        if(prefix.size() > 0)
+            opt.remove(prefix + "verbosity");
     }
+    MPI_Barrier(comm);
+    double timer = MPI_Wtime();
+    if(timing) { // tic
+        timing->resize(timing->n + 1);
+        (*timing)[timing->n - 1] = timer;
+    }
+    if(!excluded)
+        HPDDM::IterativeMethod::solve(*ptA, (K*)*ptRHS, (K*)*ptX, mu, comm);
+    else
+        HPDDM::IterativeMethod::solve<true>(*ptA, (K*)nullptr, (K*)nullptr, mu, comm);
+    timer = MPI_Wtime() - timer;
+    if(timing) { // toc
+        (*timing)[timing->n - 1] = timer;
+    }
+    if(!excluded && verbosity > 0 && rank == 0)
+        std::cout << std::scientific << " --- system solved (in " << timer << ")" << std::endl;
     if(HPDDM::Wrapper<K>::I == 'F' && mu > 1) {
         std::for_each(A->_ja, A->_ja + A->_nnz, [](int& i) { --i; });
         std::for_each(A->_ia, A->_ia + A->_n + 1, [](int& i) { --i; });
@@ -488,60 +475,11 @@ class set : public OneOperator {
 };
 template<class Type, class K>
 AnyType set_Op<Type, K>::operator()(Stack stack) const {
-    std::string params = nargs[0] ? *(GetAny<string*>((*nargs[0])(stack))) : "";
-    HPDDM::Option::get()->parse(params);
+    if(nargs[0])
+        HPDDM::Option::get()->parse(*(GetAny<string*>((*nargs[0])(stack))));
     return 0L;
 }
 
-template<class K>
-class distributedDot_Op : public E_F0mps {
-    public:
-        Expression A;
-        Expression in;
-        Expression out;
-        static const int n_name_param = 1;
-        static basicAC_F0::name_and_type name_param[];
-        Expression nargs[n_name_param];
-        distributedDot_Op<K>(const basicAC_F0& args, Expression param1, Expression param2, Expression param3) : A(param1), in(param2), out(param3) {
-            args.SetNameParam(n_name_param, name_param, nargs);
-        }
-
-        AnyType operator()(Stack stack) const;
-};
-template<class K>
-basicAC_F0::name_and_type distributedDot_Op<K>::name_param[] = {
-    {"communicator", &typeid(pcommworld)}
-};
-template<class K>
-class distributedDot : public OneOperator {
-    public:
-        distributedDot() : OneOperator(atype<K>(), atype<KN<double>*>(), atype<KN<K>*>(), atype<KN<K>*>()) { }
-
-        E_F0* code(const basicAC_F0& args) const {
-            return new distributedDot_Op<K>(args, t[0]->CastTo(args[0]), t[1]->CastTo(args[1]), t[2]->CastTo(args[2]));
-        }
-};
-template<class K, typename std::enable_if<!std::is_same<K, double>::value>::type* = nullptr>
-inline K prod(K u, double d, K v) {
-    return std::conj(u) * d * v;
-}
-template<class K, typename std::enable_if<std::is_same<K, double>::value>::type* = nullptr>
-inline K prod(K u, double d, K v) {
-    return u * d * v;
-}
-template<class K>
-AnyType distributedDot_Op<K>::operator()(Stack stack) const {
-    KN<double>* pA = GetAny<KN<double>*>((*A)(stack));
-    KN<K>* pin = GetAny<KN<K>*>((*in)(stack));
-    KN<K>* pout = GetAny<KN<K>*>((*out)(stack));
-    MPI_Comm* comm = nargs[0] ? (MPI_Comm*)GetAny<pcommworld>((*nargs[0])(stack)) : 0;
-    K dot = K();
-    for(int i = 0; i < pin->n; ++i)
-        dot += prod(pin->operator[](i), pA->operator[](i), pout->operator[](i));
-    MPI_Allreduce(MPI_IN_PLACE, &dot, 1, HPDDM::Wrapper<K>::mpi_type(), MPI_SUM, comm ? *((MPI_Comm*)comm) : MPI_COMM_WORLD);
-    return SetAny<K>(dot);
-}
-
 template<class Type, class K>
 class distributedMV_Op : public E_F0mps {
     public:
@@ -577,56 +515,28 @@ AnyType distributedMV_Op<Type, K>::operator()(Stack stack) const {
     unsigned short mu = pin->n / pA->getDof();
     MatriceMorse<K>* mA = static_cast<MatriceMorse<K>*>(&(*GetAny<Matrice_Creuse<K>*>((*Mat)(stack))->A));
     HPDDM::MatrixCSR<K> dA(mA->n, mA->m, mA->nbcoef, mA->a, mA->lg, mA->cl, mA->symetrique);
-    bool alloc = pA->setBuffer(mu);
+    bool allocate = pA->setBuffer();
     pA->GMV((K*)*pin, (K*)*pout, mu, &dA);
-    pA->clearBuffer(alloc);
+    pA->clearBuffer(allocate);
     return 0L;
 }
 
-template<class Type, class K>
-class scaledExchange_Op : public E_F0mps {
-    public:
-        Expression A;
-        Expression in;
-        static const int n_name_param = 0;
-        static basicAC_F0::name_and_type name_param[];
-        scaledExchange_Op<Type, K>(const basicAC_F0& args, Expression param1, Expression param2) : A(param1), in(param2) {
-            args.SetNameParam(n_name_param, name_param, nullptr);
-        }
-
-        AnyType operator()(Stack stack) const;
-};
-template<class Type, class K>
-basicAC_F0::name_and_type scaledExchange_Op<Type, K>::name_param[] = { };
-template<class Type, class K>
-class scaledExchange : public OneOperator {
-    public:
-        scaledExchange() : OneOperator(atype<long>(), atype<Type*>(), atype<KN<K>*>()) { }
-
-        E_F0* code(const basicAC_F0& args) const {
-            return new scaledExchange_Op<Type, K>(args, t[0]->CastTo(args[0]), t[1]->CastTo(args[1]));
-        }
-};
-template<class Type, class K>
-AnyType scaledExchange_Op<Type, K>::operator()(Stack stack) const {
-    Type* pA = GetAny<Type*>((*A)(stack));
-    KN<K>* pin = GetAny<KN<K>*>((*in)(stack));
-    unsigned short mu = pin->n / pA->getDof();
-    pA->Type::template scaledExchange<true>((K*)*pin, mu);
-    return 0L;
-}
-
-template<class T, class U, class K>
+template<class T, class U, class K, char N>
 class ProdSchwarz {
     public:
         const T t;
         const U u;
         ProdSchwarz(T v, U w) : t(v), u(w) {}
-        void prod(U x) const { bool alloc = t->setBuffer(1); t->GMV(*(this->u), *x); t->clearBuffer(alloc); };
-        static U mv(U Ax, ProdSchwarz<T, U, K> A) {
+        void prod(U x) const { bool allocate = t->setBuffer(); t->GMV(*(this->u), *x); t->clearBuffer(allocate); };
+        static U mv(U Ax, ProdSchwarz<T, U, K, N> A) {
+            *Ax = K();
             A.prod(Ax);
             return Ax;
         }
+        static U init(U Ax, ProdSchwarz<T, U, K, N> A) {
+            Ax->init(A.u->n);
+            return mv(Ax, A);
+        }
 };
 
 template<class T, class U, class K>
@@ -638,7 +548,6 @@ class InvSchwarz {
         void solve(U out) const {
             if(out->n != u->n || u->n < (*t).getDof())
                 return;
-            HPDDM::Option& opt = *HPDDM::Option::get();
             unsigned short mu = u->n / (*t).getDof();
             MPI_Allreduce(MPI_IN_PLACE, &mu, 1, MPI_UNSIGNED_SHORT, MPI_MAX, (*t).getCommunicator());
             const HPDDM::MatrixCSR<K>* A = (*t).getMatrix();
@@ -659,7 +568,7 @@ class InvSchwarz {
                 std::for_each(A->_ja, A->_ja + A->_nnz, [](int& i) { ++i; });
             }
             if(mpirank != 0)
-                opt.remove("verbosity");
+                HPDDM::Option::get()->remove((*t).prefix("verbosity"));
             HPDDM::IterativeMethod::solve(*t, (K*)*u, (K*)*out, mu, MPI_COMM_WORLD);
         }
         static U init(U Ax, InvSchwarz<T, U, K> A) {
@@ -668,6 +577,117 @@ class InvSchwarz {
         }
 };
 
+template<class R>
+class IterativeMethod : public OneOperator {
+    public:
+        typedef KN<R> Kn;
+        typedef KN_<R> Kn_;
+        class MatF_O : VirtualMatrice<R> {
+            public:
+                Stack stack;
+                mutable Kn x;
+                C_F0 c_x;
+                Expression mat1, mat;
+                typedef typename VirtualMatrice<R>::plusAx plusAx;
+                MatF_O(int n, Stack stk, const OneOperator* op) :
+                    VirtualMatrice<R>(n), stack(stk), x(n), c_x(CPValue(x)),
+                    mat1(op ? op->code(basicAC_F0_wa(c_x)) : 0),
+                    mat(op ? CastTo<Kn_>(C_F0(mat1, (aType)*op)) : 0) { }
+                ~MatF_O() {
+                    if(mat1 != mat)
+                        delete mat;
+                    delete mat1;
+                    Expression zzz = c_x;
+                    delete zzz;
+                }
+                void addMatMul(const Kn_& xx, Kn_& Ax) const {
+                    ffassert(xx.N() == Ax.N());
+                    x = xx;
+                    Ax += GetAny<Kn_>((*mat)(stack));
+                    WhereStackOfPtr2Free(stack)->clean();
+                }
+                void mv(const R* const in, const int& n, R* const out) const {
+                    KN_<R> xx((R*)in, n);
+                    KN_<R> yy(out, n);
+                    yy = R();
+                    yy = plusAx(this, xx);
+                }
+                bool ChecknbLine(int) const { return true; }
+                bool ChecknbColumn(int) const { return true; }
+        };
+        class Operator : public HPDDM::EmptyOperator<R> {
+            public:
+                MatF_O& mat;
+                MatF_O& prec;
+                Operator(MatF_O& m, MatF_O& p) : mat(m), prec(p), HPDDM::EmptyOperator<R>(m.x.N()) { }
+                void GMV(const R* const in, R* const out, const int& mu = 1) const {
+                    mat.mv(in, HPDDM::EmptyOperator<R>::_n, out);
+                }
+                template<bool = true>
+                void apply(const R* const in, R* const out, const unsigned short& mu = 1, R* = nullptr, const unsigned short& = 0) const {
+                    if(prec.mat)
+                        prec.mv(in, HPDDM::EmptyOperator<R>::_n, out);
+                    else
+                        std::copy_n(in, HPDDM::EmptyOperator<R>::_n, out);
+                }
+        };
+        class E_LCG : public E_F0mps {
+            public:
+                static const int n_name_param = 4;
+                static basicAC_F0::name_and_type name_param[];
+                Expression nargs[n_name_param];
+                const OneOperator *A, *C;
+                Expression X, B;
+                E_LCG(const basicAC_F0& args) {
+                    args.SetNameParam(n_name_param, name_param, nargs);
+                    { const Polymorphic* op = dynamic_cast<const Polymorphic*>(args[0].LeftValue());
+                        ffassert(op);
+                        A = op->Find("(", ArrayOfaType(atype<Kn*>(), false)); }
+                    if(nargs[0]) {
+                        const Polymorphic* op = dynamic_cast<const Polymorphic*>(nargs[0]);
+                        ffassert(op);
+                        C = op->Find("(", ArrayOfaType(atype<Kn*>(), false));
+                    }
+                    else
+                        C = 0;
+                    X = to<Kn*>(args[1]);
+                    B = to<Kn*>(args[2]);
+                }
+                virtual AnyType operator()(Stack stack)  const {
+                    int ret = -1;
+                    try {
+                        Kn& x = *GetAny<Kn*>((*X)(stack));
+                        int n = x.N();
+                        MPI_Comm comm = nargs[3] ? *(MPI_Comm*)GetAny<pcommworld>((*nargs[3])(stack)) : MPI_COMM_WORLD;
+                        Kn& b = *GetAny<Kn*>((*B)(stack));
+                        MatF_O AA(n, stack, A);
+                        MatF_O PP(n, stack, C);
+                        Operator Op(AA, PP);
+                        if(nargs[1])
+                            Op.setPrefix(*(GetAny<string*>((*nargs[1])(stack))));
+                        if(nargs[2])
+                            HPDDM::Option::get()->parse(*(GetAny<string*>((*nargs[2])(stack))));
+                        ret = HPDDM::IterativeMethod::solve(Op, (R*)b, (R*)x, 1, comm);
+                    }
+                    catch(...) {
+                        throw;
+                    }
+                    return SetAny<long>(ret);
+                }
+                operator aType() const { return atype<long>(); }
+        };
+        E_F0* code(const basicAC_F0& args) const { return new E_LCG(args); }
+        IterativeMethod() : OneOperator(atype<long>(), atype<Polymorphic*>(), atype<KN<R>*>(), atype<KN<R>*>()) { }
+};
+
+template<class R>
+basicAC_F0::name_and_type IterativeMethod<R>::E_LCG::name_param[] = {
+    {"precon", &typeid(Polymorphic*)},
+    {"prefix", &typeid(string*)},
+    {"sparams", &typeid(string*)},
+    {"comm", &typeid(pcommworld)}
+};
+
 template<template<class, char> class Type, class K, char S>
 void add() {
     Dcl_Type<Type<K, S>*>(Initialize<Type<K, S>>, Delete<Type<K, S>>);
@@ -679,16 +699,17 @@ void add() {
     Global.Add("set", "(", new set<Type<K, S>, K>);
     addProd<Type<K, S>, ProdSchwarz, KN<K>, K>();
     addInv<Type<K, S>, InvSchwarz, KN<K>, K>();
-    Global.Add("dscalprod", "(", new distributedDot<K>);
     Global.Add("dmv", "(", new distributedMV<Type<K, S>, K>);
-    Global.Add("scaledExchange", "(", new scaledExchange<Type<K, S>, K>);
-    Global.Add("clearRecycling", "(", new OneOperator2_<bool, Type<K, S>*, long>(Schwarz::clearRecycling<Type<K, S>, K>));
+    Global.Add("destroyRecycling", "(", new OneOperator1_<bool, Type<K, S>*>(destroyRecycling<Type<K, S>, K>));
+    Global.Add("statistics", "(", new OneOperator1_<bool, Type<K, S>*>(statistics<Type<K, S>>));
+    Global.Add("exchange", "(", new exchangeIn<Type<K, S>, K>);
+    Global.Add("exchange", "(", new exchangeInOut<Type<K, S>, K>);
+    Global.Add("IterativeMethod","(",new IterativeMethod<K>());
 }
 }
 
 static void Init_Schwarz() {
-    Global.Add("getOption", "(", new OneOperator1_<double, string*>(Schwarz::getOpt));
-    Global.Add("isSetOption", "(", new OneOperator1_<bool, string*>(Schwarz::isSetOpt));
+    Init_Common();
 #if defined(DSUITESPARSE) || defined(DHYPRE)
     const char ds = 'G';
 #else
@@ -705,14 +726,18 @@ static void Init_Schwarz() {
     // Schwarz::add<HpSchwarz, std::complex<float>, zs>();
     // zzzfff->Add("cschwarz", atype<HpSchwarz<std::complex<float>, zs>*>());
 #endif
-    // Dcl_Type<Pair<float>*>(InitP<Pair<float>>, Destroy<Pair<float>>);
-    // zzzfff->Add("spair", atype<Pair<double>*>());
-    Dcl_Type<Pair<double>*>(InitP<Pair<double>>, Destroy<Pair<double>>);
-    zzzfff->Add("dpair", atype<Pair<double>*>());
-    // Dcl_Type<Pair<std::complex<float>>*>(InitP<Pair<std::complex<float>>>, Destroy<Pair<std::complex<float>>>);
-    // zzzfff->Add("cpair", atype<Pair<std::complex<float>>*>());
-    Dcl_Type<Pair<std::complex<double>>*>(InitP<Pair<std::complex<double>>>, Destroy<Pair<std::complex<double>>>);
-    zzzfff->Add("zpair", atype<Pair<std::complex<double>>*>());
+    aType t;
+    int r;
+    if(!zzzfff->InMotClef("dpair", t, r)) {
+        // Dcl_Type<Pair<float>*>(InitP<Pair<float>>, Destroy<Pair<float>>);
+        // zzzfff->Add("spair", atype<Pair<double>*>());
+        Dcl_Type<Pair<double>*>(InitP<Pair<double>>, Destroy<Pair<double>>);
+        zzzfff->Add("dpair", atype<Pair<double>*>());
+        // Dcl_Type<Pair<std::complex<float>>*>(InitP<Pair<std::complex<float>>>, Destroy<Pair<std::complex<float>>>);
+        // zzzfff->Add("cpair", atype<Pair<std::complex<float>>*>());
+        Dcl_Type<Pair<std::complex<double>>*>(InitP<Pair<std::complex<double>>>, Destroy<Pair<std::complex<double>>>);
+        zzzfff->Add("zpair", atype<Pair<std::complex<double>>*>());
+    }
 }
 
 LOADFUNC(Init_Schwarz)
