@@ -4,15 +4,10 @@
 #define HPDDM_SCHWARZ 0
 #define HPDDM_FETI    1
 #define HPDDM_BDD     1
+
 #include "common.hpp"
 
 namespace Substructuring {
-double getOpt(string* const& ss) {
-    return HPDDM::Option::get()->val(*ss);
-}
-bool isSetOpt(string* const& ss) {
-    return HPDDM::Option::get()->set(*ss);
-}
 class Skeleton_Op : public E_F0mps {
     public:
         Expression interface;
@@ -234,23 +229,15 @@ class initDDM : public OneOperator {
 };
 template<class Type, class K>
 AnyType initDDM_Op<Type, K>::operator()(Stack stack) const {
-    const char** argv = new const char*[pkarg->n];
-    for(int i = 0; i < pkarg->n; ++i)
-        argv[i] = (*((*pkarg)[i].getap()))->data();
-    HPDDM::Option::get()->parse(pkarg->n, argv, mpirank == 0);
-    delete [] argv;
     Type* ptA = GetAny<Type*>((*A)(stack));
-    MatriceMorse<K>* mA = static_cast<MatriceMorse<K>*>(&(*GetAny<Matrice_Creuse<K>*>((*Mat)(stack))->A));
+    Matrice_Creuse<K>* pA = GetAny<Matrice_Creuse<K>*>((*Mat)(stack));
+    MatriceMorse<K>* mA = pA->A ? static_cast<MatriceMorse<K>*>(&(*pA->A)) : nullptr;
     KN<long>* ptO = GetAny<KN<long>*>((*o)(stack));
     KN<KN<long>>* ptR = GetAny<KN<KN<long>>*>((*R)(stack));
-    MPI_Comm* comm = nargs[0] ? (MPI_Comm*)GetAny<pcommworld>((*nargs[0])(stack)) : 0;
-    if(ptO && mA) {
-        HPDDM::MatrixCSR<K>* dA = new HPDDM::MatrixCSR<K>(mA->n, mA->m, mA->nbcoef, mA->a, mA->lg, mA->cl, mA->symetrique);
-        ptA->HPDDM::template Subdomain<K>::initialize(dA, STL<long>(*ptO), *ptR, comm);
-    }
+    if(ptO)
+        ptA->HPDDM::template Subdomain<K>::initialize(mA ? new HPDDM::MatrixCSR<K>(mA->n, mA->m, mA->nbcoef, mA->a, mA->lg, mA->cl, mA->symetrique) : 0, STL<long>(*ptO), *ptR, nargs[0] ? (MPI_Comm*)GetAny<pcommworld>((*nargs[0])(stack)) : 0);
     FEbaseArrayKn<K>* deflation = nargs[1] ? GetAny<FEbaseArrayKn<K>*>((*nargs[1])(stack)) : 0;
-    K** const& v = ptA->getVectors();
-    if(deflation && deflation->N > 0 && !v) {
+    if(deflation && deflation->N > 0 && !ptA->getVectors()) {
         K** ev = new K*[deflation->N];
         *ev = new K[deflation->N * deflation->get(0)->n];
         for(int i = 0; i < deflation->N; ++i) {
@@ -320,15 +307,14 @@ AnyType attachCoarseOperator_Op<Type, K>::operator()(Stack stack) const {
                 nu = opt["geneo_nu"];
 #if defined(MUMPSSUB) || defined(PASTIXSUB) || defined(MKL_PARDISOSUB)
             t = MPI_Wtime();
-            ptA->solveGEVP(nu, threshold);
+            ptA->solveGEVP();
             if(timing)
                 (*timing)[5] = MPI_Wtime() - t;
 #else
             cout << "Please change your solver" << endl;
 #endif
         }
-        K** const ev = ptA->getVectors();
-        if(!R && !ev)
+        if(!R && !ptA->getVectors())
             cout << "Problem !" << endl;
         R->resize(0);
         MPI_Barrier(MPI_COMM_WORLD);
@@ -372,7 +358,8 @@ AnyType attachCoarseOperator_Op<Type, K>::operator()(Stack stack) const {
             if(timing)
                 (*timing)[timing->n - 1] = MPI_Wtime() - t;
             delete [] pair->p->second;
-            delete pair->p;
+            pair->destroy();
+            pair = nullptr;
         }
     return 0L;
 }
@@ -426,7 +413,7 @@ AnyType solveDDM_Op<Type, K>::operator()(Stack stack) const {
     if(iter != -1)
         opt["max_it"] = iter;
     KN<double>* timing = nargs[2] ? GetAny<KN<double>*>((*nargs[2])(stack)) : 0;
-    bool excluded = nargs[3] ? GetAny<long>((*nargs[3])(stack)) : false;
+    bool excluded = nargs[3] && GetAny<bool>((*nargs[3])(stack));
     if(excluded)
         opt["master_exclude"];
     double timer = MPI_Wtime();
@@ -446,7 +433,7 @@ AnyType solveDDM_Op<Type, K>::operator()(Stack stack) const {
     timer = MPI_Wtime();
     int rank;
     MPI_Comm_rank(ptA->getCommunicator(), &rank);
-    if(rank != 0 || excluded)
+    if(rank != mpirank || rank != 0)
         opt.remove("verbosity");
     timer = MPI_Wtime();
     if(!excluded)
@@ -454,14 +441,8 @@ AnyType solveDDM_Op<Type, K>::operator()(Stack stack) const {
     else
         HPDDM::IterativeMethod::solve<true>(*ptA, (K*)nullptr, (K*)nullptr, 1, MPI_COMM_WORLD);
     timer = MPI_Wtime() - timer;
-    if(!excluded) {
-        if(rank == 0)
-            std::cout << scientific << " --- system solved (in " << timer << ")" << std::endl;
-        HPDDM::underlying_type<K> storage[2];
-        ptA->computeError(*ptX, *ptRHS, storage);
-        if(rank == 0)
-            std::cout << scientific << " --- error = " << storage[1] << " / " << storage[0] << std::endl;
-    }
+    if(!excluded && verbosity > 0 && rank == 0)
+        std::cout << scientific << " --- system solved (in " << timer << ")" << std::endl;
     return 0L;
 }
 
@@ -595,14 +576,15 @@ void add() {
     Global.Add("nbMult", "(", new OneOperator1_<long, Type<K, S>*>(nbMult));
     Global.Add("originalNumbering", "(", new OneOperator3_<long, Type<K, S>*, KN<K>*, KN<long>*>(originalNumbering));
     addInv<Type<K, S>, InvSubstructuring, KN<K>, K>();
+    Global.Add("statistics", "(", new OneOperator1_<bool, Type<K, S>*>(statistics<Type<K, S>>));
+    Global.Add("exchange", "(", new exchangeInOut<Type<K, S>, K>);
 }
 }
 
 template<class K, char S>
 using HpFetiPrec = HpFeti<HPDDM::FetiPrcndtnr::DIRICHLET, K, S>;
 static void Init_Substructuring() {
-    Global.Add("getOption", "(", new OneOperator1_<double, string*>(Substructuring::getOpt));
-    Global.Add("isSetOption", "(", new OneOperator1_<bool, string*>(Substructuring::isSetOpt));
+    Init_Common();
     Global.Add("buildSkeleton", "(", new Substructuring::Skeleton);
 #if defined(DSUITESPARSE) || defined(DHYPRE)
     const char ds = 'G';
@@ -630,14 +612,18 @@ static void Init_Substructuring() {
     // Substructuring::add<HpFetiPrec, std::complex<float>, zs>();
     // zzzfff->Add("cfeti", atype<HpFetiPrec<std::complex<float>, zs>*>());
 #endif
-    // Dcl_Type<Pair<float>*>(InitP<Pair<float>>, Destroy<Pair<float>>);
-    // zzzfff->Add("spair", atype<Pair<double>*>());
-    Dcl_Type<Pair<double>*>(InitP<Pair<double>>, Destroy<Pair<double>>);
-    zzzfff->Add("dpair", atype<Pair<double>*>());
-    // Dcl_Type<Pair<std::complex<float>>*>(InitP<Pair<std::complex<float>>>, Destroy<Pair<std::complex<float>>>);
-    // zzzfff->Add("cpair", atype<Pair<std::complex<float>>*>());
-    Dcl_Type<Pair<std::complex<double>>*>(InitP<Pair<std::complex<double>>>, Destroy<Pair<std::complex<double>>>);
-    zzzfff->Add("zpair", atype<Pair<std::complex<double>>*>());
+    aType t;
+    int r;
+    if(!zzzfff->InMotClef("dpair", t, r)) {
+        // Dcl_Type<Pair<float>*>(InitP<Pair<float>>, Destroy<Pair<float>>);
+        // zzzfff->Add("spair", atype<Pair<double>*>());
+        Dcl_Type<Pair<double>*>(InitP<Pair<double>>, Destroy<Pair<double>>);
+        zzzfff->Add("dpair", atype<Pair<double>*>());
+        // Dcl_Type<Pair<std::complex<float>>*>(InitP<Pair<std::complex<float>>>, Destroy<Pair<std::complex<float>>>);
+        // zzzfff->Add("cpair", atype<Pair<std::complex<float>>*>());
+        Dcl_Type<Pair<std::complex<double>>*>(InitP<Pair<std::complex<double>>>, Destroy<Pair<std::complex<double>>>);
+        zzzfff->Add("zpair", atype<Pair<std::complex<double>>*>());
+    }
 }
 
 LOADFUNC(Init_Substructuring)
