@@ -1515,8 +1515,6 @@ public:
 	    MPI_Bcast(  dataint, 40, MPI_INT,  0, comm );
 	    MPI_Bcast(  datadouble, 15, MPI_DOUBLE,  0, comm );
 	    
-	    fclose(pFile);
-	    delete [] retfile;
 	  }
 	  else{
 	    
@@ -2553,6 +2551,7 @@ bool SetDefault()
     DefSparseSolver<double>::solver =SparseMatSolver_R;
     DefSparseSolver<Complex>::solver =SparseMatSolver_C;
     TypeSolveMat::defaultvalue =TypeSolveMat::SparseSolver;
+       return false;
 }
 
 bool SetMUMPSmpi()
@@ -2562,6 +2561,7 @@ bool SetMUMPSmpi()
     DefSparseSolver<double>::solver  =BuildSolverMUMPSmpi;
     DefSparseSolver<Complex>::solver =BuildSolverMUMPSmpi;    
     TypeSolveMat::defaultvalue  = TypeSolveMatdefaultvalue;
+       return false;
 }
 
 
@@ -2580,6 +2580,7 @@ static void Load_Init()
   DefSparseSolver<Complex>::solver =BuildSolverMUMPSmpi;
   if(! Global.Find("defaultsolver").NotNull() )
     Global.Add("defaultsolver","(",new OneOperator0<bool>(SetDefault));
+  if(! Global.Find("defaulttoMUMPS").NotNull() )
   Global.Add("defaulttoMUMPS","(",new OneOperator0<bool>(SetMUMPSmpi));
 }
 
@@ -2597,6 +2598,7 @@ void ffinit()
   DefSparseSolver<Complex>::solver =BuildSolverMUMPSmpi;
   if(! Global.Find("defaultsolver").NotNull() )
     Global.Add("defaultsolver","(",new OneOperator0<bool>(SetDefault));
+  if(! Global.Find("defaulttoMUMPS").NotNull() )
   Global.Add("defaulttoMUMPS","(",new OneOperator0<bool>(SetMUMPSmpi));
 }
 #include "InitFunct.hpp"
