@@ -8,7 +8,7 @@
 //         Modif by F. Hecht 2011
 // E-MAIL   : Guy_Antoine_Atenekeng_Kahou@lri.fr
 //
-//ff-c++-LIBRARY-dep:  hips metis  blas  mpi
+//ff-c++-LIBRARY-dep:  hips scotchmetis scotch   blas  mpi
 //ff-c++-cpp-dep: 
 
 /* 
@@ -45,8 +45,12 @@ using namespace std;
 #endif
 
 extern "C" {
+typedef int SCOTCH_Num ;
+
+void   METIS_PartGraphKway(const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num * const, const SCOTCH_Num * const, SCOTCH_Num * const, SCOTCH_Num * const);
+
 #include "hips.h"  
-#include "metis.h"
+//#include "metis.h"
 }
 #include <stdio.h>
 #include <stdlib.h>
@@ -446,6 +450,7 @@ public:
     }
     int option[5];	option[0]=0;
     if(nproc>1){
+        
       METIS_PartGraphKway(&n, AA.lg, AA.cl, NULL, NULL, &wgtflag, &numflag,&nproc, option, &volume, riord);
     }
     else if(nproc==1){
@@ -699,9 +704,9 @@ BuildSolverHipsSolvermpi(DCL_ARG_SPARSE_SOLVER(double,A))
     cout << " BuildSolverSuperLU<double>" << endl;
   return new HipsSolver(*A,ds.epsilon,ds.data_filename, ds.lparams, ds.dparams,ds.master,(MPI_Comm *)ds.commworld);
 		    }
-class Init { public:
+/* --FH:   class Init { public:
     Init();
-};
+    }; */
 
 //  the 2 default sparse solver double and complex
 DefSparseSolver<double>::SparseMatSolver SparseMatSolver_R ; ;
@@ -742,8 +747,8 @@ bool SetHipsSolver()
   TypeSolveMat::defaultvalue =TypeSolveMatdefaultvalue;
   return 1;
 }
-LOADINIT(Init);
-Init::Init()
+
+static void Load_Init()
 {   
   SparseMatSolver_R= DefSparseSolver<double>::solver;
   //SparseMatSolver_C= DefSparseSolver<Complex>::solver;
@@ -764,3 +769,4 @@ Init::Init()
 		      
 		      
 		      
+ LOADFUNC(Load_Init)
