@@ -8,7 +8,7 @@
 // E-MAIL   : Guy_Antoine_Atenekeng_Kahou@lri.fr
 //
 // ALH - need reference to MPI to have the proper include for mpi.h on MacOS10.8
-//ff-c++-LIBRARY-dep: parms metis blas mpifc fc mpi pthread
+//ff-c++-LIBRARY-dep: parms scotchmetis scotch  blas mpifc fc mpi pthread
 //ff-c++-cpp-dep: 
 
 /* 
@@ -68,12 +68,15 @@ extern "C" {
   // included, the one from the first package specified above on the
   // line [[LIBRARY-dep]]
 
-  #include "../download/include/metis/defs.h"
+typedef int SCOTCH_Num ;
 
-  #include "metis.h"
+
+  //  #include "../download/include/metis/defs.h"
+
+ #include "metis.h"
 }
 
-#ifdef WIN32
+#ifdef _WIN32
 
 // ALH - this activates the Windows DLL default export mechanism
 
@@ -215,7 +218,7 @@ public:
 		if(param_int.N()>12){if(param_int[12]>0) prepar->lfil[4]=param_int[12]; else prepar->lfil[4]=20;}
 		else prepar->lfil[4]=20; /* lfil for Schur complement const.   */
 
-		if(param_int.N()>13){if(param_int[13]>0) prepar->lfil[5]=param_int[13]; else prepar->lfil[13]=20;}
+		if(param_int.N()>13){if(param_int[13]>0) prepar->lfil[5]=param_int[13]; else prepar->lfil[5]=20;}
 		else prepar->lfil[5]=20; /* lfil for Schur complement const.   */		
 
 		if(param_int.N()>14){if(param_int[14]>0) prepar->mc=param_int[14]; else prepar->mc=1; }
@@ -1021,9 +1024,9 @@ BuilddSolvePARMS(DCL_ARG_SPARSE_SOLVER(double,A))
     return new dSolvePARMS(*A,ds.data_filename, ds.lparams, ds.dparams,(MPI_Comm *)ds.commworld);
 }
 
-class Init { public:
+/* --FH:   class Init { public:
     Init();
-};
+    };*/
 
 //  the 2 default sparse solver double and complex
 DefSparseSolver<double>::SparseMatSolver SparseMatSolver_R ; 
@@ -1048,8 +1051,8 @@ bool SetdSolvePARMS()
      TypeSolveMat::defaultvalue  = TypeSolveMatdefaultvalue;
      return TRUE;
 }
-LOADINIT(Init);
-Init::Init()
+
+static void Load_Init()
 { 
   
   SparseMatSolver_R= DefSparseSolver<double>::solver;
@@ -1066,3 +1069,4 @@ Init::Init()
 }
 
 
+ LOADFUNC(Load_Init)
