@@ -1,5 +1,5 @@
 //   for automatic  compilation with ff-c++
-//ff-c++-LIBRARY-dep: superlu_dist  blas parmetis metis mpi fc
+//ff-c++-LIBRARY-dep: superlu_dist   ptscotchparmetis  ptscotch scotchmetis  scotch  mpi blas fc
 //ff-c++-cpp-dep: 
 //
 // ORIG-DATE: 02/2009
@@ -273,7 +273,7 @@ public:
 	  else{
 	    /*
 	      printf("\tProcess grid\t%d X %d\n", grid.nprow, grid.npcol);
-	      /* Receive matrix A from PE 0. */
+	       Receive matrix A from PE 0. */
 	    MPI_Bcast( &m,   1,   mpi_int_t,  0, grid.comm );
 	    MPI_Bcast( &n,   1,   mpi_int_t,  0, grid.comm );
 	    MPI_Bcast( &nnz, 1,   mpi_int_t,  0, grid.comm );
@@ -759,9 +759,9 @@ BuildSolverSuperLUmpi(DCL_ARG_SPARSE_SOLVER(double,A))
 }
 
 
-class Init { public:
+/* --FH:   class Init { public:
     Init();
-};
+    };*/
 
 //  the 2 default sparse solver double and complex
 DefSparseSolver<double>::SparseMatSolver SparseMatSolver_R ; ;
@@ -793,8 +793,8 @@ bool SetSuperLUmpi()
 
 
 
-LOADINIT(Init);
-Init::Init()
+
+static void Load_Init()
 { 
   
   SparseMatSolver_R= DefSparseSolver<double>::solver;
@@ -810,3 +810,4 @@ Init::Init()
   Global.Add("realdefaulttoSuperLUdist","(",new OneOperator0<bool>(SetSuperLUmpi));
 }
 
+LOADFUNC(Load_Init)
