@@ -98,6 +98,11 @@ int   ReadOnePlot(FILE *fp)
   const char *  magic2="#!ffglutdata2..";
   const char *  magic3="#!ffglutdata3..";
   const char *  magic3_1="#!ffglutdata3.1";
+  const char *  magic3_2="#!ffglutdata3.2";
+/*
+ 
+   change version 3.2 add colored curve
+ */
   const int lmagic=strlen(magic2);
     char magicxx[32];
   err=0;
@@ -106,6 +111,7 @@ int   ReadOnePlot(FILE *fp)
     {
       for(int i=0;i<lmagic;i++)
 	{ int c=getc(fp);
+            if(i==0 && c == EOF) return -2; // empty file 
 	    magicxx[i]=c;
 	  //err += c != magic[i];
 	  //if(err) break;
@@ -114,7 +120,8 @@ int   ReadOnePlot(FILE *fp)
 	if( strcmp(magicxx,magic2)==0)  version=2;
 	else if( strcmp(magicxx,magic3)==0)  version=3;
 	else if( strcmp(magicxx,magic3_1)==0)  version=3;
-	else err =1; 
+        else if( strcmp(magicxx,magic3_2)==0)  version=3;
+	else err =1;
 	
       if(err) {
 	if(debug>2)
@@ -968,7 +975,7 @@ template<class Mesh>
 	if(inCadre[i])
 	  {
 	    ccc=true;
-	    if(what%10==1)
+	    if(what%10==1 )
 	      {
 		R f=v[o+i];
 		fmn=min(f,fmn);
@@ -992,25 +999,81 @@ template<class Mesh>
     }
 }
 
+OnePlotCurve::OnePlotCurve(PlotStream & f,int nfield,ThePlot *theplot)
+:OnePlot(3,2,1)
+{
+    f >> xx>>yy;
+    if( nfield >=3)
+        f >> zz;
+    if( nfield ==4)
+        f >> cc;
+     // cout << xx << " " << yy <<endl;
+    ffassert(f.good());
+    ffassert(xx.N() && yy.N() && xx.N() == yy.N());
+    Pmin=Minc(Pmin,R2(xx.min(),yy.min()));
+    Pmax=Maxc(Pmax,R2(xx.max(),yy.max()));
+    if(zz.N())
+    {
+        Pmin.z = Min(Pmin.z,zz.min());
+        Pmax.z = Max(Pmax.z,zz.max());
+        if( cc.N() ==0)
+        {
+        fmax = Max(fmax,zz.max());
+        fmin = Min(fmin,zz.min());
+        theplot->withiso=true;
+        }
 
+    }
+    if( cc.N() )
+    {
+        fmax = Max(fmax,cc.max());
+        fmin = Min(fmin,cc.min());
+        theplot->withiso=true;
+    }
+    if(debug>3) cout << " OnePlotCurve nbfield "<< nfield << " max, N= " << xx.max() << " "
+        << xx.N() << ", " << yy.max() << " " << yy.N() << " "<< zz.N() << " " << cc.N() << " f " << fmin << " " << fmax << endl;;
 
+}
+
+void OnePlotCurve::dyn_bfv(OneWindow *win,R & fmn,R &fmx,R & vmn2,R & vmx2) const
+{
+    if( cc.N() == xx.N())
+    {
+        0; // afaire ????
+    }
+}
 void OnePlotCurve::Draw(OneWindow *win)
 {
   initlist();
-
   ThePlot & plot= *win->theplot;
-  plot.SetColorTable(16) ;
-    double z = plot.z0;
-  
+  // correct of color bug thank to Jon Sauer
+ // if( cc.N() != xx.N())
+ //     plot.SetColorTable(16) ;
+ // else
+ //   plot.SetColorTable(Max(plot.Niso,plot.Narrow)+4) ;
+    // end of correct !!!! FH nov 2016 
+  double z = plot.z0;
+    if(debug>3) cout << " OnePlotCurve::Draw " << (cc.N() != xx.N()) << endl;
   glBegin(GL_LINE_STRIP);    
   plot.color(2);
   // cout << "nePlotCurve::Draw " << xx << " " << yy << endl;
-  for (int i=0;i<xx.N();i++)
-    {
+  if( (zz.N()!=xx.N())  && (cc.N()==0)  )
+   for (int i=0;i<xx.N();i++)
       glVertex3d(xx[i],yy[i],z);
-      
-    }
-  glEnd(); 
+  else if (cc.N()==0)
+      for (int i=0;i<xx.N();i++)
+          glVertex3d(xx[i],yy[i],zz[i]);
+  else
+      for (int i=0;i<xx.N();i++)
+      {
+          int col = 2+dichotomie(plot.Viso,cc[i]);
+          if(col<1) col=1;
+          if(col>plot.Viso.N()) col=plot.Viso.N();
+          plot.color(col);
+          glVertex3d(xx[i],yy[i],zz[i]);
+      }
+
+  glEnd();
     
 }
 
@@ -1097,13 +1160,16 @@ void OneWindow::set(ThePlot *p)
     if(p)
       {
 	plotdim=p->plotdim;
+        rapz0 = p->ZScale;
       }
     if(!init)
     {
-    rapz0 =-1; // to recompute the defalut rapz
+      rapz0 =-1;
+      if(p) rapz0 = p->ZScale;
+    // to recompute the defalut rapz
     //    p->win=this;
     //    if(first)
-    DefaultView(2) ;
+     DefaultView(2) ;
     }
     
 }
@@ -1140,9 +1206,12 @@ void OneWindow::DefaultView(int state)
    {
       if(state==0 && init) return;
   }
-  else /*if(state==2)*/ rapz0=-1;
-    
-  if(theplot)
+  else /*if(state==2)*/ {
+      if( theplot && theplot->ZScale>0) rapz0=theplot->ZScale;
+      else rapz0=-1;
+  }
+
+    if(theplot)
     {
       init =1;
       plotdim=theplot->plotdim;
@@ -1153,6 +1222,7 @@ void OneWindow::DefaultView(int state)
       zmin = theplot->fmin;
       theta=theplot->theta;
       phi=theplot->phi;
+      if(theplot->ZScale>0) rapz0=theplot->ZScale;
       if(theplot->datadim==3) rapz0=1;
       else   if(rapz0<=0)
 	{ //  ( zmax-zmin )*rapz0 =  0.3 dxyy
@@ -1465,7 +1535,7 @@ void OneWindow::cadreortho(R2 A, R2 B)
     
     Bmin= M - D2;
     Bmax= M + D2;
-    
+    hpixel = (Bmax.x-Bmin.x)/width;     
     if((debug > 10)) cout << " cadreortho: "<< " :: " << Bmin << " " << Bmax <<" oB " << oBmin << " " << oBmax << endl;
     
     // if((debug > 10)) cout << "cadreortho\n";
@@ -1705,6 +1775,7 @@ ThePlot::ThePlot(PlotStream & fin,ThePlot *old,int kcount)
   psfile=0;
   cm=0;
   grey=0;
+  ZScale=-1; // auto
   ArrowSize=-1;
   if(old) {
     grey=old->grey;
@@ -1785,8 +1856,10 @@ case 20+index: {type dummy; fin >= dummy;} break;
 	  case 18: fin >= add; break;
 	  case 19: fin >= keepPV; break;
 	  case 20: fin >= echelle;break;
+          case 21: fin >= ZScale;break;
+                  
                   // unsed parameter ...
-                  READ_VTK_PARAM(1,double); // ZScale
+                  //READ_VTK_PARAM(1,double); // ZScale
                   READ_VTK_PARAM(2,bool); // WhiteBackground
                   READ_VTK_PARAM(3,bool); // OpaqueBorders
                   READ_VTK_PARAM(4,bool); // BorderAsMesh
@@ -1956,8 +2029,9 @@ case 20+index: {type dummy; fin >= dummy;} break;
       OnePlot *p=0;
       fin >> what;
       long imsh;
-      if((debug > 2)) cout << "    plot  " << i << " what " << what << endl;
-      if(what !=3 && !uaspectratio) aspectratio= true;
+      if((what !=3 && what != 13  )&& !uaspectratio) aspectratio= true;
+        if((debug > 2)) cout << "    plot  " << i << " what " << what << " as : " << aspectratio << endl;
+        
       if(what==-1)  // gestion of error (empty plot)
 	p = new OnePlotError(fin);
       else if(what==0) 
@@ -1984,6 +2058,8 @@ case 20+index: {type dummy; fin >= dummy;} break;
 	}
       else if(what==3)
 	p=new OnePlotCurve(fin);
+      else if(what==13)
+          p=new OnePlotCurve(fin,4,this);// add zz and color ..
       else if(what==4)
 	p=new OnePlotBorder(fin);
       else if(what==5) 
@@ -2052,8 +2128,9 @@ case 20+index: {type dummy; fin >= dummy;} break;
   Viso.resize(Niso);
   Varrow.resize(Narrow);
   
-  SetColorTable(Max(Niso,Narrow)+4) ;           
-  SetDefIsoV() ;
+  SetColorTable(Max(Niso,Narrow)+4) ;
+  SetDefIsoV(Niso,Narrow,fmin,fmax) ;
+   //SetDefIsoV(ni,na,fmn,fmx,vmn,vmx)
 
 }
 
@@ -2062,19 +2139,22 @@ void ThePlot::SetDefIsoV(int niso,int narr,double fmn,double fmx,double vmn,doub
 {
   bool dyn=false; 
   R d,x;
-  
+  if(debug>3 && !(fmx>fmn) )
+        cout << " SetDefIsoV  (not) " << endl;
+
   if( fmx>fmn)
     {
       if(debug>3)
-	cout << "Set Def dyn_bfv  " << fmn << " " << fmx << endl;
-
+	cout << " SetDefIsoV  " << fmn << " " << fmx << "pViso= " << pViso << endl;
+        dyn = niso !=Viso.N() ;
+        pViso = pViso && niso ==Viso.N();
       if(niso>2) 
 	Viso.resize(niso); 
       Niso=Viso.N();
       Narrow=narr;
       d =  (fmx-fmn)/(Niso-2) ;
       x =  (fmn+fmx)/2-d*0.5*(Niso-1);
-      dyn=true;
+     // dyn=true;
     }
   else
     {
@@ -2263,10 +2343,10 @@ void hsvToRgb (float h, float s, float v, float & r, float & g, float & b)
     if (s == 0) /* Grayscale */
 	r = g = b = v;
     else {
-	if (h == 1.0) h = 0;
-	h *= 6.0;
-	i =  int(h);
-	f = h - i;
+        h = h - floorf(h);
+        h*=6.;
+        i =  int(h);
+        f = h - i;
 	aa = v * (1 - s);
 	bb = v * (1 - (s * f));
 	cc = v * (1 - (s * (1 - f)));
@@ -2948,6 +3028,8 @@ int main(int argc,  char** argv)
 	    else if( strcmp(argv[i1],"-v")==0) i1++,debug=2,verbosity=2;
 	    else if( strcmp(argv[i1],"-vv")==0) i1++,debug=5,verbosity=2;
 	    else if( strcmp(argv[i1],"-vvv")==0) i1++,debug=10, verbosity=1000;
+            else if( strcmp(argv[i1],"-vvvv")==0) i1++,debug=99999, verbosity=1000;
+            else if( strcmp(argv[i1],"-0")==0) return 0; // do nothing 
 	  }
 	  if( (i1+1 < argc) && (strcmp(argv[i1],"-wait")==0)) { i1++; gwait=atof(argv[i1++]); }
 	  if( (i1+1 < argc) && (strcmp(argv[i1],"-g")==0)) { 
