@@ -73,7 +73,9 @@ inline int BinaryRand(){
  return rand() & 16384; // 2^14 (for sun because RAND_MAX is not def in stdlib.h)
 #endif
 
-} 
+}
+    
+    
 typedef P2<Real8,Real8> R2;
 typedef P2xP2<Int2,Int4> I2xI2;
 typedef P2<Real4,Real8> R2xR2;
@@ -649,7 +651,7 @@ class VertexOnGeom {  public:
   operator GeometricalEdge * () const  {return ge;}
 //  operator Real8 & () {return abscisse;}
   operator const Real8 & () const {return abscisse;}
-  int IsRequiredVertex(){ return this? (( abscisse<0 ? (gv?gv->Required():0):0 )) : 0;}
+  int IsRequiredVertex(){ return  (( abscisse<0 ? (gv?gv->Required():0):0 )) ;}
   void SetOn(){mv->on=this;mv->vint=IsVertexOnGeom;}
   friend ostream& operator <<(ostream& f, const  VertexOnGeom & vog){
     f << vog.abscisse << " " << vog.mv << " " << vog.gv << " ; ";
@@ -825,8 +827,8 @@ public:
  // void  RandomInit();
  // void  CubeInit(int ,int);
   
-  Real8 MinimalHmin() {return 2.0/coefIcoor;}
-  Real8 MaximalHmax() {return Max(pmax.x-pmin.x,pmax.y-pmin.y);}
+  Real8 MinimalHmin() const {return 2.0/coefIcoor;}
+  Real8 MaximalHmax() const {return Max(pmax.x-pmin.x,pmax.y-pmin.y);}
   const Vertex & operator[]  (Int4 i) const { return vertices[i];};
   Vertex & operator[](Int4 i) { return vertices[i];};
   const Triangle & operator()  (Int4 i) const { return triangles[i];};
@@ -889,7 +891,7 @@ public:
   
   Vertex * NearestVertex(Icoor1 i,Icoor1 j) ;
   Triangle * FindTriangleContening(const I2 & ,Icoor2 [3],Triangle *tstart=0) const;
-  void Write(const char * filename,const TypeFileMesh type = AutoMesh);
+  void Write(const char * filename,const TypeFileMesh type = AutoMesh) const;
   void Write_am_fmt(ostream &) const ;
   void Write_am(ostream &) const ;
   void Write_ftq(ostream &) const ;
