@@ -704,7 +704,7 @@ retry:
     Real8 sg;
     //   cout << "           " << Number(on) << " " <<  Number(eg0) << " " <<  Number(eg1) << " "  ; 
     if (eg0 == eg1) { 
-       register Real8 s0= vg0,s1=vg1;
+        Real8 s0= vg0,s1=vg1;
        sg =  s0 * (1.0-s) +  s * s1;
        //    cout <<"                s0=" << s0 << " s1=" << s1 
        //             << " s = " << s << " sens= " << OppositeSens << "\t\t sg = " << sg << endl ;
@@ -869,14 +869,14 @@ void Geometry::AfterRead()
         exch = 0;
         Int4  *p =  hv + i, *po = p;
 	Int4 n = *p;
-        register float angleold = -1000 ; // angle = - infini 
+         float angleold = -1000 ; // angle = - infini 
         ord = 0;
         while (n >=0) 
         {
           ord++;
-          register Int4 i1= n /2;
-          register Int4  j1 = n % 2;
-          register Int4 *pn = ev + n;
+           Int4 i1= n /2;
+           Int4  j1 = n % 2;
+           Int4 *pn = ev + n;
           float angle = j1 ? OppositeAngle(eangle[i1]):  eangle[i1];
           n = *pn;
           if (angleold > angle) // exch to have : po -> pn -> p 
@@ -1014,10 +1014,12 @@ void Geometry::AfterRead()
     NbOfCurves = 0;
     Int4  nbgem=0;
     for (int level=0;level < 2 && nbgem != nbe;level++)
-      for (i=0;i<nbe;i++) {
+     for(jj=0;jj<2;jj++) //  change the sens of loop to keep interior normal in general..  feb. 2017
+      for (i=0;i<nbe;i++)
+//        for(jj=0;jj<2;jj++)  // old code
+          {
 	GeometricalEdge & ei = edges[i];   
-	for(jj=0;jj<2;jj++) 
-	  if (!ei.Mark() && (level || ei[jj].Required())) { 
+	  if (!ei.Mark() && (level || ei[jj].Required())) {
 	    // warning ei.Mark() can be change in loop for(jj=0;jj<2;jj++) 
 	    int k0=jj,k1;
 	    GeometricalEdge *e = & ei;
@@ -1063,18 +1065,48 @@ void Geometry::AfterRead()
     for(int i=0;i<NbOfCurves ;i++)
      {
        GeometricalEdge * be=curves[i].be, *eqbe=be->link;
-       //GeometricalEdge * ee=curves[i].ee, *eqee=be->link;
+       GeometricalEdge * ee=curves[i].ee, *eqee=ee->link;
        curves[i].master=true;
        if(be->Equi() || be->ReverseEqui() ) 
         {
+            
           assert(eqbe);
           int nc = eqbe->CurveNumber;
+            GeometricalEdge * bee=curves[nc].be;
+            GeometricalEdge * eee=curves[nc].ee;
+            
           assert(i!=nc);
           curves[i].next=curves[nc].next;
           curves[i].master=false;
           curves[nc].next=curves+i;
-          if(be->ReverseEqui())
-           curves[i].Reverse();           
+          int sens =1;
+          if( eqbe != bee )
+          {
+              assert(eqee == bee);// eq end == begin
+              assert(eqbe == eee);// eq beg == end
+              assert(eqbe == eee);
+              if(be != ee)
+                sens = -1;
+            
+          }
+          else {
+              if( (be==ee) && (curves[i].kb != curves[nc].kb)) sens = -1;
+          }
+            
+           bool reverse =be->ReverseEqui();
+            //if( sens<0) reverse = !reverse; // remove 9 may 2017 
+            reverse=sens; // FH  Correct 9 may 2017 (ReverseEqui is wrong ??? maybe)
+          if(reverse)
+           curves[i].Reverse();
+            if(verbosity>9)
+            {
+                cout << " --  curve equi /reverse="<< reverse << " sens: " << sens << endl;
+                cout << "    curve  "<< i << ": "<< Number(be) << " <=> " << Number(eqbe) << " " << be->Equi() << be->ReverseEqui()
+                     <<" -->  "<< Number(ee) << " <=>" << Number(eqee) << " " << ee->Equi() << ee->ReverseEqui() <<endl;
+                cout << "    curve eq: "<< nc << ": "<< Number(bee)  << " " << bee->Equi() << bee->ReverseEqui()
+                <<" -->  "<< Number(eee)  << " !" << eee->Equi() << eee->ReverseEqui() <<endl;
+            }
+            
         }
      }
     	 
