@@ -39,7 +39,7 @@ extern "C" {
 #include "FESpace.hpp" 
 #include "DOperator.hpp"
 #include "QuadratureFormular.hpp" 
-
+extern double ff_tgv;
 using  Fem2D::Mesh;
 using  Fem2D::FESpace;
 using Fem2D::FElement;
@@ -389,7 +389,8 @@ public:
   virtual void Solve(KN_<R> & x,const KN_<R> & b) const =0;
   virtual ~MatriceCreuse(){}
   virtual R & diag(int i)=0;
-  virtual void SetBC(int i,double tgv)=0; 
+  virtual void SetBC(int i,double tgv)=0;
+  virtual void SetBC(char *wbc,double tgv) { for (int i=0; i<n; ++i)  if(wbc[i]) SetBC(i,tgv);}
   virtual R & operator()(int i,int j)=0;
   virtual R * pij(int i,int j) const =0; // Add FH 
   virtual  void  resize(int n,int m)  {AFAIRE("MatriceCreuse::resize");}  // a faire dans les classe derive ... // add march 2009  FH 
@@ -406,8 +407,8 @@ public:
   virtual void getcoef( KN_<R> & x) const =0 ;
   // Add FH oct 2005
    bool ChecknbLine(int nn) const { return n==nn;}  
-   bool ChecknbColumn(int mm) const { return m==mm;} 
-
+   bool ChecknbColumn(int mm) const { return m==mm;}
+   virtual R trace() const {ffassert(n==m);  R t=R(), *p;  for(int i=0; i<n; ++i)  { p=pij(i,i);  if(p) t+= *p;} return t; }
   // end ADD
   virtual bool sym() const {return false;}
 
@@ -471,7 +472,7 @@ public:
     
     
     {return MatriceProfile(this->n,0,0,0,U,pU);}
-
+   R trace() const {ffassert(this->n==this->m);  R t=R();  for(int i=0; i<this->n; ++i) t+= D[i]; return t; }
   void Solve(KN_<R> &x,const KN_<R> &b) const {
     /*if (typefac==0)  code faux // FH   nov 2006
       switch(typefac) 
@@ -499,6 +500,7 @@ public:
       if( tgv>=0) D[i]=tgv;
       else  { ffassert(tgv<0); }  // to hard ..
   }
+    
   R & operator()(int i,int j) { if(i!=j) ffassert(0); return D[i];} // a faire 
   R * pij(int i,int j) const { if(i!=j) ffassert(0); return &D[i];} // a faire  Modif FH 31102005
   MatriceMorse<R> *toMatriceMorse(bool transpose=false,bool copy=false) const ;
@@ -634,17 +636,46 @@ MatriceMorse(int nn,int mm,int nbc,bool sym,R *aa=0,int *ll=0,int *cc=0,bool dd=
   R  operator()(int i,int j) const {R * p= pij(i,j) ;throwassert(p); return *p;}
   R & operator()(int i,int j)  {R * p= pij(i,j) ;throwassert(p); return *p;}
   R & diag(int i)  {R * p= pij(i,i) ;throwassert(p); return *p;}
+    R trace() const {ffassert(this->n==this->m);  R t=R(),*p;  for(int i=0; i<this->n; ++i) {p=pij(i,i) ;if(p) t+= *p; } return t; }
   void SetBC (int i,double tgv) {
 	R * p= pij(i,i) ;
 	ffassert(p);
 	if( tgv>=0) *p=tgv;
 	else  {
-	    ffassert(!symetrique); 
+            ffassert(!symetrique);// 
 	    for (int k=lg[i];k<lg[i+1]; ++k) a[k]=0;// put the line to Zero.
 	    *p = 1. ; // and the diag coef to 1.
 	}  
     }
     
+void SetBC (char * wbc,double tgv)
+    {
+        for(int i=0; i< this->n; ++i)
+            if(tgv<0)
+            {
+                
+                if( wbc[i] )
+                {
+                    for (int k=lg[i];k<lg[i+1]; ++k)
+                        if( cl[k]==i)
+                            a[k] = 1.;
+                        else
+                            a[k]=0;// put the line to Zero.
+                }
+                else if(tgv < -1.999)
+                    for (int k=lg[i];k<lg[i+1]; ++k)
+                    {
+                        int j = cl[k];
+                        if( wbc[j] ) a[k]=0;//
+                    }
+            }
+            else  if( wbc[i] ) { // tgv >= 0
+                R * p= pij(i,i) ;
+                ffassert(p);
+                *p=tgv;
+            }
+    }
+  
   void SetSolver(const VirtualSolver & s){solver=&s;}
   void SetSolverMaster(const VirtualSolver * s){solver.master(s);}
   bool sym() const {return symetrique;}
@@ -799,6 +830,7 @@ int ConjuguedGradient2(const M & A,const P & C,KN_<R> &x,const KN_<R> &b,const i
        R hAh =(h,Ah);
          if (RNM::norm2(hAh)<1e-60) ExecError("CG2: Matrix is not defined (/0), sorry ");
        ro =  - (g,h)*rop/hAh ; // ro optimal (produit scalaire usuel)
+       if ( ro != ro) ExecError("CG2: Bug : ro is NaN ???  ");
        x += (ro-rop) *h;
        g += (ro/rop) *Ah; // plus besoin de Ah, on utilise avec Cg optimisation
        Cg = C*g;
@@ -828,7 +860,12 @@ class MatriceIdentite:public  VirtualMatrice<R> { public:
     MatriceIdentite(int n) :VirtualMatrice<R>(n) {}; 
  void addMatMul(const  KN_<R>  & x, KN_<R> & Ax) const { 
      ffassert(x.N()==Ax.N());
-   Ax+=x; } 
+   Ax+=x; }
+    void addMatTransMul(const  KN_<R>  & x, KN_<R> & Ax) const {
+    ffassert(x.N()==Ax.N());
+    Ax+=x; }
+     void Solve( KN_<R> & y ,const KN_<R> & x) const { y=x; }
+      bool WithSolver() const {return true;}
  plusAx operator*(const KN<R> &  x) const {return plusAx(this,x);} 
   bool ChecknbLine(int n) const { return true;}  
   bool ChecknbColumn(int m) const { return true;} 
@@ -916,6 +953,10 @@ struct Data_Sparse_Solver {
     // array for return information for mumps ...
     KN<double> * rinfo;
     KN<long> * info;
+    
+    KNM<double>* kerneln;
+    KNM<double> * kernelt;
+    long *kerneldim;
  /*   
   int *param_int;
   double *param_double;
@@ -934,7 +975,7 @@ struct Data_Sparse_Solver {
     initmat(1),
     typemat(0),
     strategy(0),
-    tgv(1e30),
+    tgv(ff_tgv),
     factorize(0),
     epsilon(1e-6),
     precon(0),
@@ -965,7 +1006,8 @@ struct Data_Sparse_Solver {
     commworld(0),
     master(0),
     rinfo(0),
-    info(0)
+    info(0),
+    kerneln(0), kernelt(0), kerneldim(0)
     {}
     
 private:
