@@ -257,7 +257,7 @@ struct  VirtualMatrice { public:
     { InternalError("VirtualMatrice::addMatTransMul not implemented "); }
   virtual bool WithSolver() const {return false;} // by default no solver          
   virtual void Solve( KN_<R> &  ,const KN_<R> & ) const 
-    { InternalError("VirtualMatrice::solve not implemented "); } 
+    { InternalError("VirtualMatrice::solve not implemented.\n In  FeeeFem++ add instruction like  set(A, solver= sparsesolver);\n// where A is the current matrix  "); } 
 
 #ifdef VersionFreeFempp
   virtual bool ChecknbLine  (int n) const= 0; 
@@ -268,18 +268,18 @@ struct  VirtualMatrice { public:
 #endif
   struct  plusAx { const VirtualMatrice * A; const KN_<R>   x;
    plusAx( const VirtualMatrice * B,const KN_<R> &  y) :A(B),x(y) 
-      { ffassert(B->ChecknbColumn(y.N())); }
+      { if(B) { ffassert(B->ChecknbColumn(y.N())); } }
     };
     
    plusAx operator*(const KN_<R> &  x) const {return plusAx(this,x);}
    
   struct  plusAtx { const VirtualMatrice * A; const KN_<R>   x;
    plusAtx( const VirtualMatrice * B,const KN_<R> &  y) :A(B),x(y) 
-    {ffassert(B->ChecknbLine(y.N()));} };
+    { if(B) { ffassert(B->ChecknbLine(y.N())); } } };
     
   struct  solveAxeqb { const VirtualMatrice * A; const KN_<R>   b;
    solveAxeqb( const VirtualMatrice * B,const KN_<R> &  y) :A(B),b(y) 
-    {ffassert(B->ChecknbColumn(y.N()));} };
+    { if(B) { ffassert(B->ChecknbColumn(y.N())); } } };
   
   virtual ~VirtualMatrice(){} 
 };
@@ -391,6 +391,7 @@ public:
   long N() const {return n;}
   bool unset() const { return !v;}
   void set(R * vv,int nn,int st=1,int nx=-1) {v=vv;n=nn;step=st;next=nx;}
+  void set(KN_<R> t) {v=t.v;n=t.n;step=t.step;next=t.next;} //  FH Add Nov 2016
   long size() const{return step?n*step:n;}
   operator R *() const {return v;}
   KN_(const KN_<R> & u) :ShapeOfArray(u),v(u.v){} 
@@ -543,15 +544,15 @@ public:
  //  KN_& operator =(const MatriceCreuseMulKN_<R> & ) ;
  //  KN_& operator +=(const MatriceCreuseMulKN_<R> & ) ;
    KN_& operator =(const typename VirtualMatrice<R>::plusAx & Ax)  
-    {*this=R(); Ax.A->addMatMul(Ax.x,*this);return *this;}
+    { if(Ax.A) { ffassert(&Ax.x[0] != &this->operator[](0));*this=R(); Ax.A->addMatMul(Ax.x,*this); } return *this;}
    KN_& operator =(const typename VirtualMatrice<R>::plusAtx & Ax)  
-    {*this=R(); Ax.A->addMatTransMul(Ax.x,*this);return *this;}
+    { if(Ax.A) { ffassert(&Ax.x[0] != &this->operator[](0));*this=R(); Ax.A->addMatTransMul(Ax.x,*this); } return *this;}
    KN_& operator +=(const typename VirtualMatrice<R>::plusAx & Ax)  
-    {  Ax.A->addMatMul(Ax.x,*this);return *this;}
+    { if(Ax.A) { ffassert(&Ax.x[0] != &this->operator[](0)); Ax.A->addMatMul(Ax.x,*this); } return *this;}
    KN_& operator +=(const typename VirtualMatrice<R>::plusAtx & Ax)  
-    {  Ax.A->addMatTransMul(Ax.x,*this);return *this;}
+    { if(Ax.A) { ffassert(&Ax.x[0] != &this->operator[](0)); Ax.A->addMatTransMul(Ax.x,*this); } return *this;}
    KN_& operator =(const typename VirtualMatrice<R>::solveAxeqb & Ab)  
-    {*this=R(); Ab.A->Solve(*this,Ab.b);return *this;}
+    { if(Ab.A) { ffassert(&Ab.b[0] != &this->operator[](0));*this=R(); Ab.A->Solve(*this,Ab.b); } return *this;}
     
   template<class  A,class B,class C,class D> KN_&  operator =  (const F_KN_<A,B,C,D>  & u) ;
   template<class  A,class B,class C,class D> KN_&  operator +=  (const F_KN_<A,B,C,D>  & u) ;
@@ -1018,9 +1019,9 @@ class KN :public KN_<R> { public:
         
         
    KN& operator =(const Mul_KNM_KN_<R> & u) 
-        { if(this->unset()) this->set(new R[u.b.N()],u.b.N());KN_<R>::operator=(u);return *this;}
+        { if(this->unset()) this->set(new R[u.A.N()],u.A.N());KN_<R>::operator=(u);return *this;}
     KN& operator =(const Mul_KNMh_KN_<R> & u)
-    { if(this->unset()) this->set(new R[u.b.N()],u.b.N());KN_<R>::operator=(u);return *this;}
+    { if(this->unset()) this->set(new R[u.A.M()],u.A.M());KN_<R>::operator=(u);return *this;}
    
 //   KN& operator =(const MatriceCreuseMulKN_<R> & Ax) 
 //       {if(this->unset()) set(new R[Ax.v.N()],Ax.v.N()); KN_<R>::operator=(Ax);return *this;}
@@ -1030,7 +1031,7 @@ class KN :public KN_<R> { public:
 //       { if(this->unset()) set(new R[A1x.v.N()],A1x.v.N());KN_<R>::operator=(A1x);return *this;}
   // correcton aout 2007 FH  add N,M flied in VirtualMatrice
    KN& operator =(const typename VirtualMatrice<R>::plusAx & Ax)  
-        { if(this->unset() && Ax.A->N ) this->set(new R[Ax.A->N],Ax.A->N);KN_<R>::operator=(Ax);return *this;}
+        { if(this->unset() && Ax.A && Ax.A->N ) this->set(new R[Ax.A->N],Ax.A->N); if(Ax.A) KN_<R>::operator=(Ax);return *this;}
    KN& operator =(const typename VirtualMatrice<R>::solveAxeqb & Ab)  
         { if(this->unset()) this->set(new R[Ab.b.N()],Ab.b.N());KN_<R>::operator=(Ab);return *this;}
    KN& operator +=(const typename  VirtualMatrice<R>::plusAx & Ax)  
