@@ -166,7 +166,8 @@ public:
 		}
 		for (int k=0;k<nt;k++) TonBoundary[k]=0;
 		
-		BoundaryEdgeHeadLink = new int[neb];       
+		BoundaryEdgeHeadLink = new int[neb];
+                int nbbadsensedge=0;
 		for (i=0;i<neb;i++)
 		{  
 		    BoundaryEdge & be(bedges[i]);
@@ -178,7 +179,8 @@ public:
 		    throwassert(i1 >=0 && i1 < nv);
 		    throwassert(i1 != i0) ;
 		    int im=Min(i0,i1);
-		    BoundaryEdgeHeadLink[i]=-1; 
+		    BoundaryEdgeHeadLink[i]=-1;
+                    int badsens =1; // bad
 		    for ( n=Head[i0+i1]; n>=0; n=TheAdjacencesLink[n])
 		    {
 			int jj=n%3,ii=n/3, jj0,jj1;
@@ -187,15 +189,16 @@ public:
 			{
 			    TonBoundary[n/3] += MaskEdge[n%3];
 			    BoundaryEdgeHeadLink[i]=n;                  
-			    if(i0==jj0) break; // FH 01072005 bon cote de l'arete 
+                            if(i0==jj0) {badsens=0;break;} // add check
+                            // FH 01072005 bon cote de l'arete
 					       // sinon on regard si cela existe?
 			}
 		    } 
 		    if ( BoundaryEdgeHeadLink[i] <0 && verbosity) 
 			cout << "   Attention l'arete frontiere " << i 
 			    << " n'est pas dans le maillage " <<i0 << " " << i1 <<  endl;
+                    else if(badsens) nbbadsensedge++;
 		}
-		
 		//  find adj
 		// reffecran();    
 		
@@ -246,7 +249,7 @@ public:
 			if (NbAdj) NbOfEdges++;
 			if(NbAdj==1)
                         {
-                            if (! (TonBoundary[i]& MaskEdge[j]))
+                            if (! (TonBoundary[i]& MaskEdge[j]) && 0)
 			    { NbOfMEdges++;
 				if(verbosity>99) 
 				    cout << " Edge (" << j0 << " "<< j1 << ") : "  << j  << " of Triangle " << &T-triangles << " on mortar \n"
@@ -270,6 +273,7 @@ public:
 		    NbMortars =0;
 		    NbMortarsPaper=0;
 	    }
+        
 		{
 		    //  construct the mortar 
 		    int * linkg = new int [nv]; //  to link
@@ -1590,15 +1594,18 @@ Mesh::Mesh(const Mesh & Th,int * split,bool WithMortar,int label)
 	    if (split[it]) 
 		hm=Min(hm,Th[it].h_min()/(R) split[it]);
 	}
-	R seuil=hm/400.0;
-	if(verbosity>5)   
-	    cout << " seuil = " <<  seuil << " hmin = " << hm <<  endl; 
-	assert(seuil>1e-15);
+	R seuil=hm/splitmax/4.0;
+	//assert(seuil>1e-15);
 	vertices = new Vertex[nvmax];
 	assert( vertices );
 	
 	nv =0;
 	quadtree = new FQuadTree(this,Pmin,Pmax,nv); // build empty the quadtree
+        long iseuil =(long) (quadtree->coef*seuil);
+        if(verbosity>5)
+         cout << " seuil = " <<  seuil << " hmin = " << hm << " iseuil/ quadtree: " << iseuil <<  endl;
+
+       ffassert(iseuil); //  zero => too smal
         {  // to keep the order of old vertices to have no problem in
              // interpolation on sub grid of RT finite element for example (Feb. 2016 F. Hecht)
         KN<bool> setofv(Th.nv,false);
@@ -1741,13 +1748,13 @@ Mesh::Mesh(const Mesh & Th,int * split,bool WithMortar,int label)
 
 		//   cout << "          (debug)  nb vertices on egdes " << nv << endl;  
 		    //  cout << " " <<  nebmax << " " << neb << endl;
-	assert(neb==nebmax);
-	assert(nebi==nebimax);
-	assert(nbsdd==nbsddmax);
+	ffassert(neb==nebmax);
+	ffassert(nebi==nebimax);
+	ffassert(nbsdd==nbsddmax);
 	
 	//   create the new vertices and the new triangle 
 	int   kt=0;
-	for (int K=0;K<Th.nt;K++)
+ 	for (int K=0;K<Th.nt;K++)
 	{ // cout << K << endl;
 	    Triangle &T(Th[K]);
 	    R2 A(T[0]);
@@ -1767,14 +1774,15 @@ Mesh::Mesh(const Mesh & Th,int * split,bool WithMortar,int label)
 		    R lmin = Min(la,lb,lc);
 		    R2 Pj= A*la+B*lb+C*lc; 
 		    Vertex *pV;
-		    pV=quadtree->ToClose(Pj,seuil);
+		    pV=quadtree->ToClose(Pj,seuil,true);
 		    // if !noregenereration => point du bord du triangle deja genere 
 		    bool addv = !pV;
 		    if(!noregenereration && pV==0) addv = lmin > 1e-5;
 		    if ( addv )			
 		    { // new vertex
 		      // cout << "    -- " << nv << "  New Vertices " << Pj << " n=" << n << " j=" << j << " " << PTj << endl;
-			assert(nv<nvmax);
+                        if( !(nv < nvmax ))
+			ffassert(nv<nvmax);
 			vertices[nv]=Pj;
 			(Label&) vertices[nv]=0; //  Internal vertices 
 			pV = vertices + nv;
@@ -2006,7 +2014,7 @@ Mesh::Mesh(const  Serialize &serialized)
     serialized.get( pp,nv);
     serialized.get( pp,neb);
     if (verbosity>2) 
-	cout << " mesh serialized : " << l << " " << nt << " " << nv << " " << neb << endl;
+	cout << " mesh serialized : l " << l << " / " << nt << " " << nv << " " << neb << endl;
     assert ( nt > 0 && nv >0 && neb >=0);
     triangles = new Triangle [nt];
     vertices  = new Vertex[nv];
@@ -2063,7 +2071,7 @@ Serialize  Mesh::serialize() const
     serialized.put( pp,nv);
     serialized.put( pp,neb);
     if (verbosity>2) 
-      cout << " mesh Serialized : " << l << " "  << nt << " " << nv << " " << neb << endl;
+      cout << " mesh dSerialized : " << l << " /"  << nt << " " << nv << " " << neb << endl;
     for (int i=0;i<nv;i++)
     {
 	serialized.put(pp, vertices[i].x);
