@@ -130,7 +130,7 @@ const   R3 TetHat[4]= { R3(0.,0.,0.),R3(1.,0.,0.),R3(0.,1.,0.),R3(0.,0.,1.) } ;
 // --------
 #include "Label.hpp"
 
-  
+  // <<TVertex>>
 template<class Rd>
 class TVertex : public Rd,public Label {
  friend class Mesh;
@@ -463,8 +463,8 @@ public:
   Mesh( const Serialize & ) ;
   Mesh(int nbv,R2 * P);
 
-  R mesure(){ return area;}
-  R bordermesure(){ return lenbord;}
+  R mesure() const { return area;}
+  R bordermesure() const { return lenbord;}
   
   Serialize serialize() const;
   Mesh(int nbv,int nbt,int nbeb,Vertex *v,Triangle *t,BoundaryEdge  *b);  
