@@ -90,11 +90,11 @@ III. Renvoie le type d'erreur:
   mshptg erreurs
 1: mshptg := 'le nb de point est < 3 or > que le nb max de point';
 2: mshptg := 'il y des points confondus ';
-3: mshptg := 'tout les points sont aligns ';
-7: mshptg := 'bug dans l''algorithme pour retrouver la frontire (Internal Bug mshfrt)';
-8: mshptg := 'Une arte a forcer traverse trop de triangles (Internal Bug : mshfr1)';
-9: mshptg := 'la frontire est croise ';
-10: mshptg := 'il y a un point qui est sur une arte frontire';
+3: mshptg := 'tout les points sont align»s ';
+7: mshptg := 'bug dans l''algorithme pour retrouver la frontiŸre (Internal Bug mshfrt)';
+8: mshptg := 'Une arte a forcer traverse trop de triangles (Internal Bug : mshfr1)';
+9: mshptg := 'la frontiŸre est crois»e ';
+10: mshptg := 'il y a un point qui est sur une arte frontiŸre';
 11: mshptg := 'Un sous domaine est reference par aucun triangle ';
 20: mshptg := 'mshopt: 3 points confondus (Internal Bug) ';
 21: mshptg := 'la pile de mshopt est trop petit (Internal Bug)';
@@ -1700,6 +1700,7 @@ L50:
 				mshfr1_ (&c[3], &nu[7], &t, &ta, &is1, &s2, err);
 				if (*err != 0)
 				   {
+                                       printf(" Error %ld inforce edge   %ld = %ld;%ld  with vertices  %ld %ld \n",*err,a,i_2, i_3, is1,s2 );
 				     return 0;
 				   }
 				goto L100;
@@ -2038,6 +2039,8 @@ L20:
   if (ta <= 0)
      {
        *err = 9;
+         long sa= nu[ (la+1)%3 + t * 6], sb=nu[(la+2)%3 + t * 6];
+       printf("mshptg: bug  boundary crossing %ld,%ld , %ld %ld \n", s1, *s2 , sa,sb);
        return 0;
      }
   t = ta / 8;
