@@ -226,7 +226,7 @@ template<class RR> RR LDecremantation(RR* a){ return --(*a);}
 template<class RR> RR RDecremantation(RR* a){ return (*a)--;}
 
 template<class RR,class B>
- RR * New_form_string(string * s) {B * r=  new B(s);delete s;return r;}// correct Mars 2011 remove * if delete
+ RR * New_form_string(string * s) {B * r=  new B(s);freestring(s);return r;}// correct Mars 2011 remove * if delete
  
  
 
@@ -320,8 +320,7 @@ AnyType TTry(Stack s ,Expression ins,Expression ccatch,Expression fin,Expression
         if(verbosity> 2) cerr << "Try:: catch (...) exception " << endl;
         a=(*ccatch)(s);         
      }
-   
-   a=(*fin)(s);
+   if(fin)  a=(*fin)(s);
   return a;
 }
 
@@ -398,6 +397,7 @@ void ShowKeyWord(ostream & f )
  
  }
 
+// <<dumptable>>
 ostream* dumptable(ostream* f)
 {
 
@@ -918,7 +918,7 @@ struct evalE_mul {
 };
 istream *Getline(istream * f, string ** s)
 {
-    if( *s==0) *s=new string;
+    if( *s==0) *s=newstring();
     getline(*f,**s);
     size_t l = (**s).length();
     if( l > 0 && ((**s)[l-1]=='\r')) (**s).resize(l-1); //
@@ -965,8 +965,47 @@ inline long fftime()
 #endif
     return -1;
 }
+long ffstrtol(string* p)
+{
+    char * pe;
+    const char *pp=p->c_str();
+    long r = strtol(pp,&pe,10);
+    const char *ppe = pe, *pppe= pp+p->size();
+   // cout << " .ffstrtol. " << pppe - pe << endl;
+    assert(ppe <= pppe);
+    for(const char *ppe = pe; ppe < pppe; ++ppe)
+        ffassert(isspace(*ppe));
+    return r;
+    
+}
+long ffstrtol(string* p,long d)
+{
+    char * pe;
+    const char *pp=p->c_str();
+    long r = strtol(pp,&pe,d);
+    const char *ppe = pe, *pppe= pp+p->size();
+  //  cout << " .ffstrtol. " << pppe - pe << endl;
+    ffassert(ppe <= pppe);
+    for(const char *ppe = pe; ppe < pppe; ++ppe)
+        ffassert(isspace(*ppe));
+    return r;
+}
 
-long atoi(string* p) {return atoi(p->c_str());}// add march 2010
+double ffstrtod(string* p)
+{
+    char * pe;
+    const char *pp=p->c_str();
+    double r = strtod(pp,&pe);
+    const char *ppe = pe, *pppe= pp+p->size();
+    //  cout << " .ffstrtol. " << pppe - pe << endl;
+    ffassert(ppe <= pppe);
+    for(const char *ppe = pe; ppe < pppe; ++ppe)
+        ffassert(isspace(*ppe));
+    return r;
+    
+}
+
+long atoi(string* p) {return atol(p->c_str());}// add march 2010
 double atof(string* p) {return atof(p->c_str());}// add march 2010
 double NaN(string* p) {
 return nan(p->c_str());}// add march 2012
@@ -985,46 +1024,92 @@ class E_ForAllLoopMapSS
     
     typedef MyMap<K,V>  *Tab;
     typedef   MyMap<K,V>::iterator TabI ;
-
+    
     typedef  ForAllLoopOpBase DataL;
     const DataL *data;
     E_ForAllLoopMapSS(const DataL *t): data(t){}
     AnyType f(Stack s) const {
+        TabI  ii;
         Tab t= GetAny<Tab >(data->tab(s));
         
-         KK * i   =   GetAny<KK* >(data->i(s));
-         VV * v   =   GetAny<VV* >(data->v(s));
+        KK * i   =   GetAny<KK* >(data->i(s));
+        VV * v   =   GetAny<VV* >(data->v(s));
         if(verbosity>1000) {
-        //      cout << i << " " << j << " " << v << " " << data->epl <<     endl;
-                cout << " i " << (char*) (void *) i -  (char*)(void*) s ;
-        cout << " vi " <<  (char*) (void *) v -  (char*)(void*) s ;
+            //      cout << i << " " << j << " " << v << " " << data->epl <<     endl;
+            cout << " i " << (char*) (void *) i -  (char*)(void*) s ;
+            cout << " vi " <<  (char*) (void *) v -  (char*)(void*) s ;
             cout << endl;}
         
         
         ffassert(i && v);
         if(t->m)
-        for (TabI ii=t->m->begin();ii != t->m->end();++ii)
+            ii=t->m->begin();
+        bool ok = true;
+        while(ok)
         {
-            String  kk = ii->first;
-            String  vv = ii->second;
+            if(verbosity>99999) cout << " new n";
+            TabI iip=ii++;
+            ok = ii != t->m->end();
+            String  kk = iip->first;
+            String  vv = iip->second;
+            const string * pvo=iip->second;
             
             *i =  kk;
             *v =  vv;
-           // for  Windows otherwise trap ???? FH. march 2016 
-            if(verbosity>99999) cout << " " << i << " "<< v  << " "  << kk << " " <<  vv << endl;
-
+            const string * pv =*v;
+            // for  Windows otherwise trap ???? FH. march 2016
+            if(verbosity>99999) cout << "  b:" << i << " "<< v  << " "  << kk << " " <<  vv << endl;
+            
             data->code(s);
+            if(verbosity>99999) cout << "  a:" << i << " "<< v  << " "  << kk << " " <<  **v << endl;
+            if( pvo  == (const string *) iip->second) // no  change m[i]=
+            {if( *v != pv ) //  v change
+                iip->second  = **v;
+            }
+            else if( *v != pv )
+            {//  v change
+                cerr << " Erreur forall change m[i] and mi (stupide please choosse) \n";
+                ffassert(0);
+            }
+            if(verbosity>99999) cout << " A;" << i << " "<< v  << " "  << kk << " " <<  vv << " ok =" << ok << endl;
             
-            ii->second  = **v;
             *i=0;
             *v=0;
+            if(verbosity>99999)
+            {
+                cout << " 0:" << i << " "<< v  << " "  << kk << " " <<  vv << endl;
+                for (TabI iii=t->m->begin();iii!= t->m->end();++iii)
+                    cout << " map =" << iii->first << " -> " << iii->second << endl;
+                cout << " end of map " << endl;
+            }           
+            // ii=iinext;//  
             
         }
-        data->end(s);
+      //  if(verbosity>99999) cout << "befor end \n";
+      //  data->end(s);
+      //  if(verbosity>99999) cout <<"afert end \n";
         return Nothing  ;
     }
     
 };
+double projection(const double & aa, const double & bb, const double & x )
+{ double a=aa,b=bb; if(a>b) std::swap(a,b); return min(max(a,x),b);}
+double dist(const double & aa, const double & bb) { return sqrt( aa*aa+bb*bb);}
+double dist(const double & aa, const double & bb,const double & cc) { return sqrt(aa*aa+bb*bb+cc*cc);}
+// Add Jan 2017 FH
+double diffpos(const double & aa, const double & bb) { return aa<bb ? bb-aa : 0.;}
+double invdiffpos(const double & aa, const double & bb) { return aa<bb ? 1./(bb-aa) : 0.;}
+double diffnp(const double & aa, const double & bb) { return aa<0. && 0.>bb ? bb-aa : 0.;}
+double invdiffnp(const double & aa, const double & bb) { return aa<0. && 0.>bb  ? 1./(bb-aa) : 0.;}
+double invdiff(const double & aa, const double & bb) { double d= aa-bb; return abs(d) < 1e-30 ? d : 1/d;}
+double invdiff(const double & aa, const double & bb,const double &eps) { double d= aa-bb; return abs(d) < eps ? d : 1/d;}
+extern double ff_tgv; // Add FH jan 2018
+double sign(double x){return (x>0.)-(x<0.); }// Add FH jan 2018
+long sign(long x){return (x>0)-(x<0); }// Add FH jan 2018
+bool ffsignbit(long x){return signbit(x);}
+bool ffsignbit(double x){return signbit(x);}
+template<typename T>
+bool  pswap(T *a,T *b) {swap(*a,*b);return 0; }
 
 void Init_map_type()
 {
@@ -1075,7 +1160,7 @@ void Init_map_type()
         
     Dcl_Type<ios::openmode>();
     
-//  les types des variables 
+    // <<known_variable_types>> les types des variables 
     
   zzzfff->Add("real",typevarreal=atype<double*>());
   zzzfff->Add("int",atype<long*>());
@@ -1137,10 +1222,14 @@ void Init_map_type()
       
      Global.New("verbosity",CPValue<long>(verbosity));
      Global.New("searchMethod",CPValue<long>(searchMethod)); //pichon
+     Global.New("tgv",CPValue<double>(ff_tgv));
+    
     extern long newconvect3;// def in global.cpp 
     Global.New("newconvect",CPValue<long>(newconvect3)); //pichon
     
+     // <<cout>> uses [[file:AFunction.hpp::CConstant]]
      Global.New("cout",CConstant<ostream*>(&cout));
+     
      Global.New("cerr",CConstant<ostream*>(&cerr));// add jan 2014 FH.
      Global.New("cin",CConstant<istream*>(&cin));
      Global.New("append",CConstant<ios::openmode>(ios::app));
@@ -1152,6 +1241,7 @@ void Init_map_type()
      Global.New("pi",CConstant<double>(3.14159265358979323846264338328));
      Global.New("version",CConstant<double>(VersionNumber()));
       
+     Global.New("showCPU",CPValue<bool>(showCPU));
      Global.New("CPUTime",CConstant<bool*>(&showCPU));
      // def de Zero et One
      pZero = new  C_F0(CConstant<double>(0.0));
@@ -1184,7 +1274,9 @@ void Init_map_type()
        new OneBinaryOperator<Op2_sub<double,double,double> >,
        new OneBinaryOperator<Op2_sub<double,double,long> >,
        new OneBinaryOperator<Op2_sub<double,long,double> >,
-       new OneBinaryOperator<Op2_sub<long,bool,bool> >,              
+       new OneBinaryOperator<Op2_sub<long,bool,bool> >,
+       new OneBinaryOperator<Op2_sub<long,long,bool> >,
+       new OneBinaryOperator<Op2_sub<long,bool,long> >,
        new OneBinaryOperator<Op2_sub<Complex,Complex,Complex> >,
        new OneBinaryOperator<Op2_sub<Complex,Complex,double> >,
        new OneBinaryOperator<Op2_sub<Complex,double,Complex> >,
@@ -1193,6 +1285,7 @@ void Init_map_type()
        );
        
      TheOperators->Add("*",
+       new OneBinaryOperator<Op2_mul<bool,bool,bool>,OneBinaryOperatorMI,evalE_mul<bool> >,
        new OneBinaryOperator<Op2_mul<long,long,long>,OneBinaryOperatorMI,evalE_mul<long> >,
        new OneBinaryOperator<Op2_mul<double,double,double>,MIMul<double,double>,evalE_mul<double> >,
        new OneBinaryOperator<Op2_mul<double,double,long>, MIMul<double,long>,evalE_mul<double,long,double> >,
@@ -1201,8 +1294,28 @@ void Init_map_type()
        new OneBinaryOperator<Op2_mul<Complex,Complex,double> >,
        new OneBinaryOperator<Op2_mul<Complex,double,Complex> >,
        new OneBinaryOperator<Op2_mul<Complex,Complex,long> >,
-       new OneBinaryOperator<Op2_mul<Complex,long,Complex> >       
+       new OneBinaryOperator<Op2_mul<Complex,long,Complex> >,
+       new OneBinaryOperator<Op2_mul<long,long,bool> >,
+       new OneBinaryOperator<Op2_mul<long,bool,long> >
        );
+    // add missing operation jan 2017 FH.
+    TheOperators->Add("*",
+                      new OneBinaryOperator<Op2_mul<Complex,Complex,bool> >,
+                      new OneBinaryOperator<Op2_mul<Complex,bool,Complex> >
+                      );
+    TheOperators->Add("+",
+                      new OneBinaryOperator<Op2_add<Complex,Complex,bool> >,
+                      new OneBinaryOperator<Op2_add<Complex,bool,Complex> >
+                      );
+    TheOperators->Add("-",
+                      new OneBinaryOperator<Op2_sub<Complex,Complex,bool> >,
+                      new OneBinaryOperator<Op2_sub<Complex,bool,Complex> >
+                      );
+    TheOperators->Add("/",
+                //      new OneBinaryOperator<Op2_sub<Complex,Complex,bool> >,
+                      new OneBinaryOperator<Op2_div<Complex,bool,Complex> >
+                      );
+
      TheOperators->Add("/",
        new OneBinaryOperator<Op2_div<long,long,long> >,
        new OneBinaryOperator<Op2_div<double,double,double> >,
@@ -1290,7 +1403,7 @@ void Init_map_type()
        new OneBinaryOperator<set_eq<long> ,OneBinaryOperatorMIWO>,
        new OneBinaryOperator<set_eq<double> ,OneBinaryOperatorMIWO>,
        new OneBinaryOperator<set_eq<Complex> ,OneBinaryOperatorMIWO>,
-       new OneBinaryOperator<set_peq<string> ,OneBinaryOperatorMIWO>  // FH string * mars 2006 
+       new OneBinaryOperator<set_peqstring ,OneBinaryOperatorMIWO>  // FH string * mars 2006 
        ); 
 
      TheOperators->Add("?:",
@@ -1512,7 +1625,7 @@ void Init_map_type()
      Global.Add("lround","(",new OneOperator1<long,double>(lround)); // add june 2007
      Global.Add("floor","(",new OneOperator1<double>(floor)); // add march 2006
      Global.Add("ceil","(",new OneOperator1<double>(ceil));  // add march 2006
-     Global.Add("rint","(",new OneOperator1<double>(rint));  // add june 2006
+    Global.Add("rint","(",new OneOperator1<double>(rint));  // add june 2006
      Global.Add("lrint","(",new OneOperator1<long,double>(lrint));  // add mars  2014
     
      Global.Add("sin","(",new OneOperator1<double>(sin));
@@ -1524,7 +1637,11 @@ void Init_map_type()
 
     Global.Add("atoi","(",new OneOperator1<long,string*>(atoi));// add march 2010
     Global.Add("atof","(",new OneOperator1<double,string*>(atof));// add march 2010
-    
+
+    Global.Add("strtol","(",new OneOperator1<long,string*>(ffstrtol));// add march 2017
+    Global.Add("strtol","(",new OneOperator2<long,string*,long>(ffstrtol));// add march 2017
+    Global.Add("strtod","(",new OneOperator1<double,string*>(ffstrtod));// add march 2017
+
 #ifdef HAVE_ATANH
      Global.Add("atanh","(",new OneOperator1<double>(atanh));
 #endif
@@ -1560,10 +1677,27 @@ void Init_map_type()
 //     Global.Add("pow","(",new OneOperator2<double,double,long>(pow));
      Global.Add("max","(",new OneOperator2_<double,double>(Max<double> ));
      Global.Add("min","(",new OneOperator2_<double,double>(Min<double> ));
+    Global.Add("diffpos","(",new OneOperator2_<double,double>(diffpos )); // jan 2018 FH
+    Global.Add("invdiffpos","(",new OneOperator2_<double,double>(invdiffpos )); // jan 2018 FH
+    Global.Add("diffnp","(",new OneOperator2_<double,double>(diffnp )); // jan 2018 FH
+    Global.Add("invdiffnp","(",new OneOperator2_<double,double>(invdiffnp )); // jan 2018 FH
+    Global.Add("invdiff","(",new OneOperator2_<double,double>(invdiff )); // jan 2018 FH
+    Global.Add("invdiff","(",new OneOperator3_<double,double,double>(invdiff )); // jan 2018 FH
+
      Global.Add("max","(",new OneOperator2_<long,long>(Max));
      Global.Add("min","(",new OneOperator2_<long,long>(Min));
-     Global.Add("atan2","(",new OneOperator2<double>(atan2));
-     Global.Add("hypot","(",new OneOperator2<double>(hypot));// add Jan 2014
+    Global.Add("max","(",new OneOperator3_<double,double>(Max<double> ));
+    Global.Add("min","(",new OneOperator3_<double,double>(Min<double> ));
+    Global.Add("max","(",new OneOperator3_<long,long>(Max));
+    Global.Add("min","(",new OneOperator3_<long,long>(Min));
+
+    Global.Add("atan2","(",new OneOperator2<double>(atan2));
+    Global.Add("fmod","(",new OneOperator2<double>(fmod));// add sep 2017
+    Global.Add("fdim ","(",new OneOperator2<double>(fdim));// add sep 2017
+    Global.Add("fmax ","(",new OneOperator2<double>(fmax));// add sep 2017
+    Global.Add("fmin ","(",new OneOperator2<double>(fmin));// add sep 2017
+ 
+    Global.Add("hypot","(",new OneOperator2<double>(hypot));// add Jan 2014
     
      Global.Add("atan","(",new OneOperator2<double>(atan2));
      Global.Add("sqrt","(",new OneOperator1<double>(sqrt,2));
@@ -1639,11 +1773,24 @@ void Init_map_type()
 
   Global.Add("NaN","(",new OneOperator1<double,string*   >(NaN));
     Global.Add("isNaN","(",new OneOperator1<long,double>(isNaN));
+    Global.Add("copysign","(",new OneOperator2<double>(copysign));// Add jan 2018 FH
+    Global.Add("sign","(",new OneOperator1<double>(sign));// Add jan 2018 FH
+    Global.Add("sign","(",new OneOperator1<long>(sign));// Add jan 2018 FH
+    Global.Add("signbit","(",new OneOperator1<bool,long>(ffsignbit));// Add jan 2018 FH
+    Global.Add("signbit","(",new OneOperator1<bool,double>(ffsignbit));// Add jan 2018 FH
+
     Global.Add("isInf","(",new OneOperator1<long,double>(isInf));
     Global.Add("isNormal","(",new OneOperator1<long,double>(isNormal));
     Global.Add("chtmpdir","(",new OneOperator0<long>(ffapi::chtmpdir));
- 
-  
+    Global.Add("projection","(",new OneOperator3_<double,double   >(projection));
+    Global.Add("dist","(",new OneOperator2_<double,double>(dist));
+    Global.Add("dist","(",new OneOperator3_<double,double>(dist));
+    Global.Add("swap","(",new OneOperator2<bool,double*>(pswap));
+    Global.Add("swap","(",new OneOperator2<bool,long*>(pswap));
+    Global.Add("swap","(",new OneOperator2<bool,bool*>(pswap));
+    Global.Add("swap","(",new OneOperator2<bool,Complex*>(pswap));
+    Global.Add("swap","(",new OneOperator2<bool,string**>(pswap));
+
 
      atype<MyMapSS*>()->Add("[","",new OneOperator2_<string**,MyMapSS*,string*>(get_elements));
 
@@ -1655,7 +1802,7 @@ void Init_map_type()
     TheOperators->Add("<<",new OneBinaryOperator<PrintP<MyMapSS*> >);
  
     TheOperators->Add("{}",new ForAllLoop<E_ForAllLoopMapSS >);
-   // add setw freb 2015 FH
+   // add setw feb 2015 FH
     Global.Add("setw","(",new OneOperator1<OP_setw,long>(defOP_setw));
     TheOperators->Add("<<", new OneBinaryOperator<Print<OP_setw> >);
 
@@ -1689,7 +1836,9 @@ void Init_map_type()
      CodeAlloc::clear();
      ShowAlloc("ClearMem: end" , lg); 
 
- } 
+ }
+
+// <<addingInitFunct>>
 static addingInitFunct TheaddingInitFunct(-10000,Init_map_type); 
 
 C_F0  opVI::code2(const basicAC_F0 &args) const
@@ -2048,8 +2197,22 @@ C_F0  opSum::code2(const basicAC_F0 &args) const
 	return C_F0(TheOperators,"[]",v);
     
 }
-
-
+//  to be sure new and delele be in see dll for windows
+string  *newstring(){string * p=new string();
+    if(verbosity>999999) cout << p << "=newstring() " <<endl;;
+      return p ;
+}
+string  *newstring(const string & c){
+    string * p=new string(c);
+    if(verbosity>999999) cout <<  p << "=newstring((string) "<< c << ") \n";
+   return p;}
+string  *newstring(const char * c){
+     string * p=new string(c);
+    if(verbosity>999999) cout << p << "=newstring((char*)  "<< c << " ) \n";
+    return p;}
+void   freestring(const string * c){
+    if(verbosity>999999) cout << c << "freestring(  "<< *c <<") \n";
+    delete c;}
 
 
 
