@@ -26,6 +26,9 @@
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#ifndef Operator_hpp_
+#define Operator_hpp_
+
 #if defined(__GNUC__) && __GNUC__+0 >= 3
 inline double pow(double x,long l) { return pow(x,(double)l);}
 #endif
@@ -271,15 +274,15 @@ struct set_eq_div: public binary_function<A*,A,A*> {
   static A* f(A* const  & a,const A & b)  { *a /= b; return a;}
 };
 
-template<class A>
-struct set_peq: public binary_function<A**,A*,A**> {
-  static A** f(A** const  & a, A * const & b)  { 
+
+struct set_peqstring: public binary_function<string**,string*,string**> {
+  static string** f(string** const  & a, string * const & b)  {
     if(*a != b )
     { 
 	//cerr << " set_peq " << *a << endl;
-	delete *a;
+	freestring(*a);
 	//cerr << " set_peq " << *a << " " << " = " << b << " " <<  endl;
-	*a = new A(*b); //(stack ptr) FH mars 2006
+	*a = newstring(*b); //(stack ptr) FH mars 2006
 	//cerr << " set_peq " << *a << " " << **a << " = " << *b << " " << b <<  endl;
     }
      return a;}
@@ -644,169 +647,4 @@ class Transpose{ public:
   template<class TT> Transpose( TT * v) : t(*v) {}  
   operator const T & () const {return t;}
 };
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-  
+#endif
