@@ -141,14 +141,18 @@ void initArrayOperatorlong()
      Add<KN_<long> >("sort",".",new OneOperator1_<KN_<K>,KN_<K> >(SortKn<K, KN_<K> >));
     // Add<KN<long> >("sort",".",new OneOperator1_<KN<K>,KN<K> >(SortKn<K, KN<K> >));
      Add<KN<long> *>("sort",".",new OneOperator1_<KN<K>*,KN<K>* >(SortpKn<K>));
-     
+     Global.Add("sort","(",new OneOperator2_<KN<K>*,KN<K>*,KN<long>* >(SortpKn2<K,long>));
+
      
 //     ArrayDCL<long>();
     
     Dcl_TypeandPtr_<KN_<String>,KN<String> *>(0,0,0,::Destroy<KN<String> >, ::ClearReturnKK_<K,KN<String>,KN_<String> >,::ClearReturnpKK<String,KN<String> >);
     atype<KN<String>* >()->Add("[","",new OneOperator2_<string**,KN<String>*,long >(get_elements));
     TheOperators->Add("<-", 
-		      new OneOperator2_<KN<String> *,KN<String> *,long>(&set_init_init));
+		      new OneOperator2_<KN<String> *,KN<String> *,long>(&set_init_init),
+                       new InitArrayfromArray<string*,KN<String>*,true>
+                      
+                      );
     map_type_of_map[make_pair(atype<long>(),atype<string*>())]=atype<KN<String>*>(); // vector
     Add<KN<String> *>("n",".",new OneOperator1<long,KN<String> *>(get_n));  
     extern   KN<String> *pkarg;
@@ -161,9 +165,15 @@ void initArrayOperatorlong()
 
     Add<KN<K> *>("imin",".",new OneOperator1<long,KN<K> *>(get_imin));
     Add<KN<K> *>("imax",".",new OneOperator1<long,KN<K> *>(get_imax));
+    Add<KNM<K> *>("imin",".",new OneOperator1<long,KNM<K> *>(get_imin));// Add april 2018 FH
+    Add<KNM<K> *>("imax",".",new OneOperator1<long,KNM<K> *>(get_imax));// Add april 2018 FH
+    Add<KNM<K> *>("jmin",".",new OneOperator1<long,KNM<K> *>(get_jmin));// Add april 2018 FH
+    Add<KNM<K> *>("jmax",".",new OneOperator1<long,KNM<K> *>(get_jmax));// Add april 2018 FH
+    TheOperators->Add("ijmax",new OneOperator3_<NothingType,KNM<K> *,long*,long*>(get_ijmax));// Add april 2018 FH
+    TheOperators->Add("ijmin",new OneOperator3_<NothingType,KNM<K> *,long*,long*>(get_ijmin));// Add april 2018 FH
     // madd FH. march 2015 ...
-    Global.Add("Unique", "(", new OneOperator2_<long, KN<K>*, KN<K>*>(Unique));
-    Global.Add("Unique", "(", new OneOperator2_<long, KN<K>*, KN<double>*>(Unique));
+    Global.Add("Unique", "(", new Unique<K, K>);
+    Global.Add("Unique", "(", new Unique<K, double>);
     // convertion double -> long (via lround)
     Dcl_Type<F_KN_<K,K,double> >();
     Global.Add("lround","(",new OneOperator1F_KN_<F_KN_<K,K,double>,K,double,KN_<double> >(lround));
@@ -175,7 +185,10 @@ void initArrayOperatorlong()
 
     TheOperators->Add("<-",new InitMapfromArray<MyMap<String,String>*,string *,string*,true> );
 
-    
+    TheOperators->Add("<<",
+                      new OneBinaryOperator<PrintPnd<KN<String> * > > // ADD mai 2018 FH.
+                      );
+
 }
 
  void xxxx() {
