@@ -130,7 +130,8 @@ void  HeapSort(T *c,long n,long o)
 template<class R,class A> A  SortKn(const A  & ca)
 { 
     A a(ca);
-    HeapSort<R>(&a[0],a.n,a.step);
+    if(a.n > 0)
+        HeapSort<R>(&a[0],a.n,a.step);
     return a;}
 
 template<class R,class RR,class A,class B> A  SortKn(const A  & ca,const B  & cb)
@@ -140,7 +141,8 @@ template<class R,class RR,class A,class B> A  SortKn(const A  & ca,const B  & cb
     const B &b(cb);
     ffassert(a.n == b.n);
     ffassert(a.step == b.step && b.step ==1);
-    HeapSort<R,RR>(&a[0],&b[0],a.n);
+    if(a.n > 0)
+        HeapSort<R,RR>(&a[0],&b[0],a.n);
     cout << b << endl;
 return a;}
 
@@ -150,12 +152,14 @@ template<class R,class RR> KN<R> *  SortpKn2( KN<R> * const & pa,KN<RR> * const
     KN<RR> &b(*pb);
     ffassert(a.n == b.n);
     ffassert(a.step == b.step && b.step ==1);
-    HeapSort<R,RR>(&a[0],&b[0],a.n);
+    if(a.n > 0)
+        HeapSort<R,RR>(&a[0],&b[0],a.n);
    return pa;}
 
 template<class R> KN<R> *  SortpKn( KN<R> * const & pa){ 
     KN<R> &a(*pa);
-    HeapSort<R>(&a[0],a.n,a.step);
+    if(a.n > 0)
+        HeapSort<R>(&a[0],a.n,a.step);
     return pa;}
 
 template<class R>
@@ -199,7 +203,7 @@ inline   bool exist_element( MyMap<String,K> *  const  &  a,string*  const   & b
 template<>
 inline   string ** get_element<string*>( MyMap<String,string*> *  const  &  a,string*  const   & b)
  { string** ret=  &((*a)[*b]); // correction FH feb 2004
-    if( *ret ==0) *ret = new string(""); //  string vide ???
+    if( *ret ==0) *ret = newstring(""); //  string vide ???
      // cout << "get_element " << *b << " : " << ret << " = "<< * ret << endl;
     // delete b;  modif mars 2006 auto del ptr
     return ret;}
@@ -274,15 +278,40 @@ template<class RR,class A,class B,class C>
 RR get_element_si(const A &  a,const B & b,const C & c){ 
  //  cout << c << " .... " << ((*a)(b,SubArray(1,c) )) << endl;;
      return  ((*a)('.',c)(b));}
-     
+
+template<class A,class B,class C>
+struct check_get_element_lineorcol
+{
+  static  void check(const A &  a,const B & b,const C & c){
+        if(c == ':' && (b<0 || a->N() <= b))
+            ExecError("Out of bound");
+        if(b == ':' && (c<0 || a->M() <= c))
+            ExecError("Out of bound");
+
+    }
+};
+template<class A,class B>
+struct check_get_element_lineorcol<A,B, char>
+{
+    static  void check(const A &  a,const B & b,const char & c){
+        if( (b<0 || a->N() <= b))
+            ExecError("Out of bound");
+    }
+};
+template<class A,class B>
+struct check_get_element_lineorcol<A, char,B>
+{
+    static  void check(const A &  a,const char & b,const B & c){
+        if( (c<0 || a->M() <= c))
+            ExecError("Out of bound");
+    }
+};
+
 template<class RR,class A,class B,class C>  
 RR get_element_lineorcol(const A &  a,const B & b,const C & c){ 
  //  cout << b << " .... " << ((*a)(SubArray(1,b),c)) << endl;;
-    if(c == ':' && (b<0 || a->N() <= b))
-            ExecError("Out of bound");
-    if(b == ':' && (c<0 || a->M() <= c))
-            ExecError("Out of bound");
-    return  ((*a)(b,c));
+     check_get_element_lineorcol<A,B,C>::check(a,b,c);
+     return  ((*a)(b,c));
     }
 
 template<class RR,class A,class B,class C>  
@@ -404,12 +433,12 @@ public:
 };
 
 
-template<class KNRR,bool isinit>
+template<class CR, class KNRR,bool isinit>
 class  InitArrayfromArray : public OneOperator { 
 public:
     typedef typename  myremove_pointer<KNRR>::type KNR ;
     typedef typename KNR::K RR;
-    typedef KNRR  A;
+     typedef KNRR  A;
     typedef KNRR  R;
     typedef E_Array B;
     
@@ -438,9 +467,9 @@ public:
         assert(&tt);
 	//      int err=0;
         for (int i=0;i<N;i++)
-	if(atype<RR>()->CastingFrom(tt[i].right() ) ) 
+	if(atype<CR>()->CastingFrom(tt[i].right() ) )
 	  {
-          tab[i]=atype<RR>()->CastTo(tt[i]);
+          tab[i]=atype<CR>()->CastTo(tt[i]);
 	    what[i]=0;
 	  }
 	else if(atype<KN_<RR> >()->CastingFrom(tt[i].right() ) ) 
@@ -479,7 +508,7 @@ public:
       {
        // cout << " ### " << i << " " << j << endl;
         if (what[i]==0)
-          a[j]= GetAny<RR>(v[i]);
+          a[j]= GetAny<CR>(v[i]);
         else if (what[i]==1) 
           a(SubArray(nn[i],j)) = GetAny<KN_<RR> >((*(tab[i]))(stack));// correct bug nov 2014
       }
@@ -656,13 +685,62 @@ public:
    
 
 
-template<class K> long get_n(KN<K> * p){ return p->N();}
-template<class K> long get_n(KNM<K> * p){ return p->N();}
-template<class K> long get_m(KNM<K> * p){ return p->M();}
-template<class K> K get_max(KN<K> * p){ return p->max();}
-template<class K> K get_min(KN<K> * p){ return p->min();}
+template<class K> long get_n(KN<K> * p){ return p->N();}//
+template<class K> long get_n(KNM<K> * p){ return p->N();}//
+template<class K> long get_m(KNM<K> * p){ return p->M();}//
+template<class K> K get_max(KN<K> * p){ return p->max();}//
+template<class K> K get_min(KN<K> * p){ return p->min();}// 
+
 template<class K> long get_imax(KN<K> * p){ long  i =0; for(long k=1;k< p->N(); ++k) if(  (*p)[k]>(*p)[i] ) i=k;  return i ;}
 template<class K> long get_imin(KN<K> * p){ long  i =0; for(long k=1;k< p->N(); ++k) if(  (*p)[k]<(*p)[i] ) i=k;  return i ;}
+template<class K> long get_imin(KNM<K> * p){
+    long  i =0,j=0;
+         for(long k=0;k< p->N(); ++k)
+             for(long l=0;l< p->M(); ++l) if(  (*p)(k,l) <(*p)(i,j) )  i=k,j=l;
+        return i ;}
+
+template<class K> long get_jmin(KNM<K> * p){
+    long  i =0,j=0;
+    for(long k=0;k< p->N(); ++k)
+        for(long l=0;l< p->M(); ++l) if(  (*p)(k,l) <(*p)(i,j) )  i=k,j=l;
+    return j ;}
+
+template<class K> long get_imax(KNM<K> * p){
+    long  i =0,j=0;
+    for(long k=0;k< p->N(); ++k)
+        for(long l=0;l< p->M(); ++l) if(  (*p)(k,l) >(*p)(i,j) )  i=k,j=l;
+    return i ;}
+/*template<class K> K get_max(KNM<K> * p){
+    K m = (*p)(0,0), mm;
+    for(long k=0;k< p->N(); ++k)
+        for(long l=0;l< p->M(); ++l) if(  (mm=(*p)(k,l)) > m ) m=mm ;
+    return m ;}
+template<class K> K get_min(KNM<K> * p){
+    K m = (*p)(0,0), mm;
+    for(long k=0;k< p->N(); ++k)
+        for(long l=0;l< p->M(); ++l) if(  (mm=(*p)(k,l)) < m ) m=mm ;
+    return m ;}
+*/
+template<class K> long get_jmax(KNM<K> * p){
+    long  i =0,j=0;
+    for(long k=0;k< p->N(); ++k)
+        for(long l=0;l< p->M(); ++l) if(  (*p)(k,l) >(*p)(i,j) )  i=k,j=l;
+    return j ;}
+template<class K> NothingType get_ijmax(KNM<K> * const & p,long * const & pi,long * const & pj ){
+    long  i =0,j=0;
+    for(long k=0;k< p->N(); ++k)
+        for(long l=0;l< p->M(); ++l) if(  (*p)(k,l) >(*p)(i,j) )  i=k,j=l;
+    *pi = i;
+    *pj = j;
+    return NothingType() ;}
+template<class K> NothingType get_ijmin(KNM<K> * const & p,long * const & pi,long * const & pj )
+{
+    long  i =0,j=0;
+    for(long k=0;k< p->N(); ++k)
+        for(long l=0;l< p->M(); ++l) if(  (*p)(k,l) <(*p)(i,j) )  i=k,j=l;
+    *pi = i;
+    *pj = j;
+    return NothingType() ;}
 
 template<class K> K get_sum(KN<K> * p){ return p->sum();}
 template<class K> double get_l2(KN<K> * p){ return p->l2();}
@@ -1037,16 +1115,50 @@ class  OneOperator_2KN_ : public OneOperator {public:
     OneOperator_2KN_<K>(): OneOperator(atype<KN_<K> >(),atype<E_Array>()) { pref=-1;}
 };
 template<class K, class L>
-long Unique(KN<K>* const& array, KN<L>* const& val)
-{
+class Unique_Op : public E_F0mps {
+    public:
+        Expression ar;
+        Expression va;
+        static const int n_name_param = 1;
+        static basicAC_F0::name_and_type name_param[];
+        Expression nargs[n_name_param];
+        Unique_Op<K, L>(const basicAC_F0& args, Expression param1, Expression param2) : ar(param1), va(param2) {
+            args.SetNameParam(n_name_param, name_param, nargs);
+        }
+        AnyType operator()(Stack stack) const;
+};
+template<class K, class L>
+basicAC_F0::name_and_type Unique_Op<K, L>::name_param[] = {
+    {"remove", &typeid(long)},
+};
+template<class K, class L>
+class Unique : public OneOperator {
+    public:
+        Unique() : OneOperator(atype<long>(), atype<KN<K>*>(), atype<KN<L>*>()) { }
+
+        E_F0* code(const basicAC_F0& args) const {
+            return new Unique_Op<K, L>(args, t[0]->CastTo(args[0]), t[1]->CastTo(args[1]));
+        }
+};
+template<class K, class L>
+AnyType Unique_Op<K, L>::operator()(Stack stack) const {
+    KN<K>* array = GetAny<KN<K>*>((*ar)(stack));
+    KN<L>* val = GetAny<KN<L>*>((*va)(stack));
     std::set<L> vals;
     for(int i = 0; i < array->n; ++i)
-        vals.insert((*array)[i]);
+    //    if(!nargs[0] || (*array)[i] != remove)
+            vals.insert((*array)[i]);
+    if( nargs[0]) // remove
+    {
+        long remove = GetAny<long>((*nargs[0])(stack)) ;
+        typename std::set<L>::iterator lr = vals.find(remove);
+        if( lr != vals.end()) vals.erase(lr);
+    }
     val->resize(vals.size());
     int i = 0;
     for(typename std::set<L>::iterator it = vals.begin(); it != vals.end(); ++it)
         (*val)[i++] = *it;
-    return vals.size();
+    return static_cast<long>(vals.size());
 }
 
 
@@ -1081,7 +1193,7 @@ class E_ForAllLoopRNM
                 data->code(s);
                 t(*i,*j)= *v;
             }
-        data->end(s);
+      //  data->end(s);
         return Nothing  ;
      }
     
@@ -1112,7 +1224,7 @@ class E_ForAllLoopRN
                 data->code(s);
                 t[*i]= *v;
             }
-        data->end(s);
+     //   data->end(s);
         return Nothing  ;
     }
     
@@ -1158,7 +1270,7 @@ class E_ForAllLoopMapSI
                 ii->second  = *v;
                 *i=0;
             }
-        data->end(s);
+      //  data->end(s);
         return Nothing  ;
     }
     
@@ -1278,7 +1390,7 @@ void ArrayOperator()
     
      TheOperators->Add("<-", 
        new OneOperator2_<KN<K> *,KN<K> *,Z>(&set_init),
-       new InitArrayfromArray<KN<K>*,true>
+       new InitArrayfromArray<K,KN<K>*,true>
     //   new OneOperator2_<KN<K> *,KN<K> *,KN<K> >(&set_init),
     //   new OneOperator2_<KN<K> *,KN<K> *,KN_<K> >(&set_init)		????       
      //  new OneOperator2_<KN<K> *,KN<K> *,KN<K> * >(&set_initp)
@@ -1309,7 +1421,7 @@ void ArrayOperator()
 
     // Add<KNM<K> *>("=","(",new OneOperator2_<KNM<K> *,KNM<K> *,Transpose<KNM<K> * > >(&set_tt));
     
-     Add<KN<K> *>("<-","(",new InitArrayfromArray<KN<K>*,true>);
+     Add<KN<K> *>("<-","(",new InitArrayfromArray<K,KN<K>*,true>);
      Add<KNM<K> *>("<-","(",new InitMatfromAArray<K,true>);
      Add<KN<K> *>("n",".",new OneOperator1<Z,KN<K> *>(get_n));
      Add<KNM<K> *>("n",".",new OneOperator1<Z,KNM<K> *>(get_n));
@@ -1329,8 +1441,8 @@ void ArrayOperator()
     
 //     AddOpeqarray<set_eqarray,KN,K>("=");
 
-     TheOperators->Add("=", new InitArrayfromArray<KN<K>*,false>(10));
-     TheOperators->Add("=", new InitArrayfromArray<KN_<K>,false>(1));// ???????? FH nov 2015 ..
+     TheOperators->Add("=", new InitArrayfromArray<K,KN<K>*,false>(10));
+     TheOperators->Add("=", new InitArrayfromArray<K,KN_<K>,false>(1));// ???????? FH nov 2015 ..
     TheOperators->Add("=", new InitMatfromAArray<K,false>
        );
      TheOperators->Add("=", new SetArrayofKNfromKN<K>
