@@ -1,7 +1,3 @@
-#ifdef WITH_PETSC
-#include <petsc.h>
-#endif
-
 /// \file
 /// ======================================================================
 /// Written by Antoine Le Hyaric
@@ -31,12 +27,17 @@
 // Proposed FreeFem++ Application Programming Interface
 // ----------------------------------------------------
 
-// headerfilter
-#include "ffapi.hpp"
+#ifdef WITH_PETSC
+#include <petsc.h>
+#endif
+
+// headerfilter [[shell:headerfilter '../../ff/src/fflib/ffapi.cpp']] [[file:~/alh/bin/headerfilter]]
+#include "ffapi.hpp" // [[file:ffapi.hpp]] found by [[file:~/alh/bin/headerfilter]]
+#include <cstdlib> 
 #ifdef FFLANG
 #include "socket.hpp"
 #include "spawn.hpp"
-#include "buffer.hpp"
+#include "buffer.hpp" // [[file:~/ffcs/src/buffer.hpp::Buffer]]
 #endif
 #include <iostream>
 #include <cstdio>
@@ -57,10 +58,14 @@
 #include "mpi.h"
 #endif
 #endif
+
+extern long verbosity ;
+
+
 // FFCS-specific implementations for the FF API
 // --------------------------------------------
 
-/// FFLANG defined means that FFCS is being compiled. I am fairly confident that FFCS will not be defined while
+/// <<FFLANG>> defined means that FFCS is being compiled. I am fairly confident that FFCS will not be defined while
 /// compiling the original FF.
 
 /// Need to choose a non-zero stream number because FF will check it (as global variable ThePlotStream)
@@ -68,6 +73,7 @@
 
 /// if FFCS is around, we need to bufferize all communications to avoid mixing up CMD_FFG and CMD_STDOUT messages
 #ifdef FFLANG
+
 void bufferwrite(const char *b,const int l){
   Socket::dialoglock->WAIT(); // need #include "socket.hpp"
 
@@ -81,13 +87,12 @@ void bufferwrite(const char *b,const int l){
   Socket::dialoglock->Free();
 }
 
-Buffer buffer(NULL,bufferwrite); // need #include "buffer.hpp"
+Buffer buffer(NULL,bufferwrite); // need #include "buffer.hpp" // [[file:~/ffcs/src/buffer.hpp::Buffer]]
 #endif
 //  allocation/definition of all ffapi in Global.cpp
 
 namespace ffapi{
     
-
   // Get a pointer to the local cin/cout (which is distinct from ffcs's stdin/stdout under Windows because each DLL owns
   // separate cin/cout objects).
 
@@ -116,10 +121,25 @@ static  void ffapi_newplot(){}
 #else
 
     // need #include <cstdio>
-    return popen(command,type);
+      FILE *f= popen(command,type);
+      if( f!=0) {
+//     ne marche pas car ffglut plante si flux  vide  
+//          int ppok=pclose(f);
+//	f =0;
+//	if(ppok==0)
+//	  f= popen(command,type);
+//	else
+//	  std::cerr << " fopen ok but plose bug ! =" << ppok <<  "\n";
+      }
+      if( f==0) { 
+	std::cerr << "Error: popen " << f << " " << command << " " << type << std::endl;
+	exit(1); }
+
+      return f;
 #endif
   }
 
+  // <<ffapi_ff_pclose>>
 static  int ffapi_ff_pclose(FILE *stream){
 #ifdef FFLANG
     // nothing to close in FFCS
@@ -234,7 +254,7 @@ static  void ffapi_winbinmode(FILE *f){
   }
 
 static  void ffapi_mpi_init(int &argc, char** &argv){
-    /// only call MPI_Init() if this has not already been done in ffcs/src/server.cpp
+    /// only call MPI_Init() if this has not already been done in [[file:~/ffcs/src/server.cpp]]
 #ifndef FFLANG
 #ifdef PARALLELE
     // need #include "mpi.h"
@@ -272,7 +292,8 @@ static  bool ffapi_protectedservermode(){
 #endif
   }
 
-void init(){
+  // <<init>> [[file:ffapi.hpp::init]] called by [[file:../lglib/mymain.cpp::ffapi::init]]
+  void init(){
     ffapi::cin = ffapi::ffapi_cin;
     ffapi::cout = ffapi::ffapi_cout;
     ffapi::cerr = ffapi::ffapi_cerr;
@@ -281,7 +302,7 @@ void init(){
     ffapi::ffstdin = ffapi::ffapi_ffstdin;
     ffapi::newplot = ffapi::ffapi_newplot;
     ffapi::ff_popen = ffapi::ffapi_ff_popen;
-    ffapi::ff_pclose = ffapi::ffapi_ff_pclose;
+    ffapi::ff_pclose = ffapi::ffapi_ff_pclose; // <<ff_pclose>>
     ffapi::fwriteinit = ffapi::ffapi_fwriteinit;
     ffapi::ff_fwrite = ffapi::ffapi_ff_fwrite;
     ffapi::ff_fflush = ffapi::ffapi_ff_fflush;
@@ -295,7 +316,6 @@ void init(){
   }
 }
 
-
 /// Local Variables:
 /// mode:c++
 /// ispell-local-dictionary:"british"
