@@ -59,6 +59,7 @@ void  mylex::Add(Key k,int i)
   Add(k,i,0); 
 }
 
+// <<mylex_Add_Key_aType>>
 void  mylex::Add(Key k,aType t)   
 {
   Check(!t,k,"type");
@@ -78,6 +79,9 @@ void  mylex::AddSpace(Key k,aType t)
   Add(k,FESPACE,t); 
 }
 
+// <<mylex_InMotClef>> looks in MotClef for the token contained in buf (buf is set by by [[mylex_basescan]]) and returns
+// its type as t and the grammar token id as r (eg [[file:~/ff/src/lglib/lg.ypp::TYPE]]).
+
 bool mylex::InMotClef  (aType & t, int & r) const {
   const_iterator i= MotClef.find(buf);
   if (i== MotClef.end()) {
@@ -89,11 +93,26 @@ bool mylex::InMotClef  (aType & t, int & r) const {
     ffassert(r);
     return true;}}
 
+// alh - 5/4/15 - <<mylex::InMotClef_string>> [[file:lex.hpp::InMotClef_string]] Same as [[mylex_InMotClef]], but checks
+// another buffer than buf
+bool mylex::InMotClef(const char *b,aType &t,int &r)const{
+  const_iterator i=MotClef.find(b);
+  if(i==MotClef.end()){t=0;r=ID;return false;}
+  else {
+    r=i->second.first;
+    t=i->second.second;
+    ffassert(r);
+    return true;
+  }
+}
+
+// <<mylex_Add_Key_int_aType>>
 void  mylex::Add(Key k,int r,aType t){ 
   iterator ii= MotClef.find(k);
   ffassert(ii==MotClef.end());
   MotClef.insert(pair<const Key,Value>(k,Value(r,t))); }
 
+// <<dump>>
 void mylex::dump(ostream & f ) 
 {
   const_iterator i=MotClef.begin(),e=MotClef.end();
@@ -184,6 +203,14 @@ int mylex::EatCommentAndSpace(string *data)
   } while (incomment);
     return (c==EOF) ? c : sep;
 }
+
+// <<mylex_basescan>>
+int mylex::basescanprint(int lvl)
+{
+    int r=basescan();
+    if (! lexdebug && echo && lvl==0 ) print(cout);
+    return r;
+}
 int mylex::basescan()
 {
   //  extern long mpirank;
@@ -201,7 +228,7 @@ int mylex::basescan()
       firsttime=false;
     if(echo) cout << setw(5) <<linenumber << " : " ; 
   }
-  EatCommentAndSpace(); 
+  EatCommentAndSpace(); // [[mylex::EatCommentAndSpace]]
   c =source().get(); // the current char 
   char nc = source().peek(); // next char
   buf[0]=c;
@@ -213,9 +240,11 @@ int mylex::basescan()
       //if (echo) cout << "ENDOFFILE "<< endl;
       if (close() )  goto debut; 
       buf[0]=0;
-      return ENDOFFILE;}
+      return ENDOFFILE;
+    }
+
+  // <<found_a_number>>
   else if (isdigit(c) || (c=='.' && isdigit(nc))) {
-    //  a number
     int i=1;
     buf[0]=c;
     bool real= (c=='.');
@@ -229,7 +258,7 @@ int mylex::basescan()
     if (nc =='+' || nc =='-' || isdigit(nc))  buf[i++]=source().get(),nc=source().peek();
     while ( isdigit(nc) && i< 50 ) buf[i++]=source().get(),nc=source().peek();
     }
-    if (i>= 50)  erreur("Number to long");
+    if (i>= 50) erreur("Number too long");
     
     buf[i]=0;
     if (nc=='i' ) 
@@ -242,6 +271,8 @@ int mylex::basescan()
     
     if(lexdebug) cout << " NUM : " << buf  << " "<<endl;
   }
+
+  // <<found_an_identifier>>
   else if (isalpha(c) || c=='\\')
     {
       ret =  ID;
@@ -249,11 +280,12 @@ int mylex::basescan()
       for (i = 1; i < 256 && isalnum(source().peek()); i++) 
         buf[i]=source().get();
       if (i == 256) 
-        erreur ("Identifyier too long");
+        erreur ("Identifier too long");
       buf[i] = 0;
-      
     }
-  else  if (c=='"')
+
+  // <<found_a_string>>
+  else if (c=='"')
     {
       int i;
       char cc,ccc;
@@ -293,6 +325,8 @@ int mylex::basescan()
       
       ret= STRING;
     }
+
+  // Anything else (eg brackets and operators)
   else 
     {
       ret = c;
@@ -384,16 +418,22 @@ int mylex::basescan()
   return ret;
 }
 
+// <<mylex_scan1>>
 int mylex::scan1()
 {
 
   // extern long mpirank;
  // bool echo = mpirank == 0; 
 
-  int ret= basescan();
+  int ret= basescan(); // [[mylex_basescan]]
   if(debugmacro)  cout << " scan1 " << ret << " " << token() << " " << ID << endl;
-    while ( ret == ID &&SetMacro(ret)); // correction mars 2014 FH
-    while ( ret == ID && CallMacro(ret)) ; // correction mars 2014 FH
+    while ( ret == ID  && (
+                              SetMacro(ret) // correction mars 2014 FH
+                          ||  CallMacro(ret) // correction mars 2014 FH
+                          || IFMacro(ret)
+                          )
+        )
+           ; // correction mars 2014 FH
     
   return ret;
 }    
@@ -404,12 +444,12 @@ int mylex::scan(int lvl)
  
   int ret= scan1(); 
   
-  // ID defined at [[file:../lglib/lg.ypp::token str ID]]
+  // ID defined at [[file:../lglib/lg.ypp::ID]] and detected at [[found_an_identifier]]
   if ( ret == ID) {
     if (! InMotClef(plglval->type,ret))  {
       int ft = FindType(buf);
 
-      // FESPACE, FESPACE1, FESPACE3 defined at [[file:../lglib/lg.ypp::token str FESPACE]]
+      // FESPACE, FESPACE1, FESPACE3 defined at [[file:../lglib/lg.ypp::FESPACE]]
       int feid3[4]  ={ ID,FESPACE1,FESPACE,FESPACE3};
 
       assert ( ft >= 0 && ft <= 3)  ;
@@ -429,8 +469,10 @@ int mylex::scan(int lvl)
 
 string mylex::token() const
   {
-   int i=-1;
-   string f;
+    int i=-1;
+    string f;
+
+    // found at [[found_a_string]]
     if (typetoken == STRING) 
       {
         f += '"';
@@ -447,7 +489,7 @@ string mylex::token() const
     return f;
   }
   
-  void mylex::print(ostream &f) const
+void mylex::print(ostream &f) const
   {
    int i=-1;
    int k=0;
@@ -471,12 +513,32 @@ string mylex::token() const
   } 
 
 char * mylex::match(int i) 
-{ if ( i != basescan() ) {// basescan -> scan1 change 2/2/2007  (non pas des substitution de parametres FH)
+{ if ( i != basescanprint() ) {// basescan -> scan1 change 2/2/2007  (non pas des substitution de parametres FH)
   cerr << " we waiting for a '" << char(i) <<"'" << endl;
   ErrorScan(" err macro ");};
  return buf; }
 
-
+bool mylex::AddMacro(string m,string def)
+{
+    bool ok=1;
+    char *macroname=newcopy(m.c_str());
+    int nbparam =0;
+    MacroData macroparm;
+    macroparm.f="";
+    macroparm.l=0;
+    macroparm.d.push_back(def);
+    MapMacroDef & MacroDef =listMacroDef->back();
+    MapMacroDef::const_iterator i=MacroDef.find(macroname);
+    if(verbosity>3) cout << " add macro "<< m << "->" << def << endl;
+    if ( i == MacroDef.end() )
+        MacroDef[macroname]=macroparm;
+    else {
+        cerr << " Error add macro varf: "<< m << "->" << def << " the macro exist"<< endl;
+        ErrorScan("Error in AddMacro parameters");
+        ok = false;
+    }
+    return ok;
+}
 bool mylex::SetMacro(int &ret)
 { 
   char endmacro[]="EndMacro";
@@ -484,18 +546,21 @@ bool mylex::SetMacro(int &ret)
     
   bool rt=false;
   int oldmacro=1;
-  if (strcmp(buf,"macro")==0 || (oldmacro=strcmp(buf,newmacro))==0 )
+  if (strncmp(buf,"macro",6)==0 || (oldmacro=strncmp(buf,newmacro,9))==0 )
     {
+      if(echo)  print(cout);//xxxxxx
       char *macroname=newcopy(match(ID));
+      //if(echo)  print(cout);
       int nbparam =0;
-      deque<string> macroparm;
-      int rr=basescan(); 
+      MacroData macroparm;
+   
+      int rr=basescanprint();
       string def;                          
       if (rr!='(') 
 	def += buf;
       else
 	{ // a '(' after macro name
-	  rr =  basescan();
+	  rr =  basescanprint();
 	  
 	  if (rr != ')' ) 
 	    do { 
@@ -507,29 +572,31 @@ bool mylex::SetMacro(int &ret)
 	      if (rr != ID) {
 		cerr <<" Erreur waiting of an ID: " << buf << " " << rr <<  endl;
 		erreur("in macro def: waiting for a ID"); }
-	      macroparm.push_back(buf);
-	      rr =  basescan(); 
+	      macroparm.d.push_back(buf);
+	      rr =  basescanprint();
 	      if (rr  == ')') break;          
 	      if ( rr != ',') erreur("in macro def: waiting  , or  ) "); 
-	      rr =  basescan();
+	      rr =  basescanprint();
 	      
 	    } while(1); 
 	}
       int kmacro=0;
       
-
+      macroparm.l = linenumber;// s
+      macroparm.f = file(); //
       do {
-	int lk=0;
+        int lk=0,nl=0;
 	string item;
 	int i = source().get();
 	if (i == EOF) {  cerr << "in macro " <<macroname <<  endl;
-	ErrorScan(" ENDOFFILE in macro definition. remark:a macro end with // ");}
+	ErrorScan(" ENDOFFILE in macro definition. remark:a macro end with // or EndMacro");}
 	int ii = source().peek();
-	if(isspace(i) && isalpha(ii) ) {
-	    def +=char(i);
+        if(isNLCR(source(),i)) { linenumber++; nl=1;}
+	else if(isalpha(i) && isalpha(ii) ) {//  Modif F.H
+	    //def +=char(i);
+            item = char(i);
 	    i = source().get();
-	    item = "";
-	    while(isalpha(i))
+            while(isalpha(i))
 	      {
 		item += char(i);
 		i = source().get();
@@ -537,9 +604,11 @@ bool mylex::SetMacro(int &ret)
 	    if( item == newmacro)  kmacro++;
 	    if( item == endmacro)  {
 		if (kmacro==0)  
-		    { source().putback(i); break;}
+		    { if(echo)cout <<item ;
+                      source().putback(i); break;}
 		kmacro--;
 	    }
+            if(echo) cout << item ;
 	    def += item;
 	    item ="";
 	    ii = source().peek();
@@ -550,12 +619,15 @@ bool mylex::SetMacro(int &ret)
 	    if (i == '/' && ii == '/') { source().putback('/'); break;} 
 	  }
 
-	def +=char(i);        
+	def +=char(i);
+          if(echo)cout << char(i) ;
+          if(echo && nl==1) cout << setw(5) <<linenumber << " # " ;
+
       } while(1);
-      macroparm.push_back(def);
-      if(echo) cout << "macro " << macroname  ;
-      for (size_t i=0;i<macroparm.size()-1;i++)
-	if(echo) cout << ( (i == 0) ? '(' : ',') << macroparm[i];
+      macroparm.d.push_back(def);
+     // if(echo) cout << "macro " << macroname  ;
+     // for (size_t i=0;i<macroparm.d.size()-1;i++)
+	// if(echo) cout << ( (i == 0) ? '(' : ',') << macroparm.d[i];
       if (nbparam)
 	if(echo) cout << " )  " ;
       for (size_t i=0;i<def.size();i++)
@@ -579,7 +651,125 @@ bool mylex::SetMacro(int &ret)
     }
   return rt;
 }
+std::string trim(std::string s, const char* t = " \t\n\r\f\v")
+{
+    s.erase(0, s.find_first_not_of(t));
+    s.erase(s.find_last_not_of(t) + 1);
+    return s;
+}
+bool mylex::IFMacro(int &ret)
+{// A faire !!!! F.H
+    bool rt=false;
+    const char *ifm="IFMACRO";
+    const char *ife="ENDIFMACRO";
+    int kif=0;
+    int isnot=0;
+    int lgifm=0;
+    if (strncmp(buf,ifm,8)==0  )
+    {
+        if (! lexdebug && echo  ) print(cout);
+        string val;
+        int rr=basescanprint();
+        if (rr!='(') {ErrorScan(" missing '(' after IFMACRO ");}
+         rr=basescanprint();
+        if( rr=='!') { isnot =1;rr=basescanprint();}
+        if (rr != ID)
+            cerr <<"IFMACRO: Erreur waiting of an ID: " << buf << " " << rr <<  endl;
+        string id =buf;
+        int withval=0;
+         rr=basescanprint();
+        if( rr == ',') {
+            while(1) {
+            int i = source().get();
+            if (i == EOF) {  cerr << "in IFMACRO " <<id <<  endl;
+                    ErrorScan(" ENDOFFILE in IFMACRO definition. remark:a end with ENDIFMACRO ");}
+            
+            if( char(i) ==')') break;
+            val +=char(i);
+            withval=1;
+              
+        }
+            if (! lexdebug && echo  ) cout << val;
+            source().putback(')');
+            rr=basescanprint();
+        }
+    
+        if (rr!=')') {ErrorScan(" missing ')' after IFMACRO(macro)  ");}
+        int kmacro=0;
+        if(debugmacro) cout << " IFMacro:: " << id << endl;
+        lgifm=linenumber;
+        string def;
+        do {
+            int lk=0,nl=0;
+            string item;
+            int i = source().get();
+           
+            if (i == EOF) {  cerr << "in IFMACRO " <<id <<  endl;
+                ErrorScan(" ENDOFFILE in IFMACRO definition. remark:a end with ENDIFMACRO ");}
+            int ii = source().peek();
+           //
+            if(isNLCR(source(),i)) { linenumber++; nl=1;}
+                 //
+            else if(isalpha(i) && isalpha(ii) ) {//  Modif F.H
+                //def +=char(i);
+                item = char(i);
+                i = source().get();
+                while(isalpha(i))
+                {
+                    item += char(i);
+                    i = source().get();
+                }
+                if( item == ifm)  kif++;
+                if( item == ife)  {
+                    if (kif==0)
+                    { source().putback(i); break;}
+                    kif--;
+                }
+                if(echo) cout << item ;
+                def += item;
+                item ="";
+               
+                ii = source().peek();
+            }
+           
+     
+           
+        def +=char(i);
+        if(echo)cout << char(i) ;
+        if(echo && nl==1) cout << setw(5) <<linenumber << " & " ;
+       } while(1);
+        
+         if (! lexdebug && echo  ) cout  << ife ;
+        bool exist=false;
+        MapMacroDef::const_iterator j;
+        for (list<MapMacroDef>::const_iterator i=listMacroDef->begin(); i != listMacroDef->end(); i++)
+        {
+            j=i->find(id.c_str());
+            if( j != i->end()) { exist =true; break;}
+        }
+        
+        if(withval && exist)
+        {
+            const MacroData  & macroparm= j->second;
+            if(macroparm.d.size()>0)
+            {
+              const string & mval = macroparm.d[macroparm.d.size()-1];
+             if(debugmacro)  cout << " check IFMACRO '"<< val << "' '"<< mval <<"'"<<endl;
+              exist = trim(mval) == trim(val);
+              rt=true;
+            }
+        }
+       if(debugmacro)  cout << "IFMacro def: " << def << "\n .. exist "<< exist  << " " << isnot << " "<< (exist == (isnot==0)) << " \n....\n";
 
+        if(exist == (isnot==0))
+         {
+          //string * ifmc= newstring(" ifmacro ");
+          input(def,new string(file()),lgifm);
+        }
+       ret =  scan1();
+    }
+ return rt;
+}
 
 bool mylex::CallMacro(int &ret)
 {
@@ -587,7 +777,7 @@ bool mylex::CallMacro(int &ret)
   // FH  jan 2005 
   // -----------------------------------------
 //  add Stringification,FILE, LINE  march 2014 FH..
-  if(strcmp(buf,"Stringification")==0)
+  if(strncmp(buf,"Stringification",16)==0)
   {
       if(debugmacro) cout <<"call Stringification : " << buf << endl;
 
@@ -618,20 +808,26 @@ bool mylex::CallMacro(int &ret)
           p += token();
           //if(echo) cout <<buf;
       }
-      
+      if(echo) cout << p << ")";
       plglval->str = newcopy(p.c_str());
       ret = STRING;
 
       return false;
   }
-  else if(strcmp(buf,"FILE")==0)
+
+  // <<FILE_macro>>
+  else if(strncmp(buf,"FILE",5)==0)
   {
+      if(echo) cout << buf  ;
       plglval->str = newcopy(filename() );
       ret = STRING;
      return false;
   }
-  else if(strcmp(buf,"LINE")==0)
+
+  // <<LINE_macro>>
+  else if(strncmp(buf,"LINE",5)==0)
   {
+    if(echo) cout << buf  ;
     plglval->lnum = linenumber;
     ret=LNUM;
     return false;
@@ -643,15 +839,18 @@ bool mylex::CallMacro(int &ret)
       
       if (j != i->end()) {
 	// int inmacroold=withmacropara;
+          const MacroData  & macroparm= j->second;
+          int nbparam= macroparm.d.size()-1;
 	  if(debugmacro) cout <<"call macro : " << buf << endl;
-	  string * macronn= new string(" macro: ");
+	  string * macronn= newstring(" macro: ");
 	  *macronn+= buf;
-	  
-	const deque<string>  & macroparm= j->second;
-	int nbparam= macroparm.size()-1;
+          *macronn+=" in ";
+          *macronn+= macroparm.f;
+	if(echo) cout << buf  ;
 	MapMacroParam  lp;
 	if (nbparam > 0 ) { 
 	  match('(');
+            
 	    for (int k=0;k<nbparam;k++)
 	      { 
 		  string p;
@@ -660,7 +859,7 @@ bool mylex::CallMacro(int &ret)
 		  int lvll=0;
 		  while (1) {
 		      int sep =  EatCommentAndSpace();
-		      int rr = basescan();// basescan -> scan1 change 2/2/2007  ( not change to pass macro name as a parameter)
+		      int rr = basescanprint();// basescan -> scan1 change 2/2/2007  ( not change to pass macro name as a parameter)
 		      if ( (rr=='}') && (--lvll==0) ) 
 			   continue; // remove first {
 		      else if ( (rr=='{') && (++lvll==1) )
@@ -687,14 +886,14 @@ bool mylex::CallMacro(int &ret)
 		      
 		  }
 		  if(debugmacro)
-		cout << "macro arg "<< k << " :" << macroparm[k] << " -> " <<  p << endl;
-	      lp.insert(pair<string,string>(macroparm[k],p));
+		cout << "macro arg "<< k << " :" << macroparm.d[k] << " -> " <<  p << endl;
+	      lp.insert(pair<string,string>(macroparm.d[k],p));
 	      //lp[macroparm[k]] = p; 
 	    }
 	}
 	if(debugmacro)
-	  cout <<   " input in : -> " << macroparm[nbparam]  << " <-> " << nbparam << endl;
-	input(macroparm[nbparam]);
+	  cout <<   " input in : -> " << macroparm.d[nbparam]  << " <-> " << nbparam << endl;
+	input(macroparm.d[nbparam],0,macroparm.l);//  no file no error here FH.
 	// ici il faut faire la substitution  de parameter 
 	// -----------------------------------------------
 	string expandtxt;
@@ -718,7 +917,7 @@ bool mylex::CallMacro(int &ret)
 	}
 	echo=echosave;
 	if(debugmacro) cout <<" (macro) eadin : " << expandtxt << endl;
-	input(expandtxt,macronn);
+	input(expandtxt,macronn,macroparm.l);
 	ret =  scan1(); // Correction FH 6/06/2004 of string parameter
 	return true;        
       }
@@ -728,42 +927,54 @@ bool mylex::CallMacro(int &ret)
 
 void  mylex::xxxx::open(mylex *lex,const char * ff) 
 {
-  //filename=ff;
   l=0;
   nf=f=0;
+
+  // Try to open the given file name without any extra path from [[file:lex.hpp::ffincludedir]]
   if(lex->ffincludedir.empty()) // if no liste 
-      nf=f= new ifstream(ff,ios_base::binary); //  modif of win32
+    nf=f= new ifstream(ff,ios_base::binary); //  modif of win32
+
+  // If it worked, set [[file:lex.hpp::filename]] to ff, otherwise try to add path elements from
+  // [[file:lex.hpp::ffincludedir]]
   if (!f || !*f)
-   {
+    {
    if ( f)  { delete f; nf=f=0; }
-   for (ICffincludedir k=lex->ffincludedir.begin();
-        k!=lex->ffincludedir.end();
-        ++k)
-   {
-    string dif_ff(*k+ff);
-    if (verbosity>=50) lex->cout  << "  --lex open :" << dif_ff << endl;
-    nf=f= new ifstream(dif_ff.c_str(),ios_base::binary); 
-    if ( f)  {
-      if ( f->good()) {  
-        filename = new string(dif_ff);
-        break;
-      }
-      delete f; nf=f=0;
-     }     
-   } 
-   } 
-   else
-      filename=new  string(ff);
-  if (!f || !*f) {
-    lex->cout << " Error openning file " <<ff<< " in: " <<endl;
+
+      // for every potential path element
       for (ICffincludedir k=lex->ffincludedir.begin();
 	   k!=lex->ffincludedir.end();
 	   ++k)
-	  lex->cout  << "  -- try  :\"" << *k+ff  << "\"\n";
+	{
+	  string dif_ff(*k+ff);
+	  if (verbosity>=50) lex->cout  << "  --lex open :" << dif_ff << endl;
+	  nf=f= new ifstream(dif_ff.c_str(),ios_base::binary); 
+	  if ( f)  {
 
-    lgerror("lex: Error input openning file ");};
+	    // If path works, set [[file:lex.hpp::filename]] and close test stream
+	    if ( f->good()) {  
+	      filename = newstring(dif_ff);
+	      break;
+	    }
+      delete f; nf=f=0;
+	  }     
+	} 
+    } 
+  else
+    filename=newstring(ff);
+
+  // Error message if no path was right
+  if (!f || !*f) {
+    lex->cout << " Error openning file " <<ff<< " in: " <<endl;
+    for (ICffincludedir k=lex->ffincludedir.begin();
+	 k!=lex->ffincludedir.end();
+	 ++k)
+      lex->cout  << "  -- try  :\"" << *k+ff  << "\"\n";
+
+    lgerror("lex: Error input openning file ");
+  }
 }
-void  mylex::xxxx::readin(mylex *lex,const string & s,const string *name, int macroargs)//,int nbparam,int bstackparam)
+
+void mylex::xxxx::readin(mylex *lex,const string & s,const string *name, int macroargs)
 {
   filename=name;
   macroarg=macroargs;
@@ -774,15 +985,22 @@ void  mylex::xxxx::readin(mylex *lex,const string & s,const string *name, int ma
     lex->cout << " Error readin string  :" <<s<< endl;
     if(f) delete f;
     nf = f =0;
-    lgerror("lex: Error readin macro ");};
+    lgerror("lex: Error readin macro ");
+  }
 }
 
 void mylex::xxxx::close() 
 { 
-  if( nf)  delete nf;
-  if (filename && (macroarg==0) ) delete filename;
+  // ALH_BUG Why does this segfault under Windows? Probably needs valgrind to find out.
+#if !defined(ENABLE_FFCS) || !defined(WIN32)
+  if(nf) delete nf;
+#endif
+  if(filename && (macroarg==0)) delete filename; // [[file:lex.hpp::filename]]
   f=nf=0;
 }
+
+// <<mylex_input_filename>>
+
 void mylex::input(const char *  filename) 
 {
   ffassert(level<99 && level >= -1);
@@ -796,7 +1014,9 @@ void mylex::input(const char *  filename)
   level++;      
 }
 
-void mylex::input(const string & str,const string * name) 
+// <<mylex_input_string>>
+
+void mylex::input(const string & str,const string * name,int lg)
 {
   
   ffassert(level<99 && level >= -1);
@@ -805,7 +1025,7 @@ void mylex::input(const string & str,const string * name)
     }
   
   pilesource[level+1].readin(this,str,name);
-  linenumber = 0;     
+  linenumber = lg;
   level++;
   
 }
@@ -815,7 +1035,7 @@ bool mylex::close() {
     cout << "\n close " << level ;
   ffassert(level >=0 && level < 100);
   // cout << "\n-- close " << level << endl;
-  pilesource[level].close(); 
+  pilesource[level].close(); // [[mylex::xxxx::close]]
   // cout << "\n ++   " << level << endl;
   if (--level<0)
     return false;
@@ -833,7 +1053,8 @@ bool mylex::close() {
    }
 }
 
- mylex::mylex(ostream & out,bool eecho):
+ mylex::mylex(ostream & out,bool eecho,const KN<String> *pargs)
+:
     linenumber(1),
     charnumber(0),
     ffincludedir(ffenvironment["includepath"]),
@@ -845,11 +1066,28 @@ bool mylex::close() {
     listMacroParam(0)
  {    
     listMacroDef->push_front(MapMacroDef());
+     if(pargs)
+     {
+         const KN<string> &args=*pargs;
+         for (int i=1;i<args.N();++i)
+         {
+             const string &a= args[i];
+             size_t id=a.find_first_of("-D");
+             if( id == 0)
+             {
+                 size_t ie=a.find_first_of("=");
+                 if( ie != std::string::npos && ie > 3)
+                     AddMacro(a.substr(2,ie-2),a.substr(ie+1,a.length()-ie-1));
+                
+             }
+             
+         }
+     }
    };
    
-mylex * Newlex(  ostream & out,bool eecho)
+mylex * Newlex(  ostream & out,bool eecho,KN<String> * args)
   {
-    return new mylex(out,eecho);
+    return new mylex(out,eecho,args);
   }
 void Destroylex(mylex * m)
  {
