@@ -31,7 +31,8 @@
 //  l'espace change 
 //   cf la fonction set qui reconstruit FESpace
 
-
+#ifndef lgfem_hpp_
+#define lgfem_hpp_
 extern Block *currentblock;
 
 void init_lgmat(); // initialisation for sparse mat functionnallity
@@ -100,7 +101,7 @@ namespace {
   lgElement(const Mesh * Th,Triangle * kk): pTh(Th),k(kk) {}
   operator int() const { Check(); return (* pTh)(k);} 
   lgVertex operator [](const long & i) const { Check(); return lgVertex(pTh,&(*k)[i]);}   
-  long lab() const {Check() ; return k ? k->lab : 0;}
+      long lab() const {Check() ; return k ? k->lab : LONG_MIN;}
   double area() const {Check() ; return k->area ;}
   long n() const { return k ? 3: 0 ;}
   bool operator==(const lgElement & l) const { return pTh==l.pTh && k == l.k;}
@@ -153,7 +154,7 @@ bool BuildPeriodic(
   const Mesh &Th,Stack stack,
   int & nbdfv, KN<int> & ndfv,int & nbdfe, KN<int> & ndfe);
 
-  
+// <<v_fes>> uses [[file:~/ff/src/femlib/RefCounter.hpp::RefCounter]]
 class v_fes : public RefCounter { 
 public:
   typedef ::pfes pfes;
@@ -326,7 +327,7 @@ public:
   }
 };
  
-template<class K,class v_fes> class FEbase;
+template<class K,class v_fes> class FEbase; // <<FEbase>>
 template<class K,class v_fes> 
 class FEcomp {
 public:
@@ -546,8 +547,8 @@ inline FESpace * v_fes::update() {
        KN<int> ndfv(Th.nv);
        KN<int> ndfe(Th.neb);
        int nbdfv,nbdfe;    
-       buildperiodic(stack,nbdfv,ndfv,nbdfe,ndfe);
-       return   buildupdate(nbdfv,ndfv,nbdfe,ndfe);
+       bool ret = buildperiodic(stack,nbdfv,ndfv,nbdfe,ndfe);
+       return   ret ? buildupdate(nbdfv,ndfv,nbdfe,ndfe) : buildupdate();
       }
      else 
        return  buildupdate();
@@ -563,8 +564,8 @@ inline FESpace3 * v_fes3::update() {
    //    KN<int> ndfv(Th.nv);
        KN<int> ndfe;
      //  int nbdfv,nbdfe;    
-	  buildperiodic(stack,ndfe);
-       return   buildupdate(ndfe);
+       bool ret = buildperiodic(stack,ndfe);
+       return   ret ? buildupdate(ndfe) : buildupdate();
       }
      else 
        return  buildupdate();
@@ -812,4 +813,82 @@ public:
     t0( map_type[typeid(A).name()] ),t1(map_type[typeid(B).name()] ){}
 };
 
+template<typename KK,typename vv_fes,typename CC>
+struct FFset3 {
+    typedef KK K;
+    typedef vv_fes v_fes;
+    typedef vv_fes *pfes;
+    typedef FEbase<K,vv_fes> ** R;
+    typedef R  A;
+    typedef pfes* B;
+    typedef CC C;
+    typedef Expression MC;
+    static Expression Clone(Expression e){return e;}
+    static bool Check(MC l) {return true;}
+    static void f(KN<K>   *x,MC a,A pp,Stack stack) {
+        CC at= GetAny<C>((*a)(stack));
+        *x = at;}
+};
+
+
+
+template<typename FF >
+class init_FE_eqarray : public OneOperator {
+public:
+    // <pferbase*,pferbase*,pfes*
+    typedef typename FF::K K;
+    typedef typename FF::v_fes v_fes;
+    typedef v_fes *pfes;
+    typedef typename v_fes::FESpace FESpace;
+    typedef FEbase<K,v_fes> ** R;
+    typedef R  A;
+    typedef pfes* B;
+    typedef typename FF::C C;
+    typedef typename FF::MC MC;
+    class CODE :public  E_F0{ public:
+        Expression a0,a1;
+        MC a2;
+        CODE(Expression aa0,Expression aa1,Expression aa2)
+        : a0(aa0),a1(aa1),a2(FF::Clone(aa2)) {ffassert(FF::Check(a2));}
+        AnyType operator()(Stack s)  const
+        {
+            
+            AnyType aa0= (*a0)(s);
+            AnyType aa1= (*a1)(s);
+           
+            A pp = GetAny<A>(aa0);
+            B a=  GetAny<B>(aa1);
+            *pp=new FEbase<K,v_fes>(a);
+            KN<K>   *x = (**pp).x();
+            if ( !x) {  // defined
+                FESpace * Vh= (*pp)->newVh();
+                throwassert( Vh);
+                **pp = x = new KN<K>(Vh->NbOfDF);
+                *x=K();
+            }
+            int n= x->N();
+            FF::f(x,a2,pp,s);
+            int nn= x->N();
+            ffassert(n==nn);
+            return  aa0;
+        }
+        virtual size_t nbitem() const {return a2->nbitem(); }
+        bool MeshIndependent() const {return a0->MeshIndependent() && a1->MeshIndependent()&& a2->MeshIndependent();} //
+        
+    };
+    init_FE_eqarray(int ppref):  OneOperator(
+                                             map_type[typeid(R).name()],
+                                             map_type[typeid(A).name()],
+                                             map_type[typeid(B).name()],
+                                             map_type[typeid(C).name()])
+    {pref=ppref;}
+    E_F0 * code(const basicAC_F0 & args) const
+    {
+        
+        return new CODE(args[0],args[1],map_type[typeid(C).name()]->CastTo(args[2]));
+    }
     
+};
+
+
+#endif
