@@ -80,7 +80,7 @@ class classBuildMesh :  public E_F0mps { public:
    typedef pmesh  Result;
 
    static basicAC_F0::name_and_type name_param[] ;
-   static const int n_name_param =3;
+   static const int n_name_param =5;
    
     Expression nargs[n_name_param];
    
@@ -88,6 +88,7 @@ class classBuildMesh :  public E_F0mps { public:
    
    long arg(int i,Stack stack,long a) const{ return nargs[i] ? GetAny<long>( (*nargs[i])(stack) ): a;}
    bool arg(int i,Stack stack,bool a) const{ return nargs[i] ? GetAny<bool>( (*nargs[i])(stack) ): a;}
+   double arg(int i,Stack stack,double a) const{ return nargs[i] ? GetAny<double>( (*nargs[i])(stack) ): a;}
    KNM<double>* arg(int i,Stack stack,KNM<double>* p) const{ return nargs[i] ? GetAny<KNM<double>*>( (*nargs[i])(stack) ): p;}
     
     classBuildMesh(const basicAC_F0 & args) 
@@ -194,8 +195,10 @@ class classBuildMeshArray :  public E_F0mps { public:
 
 basicAC_F0::name_and_type  classBuildMesh::name_param[]= {
     {  "nbvx", &typeid(long)} ,
-    {"fixeborder", &typeid(bool)},
-    {"points", &typeid(KNM<double>*)}
+    {"fixeborder", &typeid(bool)},// obsolete 
+    {"points", &typeid(KNM<double>*)},
+    {"fixedborder", &typeid(bool)},
+     {"alea", &typeid(double)}
 };
 // modif aout 2007
 class BuildMeshFile :  public E_F0mps { public:  
@@ -485,10 +488,12 @@ basicAC_F0::name_and_type Op_trunc_mesh::Op::name_param[Op_trunc_mesh::Op::n_nam
 AnyType classBuildMesh::operator()(Stack stack)  const { 
     const E_BorderN * borders = GetAny<const E_BorderN *>((*getborders)(stack));
    long  nbvx         = arg(0,stack,0L); 
-   bool  requireborder=  arg(1,stack,false);
+   bool  requireborder= arg(3,stack,arg(1,stack,false));
     KNM<double> * p=0;  p=arg(2,stack,p);
+    double alea = arg(4,stack,0.);
+
    ffassert(   nbvx >= 0);
-   return SetAny<pmesh>(Add2StackOfPtr2FreeRC(stack,BuildMesh(stack,borders,false,nbvx,requireborder,p)));
+   return SetAny<pmesh>(Add2StackOfPtr2FreeRC(stack,BuildMesh(stack,borders,false,nbvx,requireborder,p,alea)));
 
 }
 
@@ -508,7 +513,7 @@ AnyType Op_trunc_mesh::Op::operator()(Stack stack)  const {
     // trunc(trunc(Th,op1),op2) =trunc(trunc(Th,op2),op1) => no renumbering ....
     using namespace    Fem2D;
     const Mesh & Th = *GetAny<pmesh>((*getmesh)(stack));
-    long kkksplit =arg(0,stack,1L);
+    long kkksplit =std::max(1L, arg(0,stack,1L));
     long label =arg(1,stack,2L);
     KN<long> * pn2o =  arg(2,stack);
     KN<long> * po2n =  arg(3,stack);
@@ -725,7 +730,7 @@ AnyType Adaptation::operator()(Stack stack) const
   
 
   Real8 err         = arg(2,stack,0.01);    // coef in the metric
-  Real8  errg       = Min(arg(3,stack,0.01),err);
+  Real8  errg       = arg(3,stack,Min(0.01,err));// Modif FH 201217 
   long nbsx         = Max(100L,arg(4,stack,9000L));
   long nbsmooth     =  arg(5,stack,3L);
   long nbjacobi     =  arg(6,stack,0L) ;              // if increased will be more smooth
@@ -1940,8 +1945,34 @@ long Boundingbox(pmesh3 const& pTh,KN<double>* const& pb )
 { return  Boundingbox(pb,pTh);}
 long Boundingbox(pmesh const& pTh,KN<double>* const& pb )
 { return  Boundingbox(pb,pTh);}
-
-
+double Chi(Stack stack,pmesh const &pTh)
+{  // version 2d  oct 2017 FH.
+    if(pTh == 0) return 0.;
+    R2 PHat;
+    bool outside;
+    MeshPoint & mp = *MeshPointStack(stack);
+    if(pTh == mp.Th) return 1.;// point of mesh
+    const Triangle * K=pTh->Find(mp.P.p2(),PHat,outside);
+    if (!outside)
+        mp.set(*pTh,mp.P.p2(),PHat,*K,K->lab);
+    else return 0.;
+    return 1.;
+}
+double Chi(Stack stack,pmesh3 const &pTh)
+{ // version 3d oct 2017 FH.
+    if(pTh == 0) return 0.;
+    R3 PHat;
+    bool outside;
+    
+    MeshPoint & mp = *MeshPointStack(stack);
+    
+    if(pTh == mp.Th3) return 1.;
+    const Tet * K=pTh->Find(mp.P,PHat,outside);
+    if (!outside)
+        mp.set(*pTh,mp.P,PHat,*K,K->lab);
+    else return 0.;
+    return 1.;
+}
 
 void init_lgmesh() {
   if(verbosity&&(mpirank==0) )  cout <<"lg_mesh ";
@@ -1985,6 +2016,9 @@ void init_lgmesh() {
     Global.Add("boundingbox", "(", new OneOperator2_<long, KN<double>*, pmesh3>(Boundingbox));
     Global.Add("boundingbox", "(", new OneOperator2_<long, pmesh,KN<double>*>(Boundingbox));
     Global.Add("boundingbox", "(", new OneOperator2_<long, pmesh3,KN<double>*>(Boundingbox));
+    
+    Global.Add("chi", "(", new OneOperator1s_<double,pmesh,E_F_F0s_<double,pmesh,E_F0mps> >(Chi)); // oct 2017 FH function characteristic
+    Global.Add("chi", "(", new OneOperator1s_<double,pmesh3,E_F_F0s_<double,pmesh3,E_F0mps> >(Chi));// oct 2017 FH function characteristic
 
   TheOperators->Add("<-",
 		    new OneOperator2_<pmesh*,pmesh*,pmesh >(&set_copy_incr));
