@@ -373,6 +373,10 @@ bool BuildPeriodic(
 	      delete [] link2;
 	      return true; //new FESpace(**ppTh,*tef,nbdfv,ndfv,nbdfe,ndfe);
 	  }
+        else {
+	      delete [] link1;
+	      delete [] link2;
+        }
 	
     }
     return false;   
@@ -434,7 +438,7 @@ typedef typename Mesh::Element Element;
   GlgElement(const Mesh * Th,Element * kk): pTh(Th),k(kk) {}
   operator int() const { Check(); return (* pTh)(k);} 
   GlgVertex<Mesh> operator [](const long & i) const { Check(); return GlgVertex<Mesh>(pTh,&(*k)[i]);}   
-  long lab() const {Check() ; return k ? k->lab : 0;}
+  long lab() const {Check() ; return k ? k->lab : LONG_MIN;}
   double mes() const {Check() ; return k->mesure() ;}
   long n() const { return k ? Element::nv: 0 ;}
     
@@ -661,8 +665,10 @@ AnyType SaveMesh3::operator()(Stack stack) const
   
    pmesh3 Thh = GetAny<pmesh3>((*getmesh)(stack));
    string * fn =  GetAny<string*>((*filename)(stack));
-   cout << "SaveMesh3 " << *fn << " " << Thh << endl;
-   Thh->Save(*fn);
+   if (verbosity > 2)
+       cout << "SaveMesh3 " << *fn << " " << Thh << endl;
+   int ret=Thh->Save(*fn);
+    if( ret!=0) {ExecError("PB Write error !");}
    return SetAny<pmesh3>(Thh);
 
 }
@@ -769,7 +775,8 @@ AnyType MoveMesh3::operator()(Stack stack) const
 
 inline pmesh3 *  initMesh(pmesh3 * const & p, string * const & s) {
   Mesh3 * m;
-  cout << " initMesh " << *s << endl;
+  if(verbosity > 2)
+      cout << " initMesh " << *s << endl;
   *p= m =new Mesh3(*s); 
   m->BuildGTree();
  //  delete s;  modif mars 2006 auto del ptr 
@@ -1544,6 +1551,7 @@ void init_lgmesh3() {
    Add<pmesh3*>("mesure",".",new OneOperator1<double,pmesh3*>(pmesh_mes));
     Add<pmesh3*>("measure",".",new OneOperator1<double,pmesh3*>(pmesh_mes));
    Add<pmesh3*>("bordermesure",".",new OneOperator1<double,pmesh3*>(pmesh_mesb));
+    Add<pmesh3*>("bordermeasure",".",new OneOperator1<double,pmesh3*>(pmesh_mesb));
    Add<pmesh3*>("nt",".",new OneOperator1<long,pmesh3*>(pmesh_nt));
    Add<pmesh3*>("nv",".",new OneOperator1<long,pmesh3*>(pmesh_nv));
    Add<pmesh3*>("nbe",".",new OneOperator1<long,pmesh3*>(pmesh_nbe));
