@@ -25,6 +25,8 @@
  along with Freefem++; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
+#ifndef lgsolver_hpp_
+#define lgsolver_hpp_
 #include "gmres.hpp"
 typedef void *    pcommworld;
 
@@ -534,12 +536,15 @@ BuildSolverCG(DCL_ARG_SPARSE_SOLVER(R,A)  )
     { "commworld", &typeid(pcommworld)}, \
     { "master", &typeid(long)}, \
     { "rinfo", &typeid(KN<double>*)}, \
-    { "info", &typeid(KN<long>*)} \
+    { "info", &typeid(KN<long>*)}, \
+    { "kerneln", &typeid(  KNM<double> *)}, \
+    { "kernelt", &typeid(  KNM<double> *)}, \
+    { "kerneldim", &typeid(long*)} \
 
 
 
 
-const int NB_NAME_PARM_MAT =  24  ;
+const int NB_NAME_PARM_MAT =  24 +3  ;
     
     
 template<class R>
@@ -581,6 +586,11 @@ inline void SetEnd_Data_Sparse_Solver(Stack stack,Data_Sparse_Solver & ds,Expres
         // add FH nov 2015 ..
         if (nargs[++kk]) ds.rinfo = GetAny<KN<double>* >((*nargs[kk])(stack));
         if (nargs[++kk]) ds.info = GetAny<KN<long>* >((*nargs[kk])(stack));
-	assert(++kk == n_name_param);
+         // add FH juin 2018   ..
+        if (nargs[++kk]) ds.kerneln = GetAny< KNM<double>* >((*nargs[kk])(stack));
+        if (nargs[++kk]) ds.kernelt = GetAny< KNM<double>* >((*nargs[kk])(stack));
+        if (nargs[++kk]) ds.kerneldim = GetAny<long * >((*nargs[kk])(stack));
+        ffassert(++kk == n_name_param);
     }
 } // end of namespace Fem2D
+#endif
