@@ -44,17 +44,19 @@ namespace  Fem2D {
 
 template<class K> class SolveGCDiag; 
 class Plot;
-class v_fes;
+class v_fes; // [[file:lgfem.hpp::v_fes]]
 
-typedef FEbase<double,v_fes> * pferbase ;
-typedef FEbaseArray<double,v_fes> * pferbasearray ;
-typedef pair<pferbase,int> pfer ;
-typedef pair<pferbasearray,int> pferarray ;
+// real [[file:lgfem.hpp::FEbase]] using [[file:lgfem.hpp::v_fes]]
+typedef FEbase<double,v_fes> * pferbase ;		// <<pferbase>>
+typedef FEbaseArray<double,v_fes> * pferbasearray ;	// <<pferbasearray>>
+typedef pair<pferbase,int> pfer ;			// <<pfer>>
+typedef pair<pferbasearray,int> pferarray ;		// <<pferarray>>
 
-typedef FEbase<Complex,v_fes> * pfecbase ;
-typedef FEbaseArray<Complex,v_fes> * pfecbasearray ;
-typedef pair<pfecbase,int> pfec ;
-typedef pair<pfecbasearray,int> pfecarray ;
+// complex [[file:lgfem.hpp::FEbase]] using [[file:lgfem.hpp::v_fes]]
+typedef FEbase<Complex,v_fes> * pfecbase ;		// <<pfecbase>>
+typedef FEbaseArray<Complex,v_fes> * pfecbasearray ;	// <<pfecbasearray>>
+typedef pair<pfecbase,int> pfec ;			// <<pfec>>
+typedef pair<pfecbasearray,int> pfecarray ;		// <<pfecarray>>
 
 
 //typedef pair<pmesh *,int> pmesharray ;
@@ -126,6 +128,7 @@ class C_args: public E_F0mps  {public:
           {
             const C_args * a = dynamic_cast<const C_args *>(args[i].LeftValue());
             for (list<C_F0>::const_iterator i=a->largs.begin();i!=a->largs.end();i++)
+              if( ! i->Zero()) // skip Zero term
               largs.push_back(*i);                   
           } 
         else 
@@ -136,7 +139,7 @@ class C_args: public E_F0mps  {public:
   operator aType () const { return atype<const C_args *>();}         
   
   static  E_F0 * f(const basicAC_F0 & args) { return new C_args(args);}
-  bool Zero() { return !largs.empty();}
+  bool Zero()  const { return largs.empty();} // BIG WARNING April and wrong functon FH v 3.60 .......
   bool IsLinearOperator() const;
   bool IsBilinearOperator() const;
 };
@@ -271,7 +274,7 @@ class BC_set : public E_F0mps { public:
 
 class CDomainOfIntegration: public E_F0mps { 
 public:
-  static const int n_name_param =10;
+  static const int n_name_param =12;
   static basicAC_F0::name_and_type name_param[] ;
   Expression nargs[n_name_param];
   enum typeofkind  { int2d=0, int1d=1, intalledges=2,intallVFedges=3, int3d = 4, intallfaces= 5,intallVFfaces=6 } ; //3d
@@ -279,12 +282,15 @@ public:
   int d; // 3d
    typedef const CDomainOfIntegration* Result;
   Expression Th;
+  Expression mapt[3],mapu[3];
   vector<Expression> what;
   vector<int> whatis; // 0 -> long , 1 -> array ??? 
   CDomainOfIntegration( const basicAC_F0 & args,typeofkind b=int2d,int ddim=2) // 3d
     :kind(b),d(ddim), Th(0), what(args.size()-1),whatis(args.size()-1)
      
   {
+    mapt[0]=mapt[1]=mapt[2]=0; // no map of intergration points for test function
+    mapu[0]=mapu[1]=mapu[2]=0; // no map of intergration points for unknows function
     args.SetNameParam(n_name_param,name_param,nargs);
     if(d==2) // 3d
       Th=CastTo<pmesh>(args[0]);
@@ -304,7 +310,22 @@ public:
 	   whatis[i-1]=1;
 	   what[i-1]=CastTo<KN_<long> >(args[i]); 	   
 	 }
-    // cout << " CDomainOfIntegration " << this << endl;       
+      const E_Array *pmapt = dynamic_cast<E_Array *>(nargs[10]);
+      const E_Array *pmapu = dynamic_cast<E_Array *>(nargs[11]);
+      if(pmapt )
+      {
+          if( pmapt->size() != d ) ErrorCompile("mapt bad arry size ",1);
+          for(int i=0; i<d; ++i)
+              mapt[i]=CastTo<double >((*pmapt)[i]);
+      }
+      if(pmapu )
+      {
+          if( pmapu->size() != d ) ErrorCompile("mapu bad arry size ",1);
+          for(int i=0; i<d; ++i)
+              mapu[i]=CastTo<double >((*pmapu)[i]);
+         
+      }
+    // cout << " CDomainOfIntegration " << this << endl;
   }
   static  ArrayOfaType  typeargs() {  return ArrayOfaType(atype<pmesh>(), true);} // all type
   AnyType operator()(Stack ) const  { return SetAny<const CDomainOfIntegration *>(this);}
@@ -319,7 +340,7 @@ public:
   bool intmortar(Stack s) const { return nargs[7] ? GetAny<bool>( (*(nargs[7])) (s) )  : 1;} // truc  pour 
   double levelset(Stack s) const { return nargs[9] ? GetAny<double>( (*(nargs[9]))(s) )  : 0;}
   bool  islevelset() const { return nargs[9] != 0; }
-    
+  bool withmap() const {return mapu[0] || mapt[0]; }
 };  
 
 class CDomainOfIntegrationBorder: public CDomainOfIntegration { 
@@ -555,7 +576,7 @@ class FormLinear : public E_F0mps { public:
 };
 
 template<class VFES>
-class Call_FormLinear: public E_F0mps 
+class Call_FormLinear: public E_F0mps
 { 
 public:
   const int d;
@@ -1046,7 +1067,7 @@ AnyType OpArraytoLinearForm<R,v_fes>::Op::operator()(Stack stack)  const
   pfes  &  pp= *GetAny<pfes * >((*l->ppfes)(stack));
   FESpace * pVh = *pp ;
   FESpace & Vh = *pVh ;
-  double tgv= 1e30;
+  double tgv= ff_tgv;
   if (l->nargs[0]) tgv= GetAny<double>((*l->nargs[0])(stack));  
   long NbOfDF =  pVh ? Vh.NbOfDF: 0;
   KN<R> *px=0;
@@ -1072,6 +1093,30 @@ AnyType OpArraytoLinearForm<R,v_fes>::Op::operator()(Stack stack)  const
   return SetAny<KN_<R> >(xx);
 }
 
+template<typename KK,typename vv_fes,typename CC>
+struct FF_L_args {
+    typedef  KK K;
+    typedef vv_fes v_fes;
+    typedef v_fes *pfes;
+    typedef typename  v_fes::FESpace FESpace;
+    typedef FEbase<K,v_fes> ** R;
+    typedef R  A;
+    typedef pfes* B;
+    typedef const CC *  C;
+    typedef  CC *  MC;
+    static bool Check(MC l) {return IsComplexType<K>::value==FieldOfForm(l->largs,IsComplexType<K>::value);}
+    static MC  Clone(Expression ll){C l = dynamic_cast<C>(ll);ffassert(l);return new CC(*l); }
+    static void f(KN<K>   *x,MC l,A pp,Stack stack)
+    {
+        ffassert(l);
+        double tgv=ff_tgv;
+        if (l->nargs[0]) tgv= GetAny<double>((*l->nargs[0])(stack));
+        FESpace * pVh= (*pp)->newVh();
+        KN_<K>  xx=*x;
+        if (  pVh && AssembleVarForm<K,MatriceCreuse<K>,FESpace >(stack,pVh->Th,*pVh,*pVh,false,0,&xx,l->largs) )
+            AssembleBC<K,FESpace>(stack,pVh->Th,*pVh,*pVh,false,0,&xx,0,l->largs,tgv);
+    }
+};
 template<class R>
 void SetSolver(Stack stack,bool VF,MatriceCreuse<R> & A, Data_Sparse_Solver & ds) 
 	       /*Stack stack,MatriceCreuse<R> & A,const TypeSolveMat *typemat,bool VF,double eps,int NbSpace,int itmax,
@@ -1170,7 +1215,7 @@ AnyType OpMatrixtoBilinearForm<R,v_fes>::Op::operator()(Stack stack)  const
     long NbSpace = 50; 
   long itmax=0; 
   double epsilon=1e-6;
-  double tgv = 1e30;
+  double tgv = ff_tgv;
   int strategy=0;
   double tol_pivot=-1;
   double tol_pivot_sym=-1;
