@@ -1,5 +1,5 @@
 #include "config-wrapper.h"
-#include "strversionnumber.hpp"
+#include "strversionnumber.hpp" // [[file:strversionnumber.hpp]]
 #include <cstdlib>
 using namespace std;
 #define TOSTRING1(i) #i
@@ -18,6 +18,6 @@ string StrVersionNumber(){
 //  buffer.precision(8);
 //  buffer<<VersionNumber();
   static char buffer[100];
-  sprintf(buffer," %9f (date Ven  3 jui 2016 12:57:03 CEST)",VersionNumber());
-  return buffer; //.str()+" (date Ven  3 jui 2016 12:57:03 CEST)" ;
+  sprintf(buffer," %9f (date Jeu 12 jul 2018 12:05:08 CEST)",VersionNumber());
+  return buffer; //.str()+" (date Jeu 12 jul 2018 12:05:08 CEST)" ;
 }
