@@ -49,18 +49,18 @@ include 'phpfiles.php';
 ?>
 
 <p class="p1"><span class="s1">How to compile FreeFem++ under Linux.<span class="Apple-converted-space"> </span></span></p>
-<p class="p2"><span class="s1"><i>in tested FH (08/03/2016) on Ubuntu 14.04.4 LTS (form scatch) .</i></span></p>
+<p class="p2"><span class="s1"><i>in tested FH (04/07/2017) on Ubuntu 16.04.4 LTS (form scatch) .</i></span></p>
 <p class="p3"><span class="s1"><br>
 </span></p>
 <p class="p5"><span class="s1"> Before the build process check the installed software <A HREF="linux.php#before">   (here) </A>  </p> 
 <p class="p5"><span class="s1">First download the tar.gz file contening source from  
     <? ppfile($sfile,"Source code") ?> </p>
  <p class="p5"><span class="s1"> or you can get the latest source from
-an anonymous <A href="http://mercurial.selenic.com/" > Mercurial SCM </A> copy with the following unix shell commands&nbsp;:
+an anonymous <A href="https://git-scm.com/downloads" > git SCM </A> copy with the following unix shell commands&nbsp;:
 </span></p>
-<p class="p5"><span class="s3"><b>hg clone http://www.freefem.org/ff++/ff++<br>
+<p class="p5"><span class="s3"><b>git clone https://github.com/FreeFem/FreeFem-sources ff++<br>
 </b></span><span class="s4"><br>
-i</span><span class="s5">f your version of automake &gt;= 1.13 , autoconf &gt;= 2.69 do <br>
+</span><span class="s5">if your version of automake &gt;= 1.13 , autoconf &gt;= 2.69 do <br>
 </span><span class="s3"><b>autoreconf -i </b></span><span class="s1"><br>
 </span><span class="s5">else it's  too old, then do <br>
 </span><span class="s3"><b>tar zxvf AutoGeneratedFile.tar.gz<span class="Apple-converted-space"> </span></b></span></p>
@@ -76,86 +76,59 @@ i</span><span class="s5">f your version of automake &gt;= 1.13 , autoconf &gt;=
 <p class="p7"><span class="s6"><b><span class="Apple-converted-space"> </span></b></span><span class="s1"> <span class="Apple-converted-space"> </span></span></p>
 <p class="p3"><span class="s7"><span class="Apple-converted-space"> </span></span><span class="s1">To compile with lot of additionnal software.</span></p>
 <p class="p6"><span class="s8"><br>
-</span><span class="s1"><b>./configure --enable-download<span class="Apple-converted-space"> </span></b></span></p>
+</span><span class="s1"><b>./configure --enable-download  --disable-pastix <span class="Apple-converted-space"> </span></b></span></p>
 <p class="p6"><span class="s1"><b><span class="Apple-converted-space">  </span>make<span class="Apple-converted-space"> </span></b></span></p>
 <p class="p6"><span class="s1"><b><span class="Apple-converted-space">  </span>make check<span class="Apple-converted-space">    </span>(to test de version)</b></span></p>
 <p class="p6"><span class="s1"><b><span class="Apple-converted-space">  </span>make install<span class="Apple-converted-space">  </span>(under root)</b></span></p>
 <p class="p8"><span class="s1"><b></b></span><br></p>
 <p class="p7"><span class="s1"></span><br></p>
- <A NAME="before">   
+
+
+<p class="p12"><span class="s1">To configure <span class="Apple-converted-space">  </span>with petsc and hpddm (need<span class="Apple-converted-space">  </span>c++ 11 features )</span></p>
+<p class="p13"><span class="s9"></span><br></p>
+<p class="p3"><span class="s1">1)  install  git and cmake</p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>sudo apt-get install git cmake</b></span></p>
+<p class="p3"><span class="s1">2)  configure to def on compiler, mpi , ...</p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>./configure --enable-download  --disable-pastix .....</b></span></p>
+<p class="p3"><span class="s1">3)  download and compile petsc, slepc in real and complex, ...</p>
+<p class="p3"><span class="s1">  ...  Warning: Check that you can write in  directory : path_install/ff-petsc</p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>cd download/ff-petsc</b></span></p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>make petsc-slepc SUDO=sudo (if need root access to install dir)</b></span></p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>make petsc-slepc</b></span></p>
+<p class="p9"><span class="s9"><i>.... wait , ... wait , wait ...</i></span></p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>cd -</b></span></p>
+<p class="p3"><span class="s1">4)  reconfigure with  petsc, slepc in real and complex, ...</p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>./reconfigure <b></span></p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>make <b></span></p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>make check <b></span></p>
+<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>make install <b>(under root)</span></p>
+<p class="p12"></p>
+<p class="p12"></p>
+ <A NAME="before all install ">   
 <p class="p3"><span class="s1">Before you have to install some usefull package with apt-get under Debian / Ubuntu.<span class="Apple-converted-space"> </span></span></p>
 <p class="p3"><span class="s1"><span class="Apple-converted-space"> </span>you can install this package <span class="Apple-converted-space">  </span>software</span></p>
 <p> </p>
 <p class="p9"><span class="s9"><b>sudo apt-get install cpp  freeglut3-dev g++ gcc  gfortran</b></span></p>
 <p class="p9"><span class="s9"><b>sudo apt-get install<span class="Apple-converted-space">  </span>ghostscript  m4 make  patch pkg-config wget python</b></span></p>
 <p class="p3"><span class="s9"> # other lib of a full freefem++ can be change </span></p>
-<p class="p9"><span class="s9"><b>sudo apt-get install<span class="Apple-converted-space">  </span>libopenblas-dev liblapack-dev libhdf5-dev libgsl0-dev</b></span></p>
+<p class="p9"><span class="s9"><b>sudo apt-get install<span class="Apple-converted-space">  </span>libopenblas-dev liblapack-dev libhdf5-dev libgsl2-dev</b></span> </p>
+<p class="p3"><span class="s9"> # Remark:  On old version pkg  "libgsl2-dev" is  "libgsl0-dev"</span></p>
+<p class="p3"><span class="s9"> # Remark:  the next package can be download automaticaly by FreeFem++"</span></p>
+
 <p class="p9"><span class="s9"><b>sudo apt-get install<span class="Apple-converted-space">  </span> libscotch-dev  libfftw3-dev   libarpack2-dev libsuitesparse-dev  </b></span></p>
 <p class="p9"><span class="s9"><b>sudo apt-get install<span class="Apple-converted-space">  </span> libmumps-seq-dev  libnlopt-dev  coinor-libipopt-dev  libgmm++-dev libtet1.5-dev </b></span></p>
 <p class="p3"><span class="s9"> # Remark:  superlu-dev  package is too old on ubuntu  Trusty (14.0.4) </span></p>
-<p class="p3"><span class="s9"> # not mandatory package  but usefull for develloppenemt </span></p>
+<p class="p3"><span class="s9"> # not mandatory package  but usefull for developpement </span></p>
 <p class="p9"><span class="s9"><b>sudo apt-get install autoconf automake autotools-dev  bison  flex gdb  valgrind </b></span></p>
-<p class="p3"><span class="s9"> # not mandatory package for downlaod with hg / mercurail tool</span></p>
-<p class="p9"><span class="s9"><b>sudo apt-get install mercurial </b></span></p>
+<p class="p3"><span class="s9"> # not mandatory package for download with git  form https://git-scm.com/downloads </span></p>
+<p class="p9"><span class="s9"><b>sudo apt-get install git </b></span></p>
 <p class="p9"><span class="s9"><b>sudo apt-get install gnuplot-qt </b></span></p>
 
 <p class="p10"><span class="s9"></span><br></p>
 <p class="p11"><span class="s9"># install de mpich for parallel version <span class="Apple-converted-space"> </span></span></p>
 <p class="p9"><span class="s9"><b>sudo apt-get install<span class="Apple-converted-space">  </span>mpich</b></span></p>
 <p class="p10"><span class="s9"></span><br></p>
-<p class="p12"><span class="s1">To configure <span class="Apple-converted-space">  </span>with petsc and hpddm (need<span class="Apple-converted-space">  </span>c++ 11 features )</span></p>
-<p class="p13"><span class="s9"></span><br></p>
-<p class="p11"><span class="s9"># compilation de PETSc<span class="Apple-converted-space"> </span></span></p>
-<p class="p14"><span class="s9"><b>wget http://ftp.mcs.anl.gov/pub/petsc/release-snapshots/petsc-lite-3.6.3.tar.gz</b></span></p>
-<p class="p14"><span class="s9"><b>tar zxvf petsc-lite-3.6.3.tar.gz</b></span></p>
-<p class="p14"><span class="s9"><b>cd petsc-3.6.3</b></span></p>
-<p class="p11"><span class="s9">#build the python, file ff-conf.py<span class="Apple-converted-space">  </span>python for configuration of PETSc<span class="Apple-converted-space"> </span></span></p>
-<p class="p9"><span class="s9"><b>cat &gt;ff-conf.py &lt;&lt;EOF</b></span></p>
-<p class="p9"><span class="s9"><b>#!/usr/bin/python</b></span></p>
-<p class="p9"><span class="s9"><b>if __name__ == '__main__':</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>import sys</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>import os</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>sys.path.insert(0, os.path.abspath('config'))</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>import configure</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>configure_options = [</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--CFLAGS=-O2',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--COPTFLAGS=-O3',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--CXXFLAGS=-O2 -std=c++11',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--CXXOPTFLAGS=-O3',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--FFLAGS=-O2',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--FOPTFLAGS=-O3',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-fftw',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-hypre',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-metis',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-ml',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-mumps',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-parmetis',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-pastix',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-ptscotch',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-scalapack',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-suitesparse',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--download-superlu',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--prefix=(The dir install petsc)',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'--with-blas-lapack-lib=-llapack -lblas',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">    </span>'PETSC_ARCH=arch-ff++',</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>]</b></span></p>
-<p class="p9"><span class="s9"><b><span class="Apple-converted-space">  </span>configure.petsc_configure(configure_options)</b></span></p>
-<p class="p9"><span class="s9"><b>EOF</b></span></p>
-<p class="p9"><span class="s9"><b>chmod a+x ff-conf.py</b></span></p>
-<p class="p15"><span class="s9"># launch the petsc configure and follow the instruction<span class="Apple-converted-space"> </span></span></p>
-<p class="p14"><span class="s9"><b>./ff-conf.py</b></span></p>
-<p class="p14"><span class="s9"><b>make PETSC_DIR=(The dir install petsc) PETSC_ARCH=arch-ff++ all</b></span></p>
-<p class="p14"><span class="s9"><b>sudo make PETSC_DIR=(The dir install petsc) PETSC_ARCH=arch-ff++ install</b></span></p>
-<p class="p3"><span class="s7"><br>
-</span><span class="s1">Now you can reconfigure freefem++ with petsc (need<span class="Apple-converted-space">  </span>c++ 11 features )</span></p>
-<p class="p3"><span class="s1"><br>
-<span class="Apple-converted-space">  </span>Go in freefem++ directory.<span class="Apple-converted-space"> </span></span></p>
-<p class="p16"><span class="s1"><br>
-</span></p>
-<p class="p14"><span class="s10"><b><span class="Apple-converted-space"> </span></b></span><span class="s11"><b>./configure --enable-download </b></span><span class="s9"><b>--with-petsc=/usr/local/ff++/petsc/lib/petsc/conf/petscvariables</b></span></p>
-<p class="p17"><span class="s1"><b><span class="Apple-converted-space">  </span>make<span class="Apple-converted-space"> </span></b></span></p>
-<p class="p17"><span class="s1"><b><span class="Apple-converted-space">  </span>make check<span class="Apple-converted-space">    </span>(to test de version)</b></span></p>
-<p class="p17"><span class="s1"><b><span class="Apple-converted-space">  </span>make install<span class="Apple-converted-space">  </span>(under root)</b></span></p>
+
 <p class="p18"><span class="s1"><br>
 </span></p>
 <p class="p16"><span class="s1"><br>
@@ -257,10 +230,12 @@ i</span><span class="s5">f your version of automake &gt;= 1.13 , autoconf &gt;=
 <p class="p7"><span class="s1"><span class="Apple-converted-space"> </span></span></p>
 <p class="p16"><span class="s1"><br>
 </span></p>
-<p class="p20"><span class="s1"><b><span class="Apple-converted-space"> </span>./configure --enable-download<span class="Apple-converted-space">  </span>--with-mpi=mpic++</b></span></p>
+<p class="p20"><span class="s1"><b><span class="Apple-converted-space"> </span>./configure --enable-download  --disable-pastix <span class="Apple-converted-space">  </span>--with-mpi=mpic++</b></span></p>
 <p class="p20"><span class="s1"><b>#<span class="Apple-converted-space">  </span>utility of parameter:</b></span></p>
 <p class="p20"><span class="s1"><b>#<span class="Apple-converted-space">  </span>--enable-download :<span class="Apple-converted-space">  </span>for fft and tetgen ...<span class="Apple-converted-space"> </span></b></span></p>
-<p class="p20"><span class="s1"><b>#<span class="Apple-converted-space">  </span>--with-mpi=mpic++ : for mpi version<span class="Apple-converted-space"> </span></b></span></p>
+<p class="p20"><span class="s1"><b>#<span class="Apple-converted-space">  </span>--with-mpi=mpic++ : for mpi version<span 
+<p class="p20"><span class="s1"><b>#<span class="Apple-converted-space">  </span>--disable-pastix : for mpi version<span 
+class="Apple-converted-space"> </span></b></span></p>
 <p class="p20"><span class="s1"><b><span class="Apple-converted-space">  </span>make<span class="Apple-converted-space"> </span></b></span></p>
 <p class="p16"><span class="s12"><b><span class="Apple-converted-space">  </span>make check<span class="Apple-converted-space">    </span></b></span><span class="s1">(to test de version)</span></p>
 <p class="p20"><span class="s1"><b><span class="Apple-converted-space">  </span>make install<span class="Apple-converted-space"> </span></b> </span><span class="s8">(under root)</span></p>
