@@ -43,32 +43,34 @@ include 'phpfiles.php';
 ?>
 
 <p class="p1"><b>Installation from a pkg<span class="Apple-converted-space">  </span>file is standard,</b></p>
-<p class="p1"><b>Remark </b>on version 10.10, 10.9 and 10.8 to authorise the install from everywhere <a href="http://maypalo.com/2013/1
+<br>
+<p class="p1">   on precompile version FreeFem++-3.60-MacOS_10.13.pkg to use petsc/slepc 
+        you need to install XQuartz 
+        form <a href="https://dl.bintray.com/xquartz/downloads/XQuartz-2.7.11.dmg"> XQuartz-2.7.11.dmg </a></p>
+<p class="p1"><br><b>Remark </b>on version 10.10, 10.9 and 10.8 to authorise the install from everywhere <a href="http://maypalo.com/2013/1
 0/28/install-non-approved-apps-mavericks-10-9/"> (see System Preferences > Security & Privacy > General > Allow applications dow
 nloaded from > Anywhere)<A> 
 
 
-<p class="p1">Take the file from and download </p> 
-	<? pfile($m11file,"MacOS 10.11 (with mpich), ifort MKL     ") ?>		
-	<? pfile($m10file,"MacOS 10.10 (with mpich), ifort MKL    ") ?>		
+<br><p class="p1">Take the file from and download </p> 
+	<? pfile($m13file,"MacOS 10.13 (with openmpi,petc), ifort MKL     ") ?>		
+	<? pfile($m12file,"MacOS 10.12 (with openmpi,petc), ifort MKL     ") ?>		
+	<? pfile($m11file,"MacOS 10.11 (with openmpi,petc), ifort MKL     ") ?>		
+	<? pfile($m10file,"MacOS 10.10 (with openmpi,petc), ifort MKL    ") ?>		
 	<? pfile($m9file,"MacOS 10.9 (with mpich), ifort MKL   ") ?>		
 	<? pfile($m8file,"MacOS 10.8") ?>		
 	<? pfile($m7file,"MacOS 10.7") ?>		
 	<? pfile($m6file,"MacOS 10.6") ?>	
 	<? pfile($mfile,"MacOS 10.4") ?>	
-<p></p>
+<p class="p1"><br></p>
+<p class="p1">The <tt>textmate 2</tt> editor is a good editor for  Mac 10.7 or better, <a href="https://macromates.com/download"> download  from macromates.com </a> and install it, when  get the 
+  <a href="http://www.freefem.org/ff++/Textmate2-ff++.zip"> textmate freefem++ syntax from www.freefem.org/ff++/Textmate2-ff++.zip (version nov.  2107)</a> unzip Textmate2-ff++.zip and follow the explanation given in file \texttt{How\_To.rtf}.</p>
+<p class="p1"><br></p>
 
 <p class="p1"><span class="s1"> Please read the <a href="README_MAC.txt"> README_MAC</a> form the distribution for the full infomation </span></p>
 <p class="p1"><span class="s1">Take the file from and download<span class="Apple-converted-space"> </span></span></p>
 <p class="p3"><span class="s1"><br>
 </span></p>
-<p class="p1"><span class="s1">Just follow the install, openmpi or mpich<span class="Apple-converted-space">  </span>is in the package for version 10.6 and 10.7 the version is compile with g++ v 4.9 (http://prdownloads.sourceforge.net/hpc/gfortran-4.9-bin.tar.gz?download)</span></p>
-<p class="p1"><span class="s1">take <a href="http://hpc.sourceforge.net/"><span class="s3">hpc.sourceforge.net </span></a>remarkversion.<span class="Apple-converted-space"> </span></span></p>
-<p class="p1"><span class="s1">to install do:<span class="Apple-converted-space"> </span></span></p>
-<p class="p4"><span class="s1"><span class="Apple-converted-space"> </span></span></p>
-<p class="p5"><span class="s1"><span class="Apple-converted-space"> </span># sudo tar zxvf gfortran-4.9-bin.tar.gz -C /</span></p>
-<p class="p1"><span class="s1"><br>
-</span></p>
 <p class="p1"><span class="s1">remark,<span class="Apple-converted-space">  </span>all examples and documentation are in the directory</span></p>
 <p class="p2"><span class="s2"><span class="Apple-converted-space"> </span><a href="file:///usr/local/share/freefem++"><span class="s3">file:///usr/local/share/freefem++</span></a></span></p>
 <p class="p1"><span class="s1">the library install file are in<span class="Apple-converted-space"> </span></span></p>
@@ -77,11 +79,94 @@ nloaded from > Anywhere)<A>
 </span></p>
 <p class="p1"><span class="s1"><br>
 </span></p>
-<p class="p1"><span class="s1"><b>To compile a full version of FreeFem++ under MacOS<span class="Apple-converted-space"> </span></b></span></p>
+<p class="p1"><span class="s1"><b>(In progress (May 2018 FH) ) To compile a full version of FreeFem++ under MacOS<span class="Apple-converted-space"> </span></b></span></p>
 <p class="p1"><span class="s1">---------------------------------------------------------------------------</span></p>
 <p class="p1"><span class="s1">First download the tar.gz file contening source from<span class="Apple-converted-space"> </span></span></p>
-<p class="p1"><span class="s1">or you can get the latest source from an anonymous <a href="http://mercurial.selenic.com/"><span class="s3">Mercurial SCM </span></a>copy with the following unix shell commands :</span></p>
-<p class="p6"><span class="s1"># hg clone http://www.freefem.org/ff++/ff++<span class="Apple-converted-space"> </span></span></p>
+<p class="p1"><span class="s1">or you can get the latest source from an anonymous <A href="https://git-scm.com/downloads" > git SCM </A> copy with the following unix shell commands :</span></p>
+<p class="p6"><span class="s1"># git clone https://github.com/FreeFem/FreeFem-sources ff++<span class="Apple-converted-space"> </span></span></p>
+<p class="p1"><span class="s1"><b>Under Hight Sierra or better<span class="Apple-converted-space">  </span>(10.13)<span class="Apple-converted-space">  </span>2015 ( xcode 9.3 <span class="Apple-converted-space">  </span>form scratch)</b></span></p>
+<p class="p1"><span class="s1"><b>-------------------------------------------------</b></span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>remark: All line beginning by # are<span class="Apple-converted-space">  </span>shell command in terminal.<span class="Apple-converted-space"> </span></span></p>
+<p class="p4"><span class="s1"><span class="Apple-converted-space"> </span></span></p>
+<p class="p1"><span class="s1"><br>
+</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>1) install xcode  ,<span class="Apple-converted-space"> </span></span></p>
+<p class="p4"><span class="s1"></span><br></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>2) to install<span class="Apple-converted-space">  </span>the xcode command line tools <span class="Apple-converted-space"> </span></span></p>
+<p class="p4"><span class="s1"></span><br></p>
+<p class="p5"><span class="s1">#<span class="Apple-converted-space">  </span>gcc<span class="Apple-converted-space"> </span></span></p>
+<p class="p1"><span class="s1">answer to the dialogue<span class="Apple-converted-space">  </span>box</span></p>
+<p class="p1"><span class="s1">to check after</span></p>
+<p class="p7"><span class="s1"># clang++ -v</span></p>
+<p class="p8"><span class="s1">Apple LLVM version 9.1.0 (clang-902.0.39.1)</span></p>
+<p class="p8"><span class="s1">Target: x86_64-apple-darwin17.5.0</span></p>
+<p class="p8"><span class="s1">Thread model: posix</span></p>
+<p class="p8"><span class="s1">InstalledDir: /Applications/Xcode.app/Contents/Developer/Toolchains/XcodeDefault.xctoolchain/usr/bin</span></p>
+<p class="p1"><span class="s1"><br>
+<p class="p1"><span class="s1"><span class="Apple-converted-space">  </span>3) install<span class="Apple-converted-space">  </span>ifort or gfortran with brew installer</p>
+<p class="p1"><span class="s1"><br>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>4) install </span><span class="s6">mpich or openmpi<span class="Apple-converted-space"> </span></span><span class="s1"> form the source<span class="Apple-converted-space"> </span></span></p>
+<p class="p4"><span class="s1"><span class="Apple-converted-space"> </span></span></p>
+<p class="p1"><span class="s1">get :</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space">  </span>http://www.mpich.org/static/downloads/3.1.3/mpich-3.2..tar.gz</span></p>
+<p class="p1"><span class="s1">or<span class="Apple-converted-space"> </span></span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space">  </span>https://www.open-mpi.org/software/ompi/v3.0/downloads/openmpi-3.0.1.tar.gz</span></p>
+<p class="p1"><span class="s1">un tar and:<span class="Apple-converted-space"> </span></span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span>#<span class="Apple-converted-space">  </span>./configure 'F77=gfortran' 'FC=gfortran' 'CC=clang' 'CXX=clang++'</span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># make<span class="Apple-converted-space"> </span></span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># sudo make install</span></p>
+<p class="p1"><span class="s1">remark: you can no use precompile version because the fortran in not correct.<br>
+</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>5) install gsl<span class="Apple-converted-space"> </span></span></p>
+<p class="p9"><span class="s2"><span class="Apple-converted-space">   </span>use homebrew (</span><span class="s1">http://brew.sh ) or do:</span></p>
+<p class="p4"><span class="s1"></span><br></p>
+<p class="p5"><span class="s7"><span class="Apple-converted-space"> </span></span><span class="s1"> # curl -O http://git.savannah.gnu.org/cgit/gsl.git/snapshot/gsl-release-2-4.tar.gz</span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># tar zxvf gsl-release-2-4.tar.gz</span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># cd gsl-release-2-4.tar.gz</span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span>#./configure CC=clang</span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># make</span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># sudo make install<span class="Apple-converted-space"> </span></span></p>
+<p class="p5"><span class="s1"><br>
+</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>6) Install git from the web <A href="https://git-scm.com/downloads" > https://git-scm.com/downloads </A>/span></p>
+<p class="p4"><span class="s1"><span class="Apple-converted-space">    </span></span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>7)<span class="Apple-converted-space">  </span>Download freefem++ <span class="Apple-converted-space"> </span></span></p>
+<p class="p4"><span class="s1"></span><br></p>
+<p class="p10"><span class="s1"><span class="Apple-converted-space">  </span># git clone https://github.com/FreeFem/FreeFem-sources ff++</span></p>
+<p class="p11"><span class="s1"></span><br></p>
+<p class="p12"><span class="s1"><span class="Apple-converted-space">  </span>9) install autoconf and automake now not in xcode</span></p>
+<p class="p13"><span class="s1"></span><br></p>
+<p class="p12"><span class="s1"><span class="Apple-converted-space">   </span>I use the macport <span class="Apple-converted-space">  </span>distribution form http://www.macports.org</span></p>
+<p class="p12"><span class="s1"><span class="Apple-converted-space">    </span>or the home brew distribution from http://brew.sh</span></p>
+<p class="p10"><span class="s7"><span class="Apple-converted-space"> </span></span><span class="s1"><span class="Apple-converted-space">  </span># sudo port install<span class="Apple-converted-space">  </span>autoconf</span></p>
+<p class="p10"><span class="s1"><span class="Apple-converted-space">   </span># sudo port install<span class="Apple-converted-space">  </span>automake</span></p>
+<p class="p12"><span class="s1"><span class="Apple-converted-space">  </span>and to build<span class="Apple-converted-space">  </span>the file<span class="Apple-converted-space">  </span></span><span class="s8">./configure </span><span class="s9">with automake/autoconf<span class="Apple-converted-space">  </span>do<span class="Apple-converted-space"> </span></span></p>
+<p class="p14"><span class="s5"><span class="Apple-converted-space">  </span></span><span class="s1"># cd ff++</span></p>
+<p class="p5"><span class="s7"><span class="Apple-converted-space"> </span></span><span class="s1"># autoreconf <span class="Apple-converted-space"> </span></span></p>
+<p class="p15"><span class="s1"></span><br></p>
+<p class="p12"><span class="s1"><span class="Apple-converted-space">  </span>9bis) Or<span class="Apple-converted-space">  </span>untar AutoGeneratedFile.tar.gz</span></p>
+<p class="p13"><span class="s1"></span><br></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space"> </span># cd ff++</span></p>
+<p class="p5"><span class="s10"><span class="Apple-converted-space">  </span></span><span class="s1"># </span><span class="s11">tar zxvf AutoGeneratedFile.tar.gz<span class="Apple-converted-space"> </span></span></p>
+<p class="p10"><span class="s1"><br>
+</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>10)<span class="Apple-converted-space">  </span>compilation of freefem++<span class="Apple-converted-space"> </span></span></p>
+<p class="p4"><span class="s1"></span><br></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># cd ff++</span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span>#<span class="Apple-converted-space">  </span>./configure -—enable-download<span class="Apple-converted-space"> </span></span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># make</span></p>
+<p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># sudo make install</span></p>
+<p class="p1"><span class="s1"><br>
+</span></p>
+<p class="p1"><span class="s1"><br>
+
+<p class="p1"><span class="s1"><br>
+</span></p>
+<p class="p1"><span class="s1"><b>(Out of Date ) To compile a full version of FreeFem++ under MacOS<span class="Apple-converted-space"> </span></b></span></p>
+<p class="p1"><span class="s1">---------------------------------------------------------------------------</span></p>
+<p class="p1"><span class="s1">First download the tar.gz file contening source from<span class="Apple-converted-space"> </span></span></p>
+<p class="p1"><span class="s1">or you can get the latest source from an anonymous <A href="https://git-scm.com/downloads" > git SCM </A> copy with the following unix shell commands :</span></p>
+<p class="p6"><span class="s1"># git clone https://github.com/FreeFem/FreeFem-sources ff++<span class="Apple-converted-space"> </span></span></p>
 <p class="p1"><span class="s1"><b>Under Mavericks or better<span class="Apple-converted-space">  </span>(10.9,10.10)<span class="Apple-converted-space">  </span>2015 ( xcode 6.1.1<span class="Apple-converted-space">  </span>form scratch)</b></span></p>
 <p class="p1"><span class="s1"><b>-------------------------------------------------</b></span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>remark: All line beginning by # are<span class="Apple-converted-space">  </span>shell command in terminal.<span class="Apple-converted-space"> </span></span></p>
@@ -136,11 +221,11 @@ nloaded from > Anywhere)<A>
 <p class="p5"><span class="s1"><span class="Apple-converted-space">  </span># sudo make install<span class="Apple-converted-space"> </span></span></p>
 <p class="p5"><span class="s1"><br>
 </span></p>
-<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>6) Install mercurial from the web http://mercurial.selenic.com</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>6) Install git from the web <A href="https://git-scm.com/downloads" > https://git-scm.com/downloads </A>/span></p>
 <p class="p4"><span class="s1"><span class="Apple-converted-space">    </span></span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>7)<span class="Apple-converted-space">  </span>Download freefem++ <span class="Apple-converted-space"> </span></span></p>
 <p class="p4"><span class="s1"></span><br></p>
-<p class="p10"><span class="s1"><span class="Apple-converted-space">  </span># hg clone<span class="Apple-converted-space">  </span>http://www.freefem.org/ff++/ff++</span></p>
+<p class="p10"><span class="s1"><span class="Apple-converted-space">  </span># git clone https://github.com/FreeFem/FreeFem-sources ff++</span></p>
 <p class="p11"><span class="s1"></span><br></p>
 <p class="p12"><span class="s1"><span class="Apple-converted-space">  </span>9) install autoconf and automake now not in xcode</span></p>
 <p class="p13"><span class="s1"></span><br></p>
@@ -201,10 +286,10 @@ nloaded from > Anywhere)<A>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span>#./configure CC=/usr/local/bin/gcc</span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># make</span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># sudo make install<span class="Apple-converted-space"> </span></span></p>
-<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>7) install mecurial from the web http://mercurial.selenic.com</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>7) install git from the web <A href="https://git-scm.com/downloads" > https://git-scm.com/downloads </A></span></p>
 <p class="p4"><span class="s1"><span class="Apple-converted-space">    </span></span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>8)<span class="Apple-converted-space">  </span>downlaod <span class="Apple-converted-space"> </span></span></p>
-<p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># hg clone<span class="Apple-converted-space">  </span>http://www.freefem.org/ff++/ff++</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># git clone https://github.com/FreeFem/FreeFem-sources ff++</span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>9)<span class="Apple-converted-space">  </span>compilation of freefem++<span class="Apple-converted-space"> </span></span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># cd ff++</span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># ./configure '-with-suffix=macos-10.8' '-without-fltk' '--enable-download' '--enable-optim' 'MPIRUN=/usr/local/bin/mpirun' '--enable-m64' '--without-x' 'CC=clang -isysroot /Applications/Xcode.app//Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.8.sdk' 'CFLAGS=-mmacosx-version-min=10.8' 'CXXFLAGS=-mmacosx-version-min=10.8 -std=c++11' 'CXX=clang++ -isysroot /Applications/Xcode.app//Contents/Developer/Platforms/MacOSX.platform/Developer/SDKs/MacOSX10.8.sdk' 'F77=/usr/local/bin/gfortran' 'FC=/usr/local/bin/gfortran' 'MPICXX=/usr/local/bin/mpic++' 'MPICC=/usr/local/bin/mpicc' 'MPIFC=/usr/local/bin/mpif90' 'MPIF77=/usr/local/bin/mpif90' '--enable-maintainer-mode'</span></p>
@@ -248,10 +333,10 @@ nloaded from > Anywhere)<A>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span>#./configure CC=/usr/local/bin/gcc</span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># make</span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># sudo make install<span class="Apple-converted-space"> </span></span></p>
-<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>7) install mecurial from the web http://mercurial.selenic.com</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>7) install git from the web <A href="https://git-scm.com/downloads" > git SCM </A></span></p>
 <p class="p4"><span class="s1"><span class="Apple-converted-space">    </span></span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>8)<span class="Apple-converted-space">  </span>download <span class="Apple-converted-space"> </span></span></p>
-<p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># hg clone<span class="Apple-converted-space">  </span>http://www.freefem.org/ff++/ff++</span></p>
+<p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># git clone https://github.com/FreeFem/FreeFem-sources ff++</span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space"> </span>9)<span class="Apple-converted-space">  </span>compilation of freefem++<span class="Apple-converted-space"> </span></span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># cd ff++</span></p>
 <p class="p1"><span class="s1"><span class="Apple-converted-space">  </span># ./configure '-with-suffix=lion' '-without-fltk' '--enable-download' '--enable-optim' 'MPIRUN=/usr/local/bin/mpirun' '--enable-m64' '--without-x' 'CC=/usr/local/bin/gcc' 'CXX=/usr/local/bin/g++' 'F77=/usr/local/bin/gfortran' 'FC=/usr/local/bin/gfortran' <span class="Apple-converted-space"> </span></span></p>
